"""
If any part of this module is used for a publication please cite:

R. Tom, T. Rose, I. Bier, H. OBrien, A. Vazquez-Mayagoitia,
and N. Marom "Genarris 2.0: A Random Structure Generator for Molecular Crystals 
arXiv 1909.10629 (2019)
"""


import json
from ase.calculators.aims import Aims
from ibslib.calculators import Slurm,MBDBatchCalc, \
                               tin_arguments,mbd_settings

__author__ = "Rithwik Tom, Timothy Rose, Immanuel Bier, Harriet OBrien," + \
                "Xiayue Li, Farren Curtis, Alvaro Vazquez-Mayagoita"
__copyright__ = "Copyright 2019, Carnegie Mellon University"
__credits__ = ["Rithwik Tom", "Timothy Rose", "Immanuel Bier", "Harriet OBrien",\
               "Xiayue Li", "Farren Curtis", "Alvaro Vazquez-Mayagoita", "Noa Marom"]
__license__ = "BSD-3"
__version__ = "2.0_190923"
__maintainer__ = "Noa Marom"
__email__ = "nmarom@andrew.cmu.edu"
__url__ = "http://www.noamarom.com"






# Directory with structure files to calculate
struct_dir = "struct_dir"
# Directory where calculations will take place
calc_dir = "batch_calc"
# Setup command for calculations
tin_arguments["command"] = "mpirun -np 12 /home/ibier/Software/MBD/DFT_MBD_AT_rsSCS.x geometry.xyz setting.in > mbd.out" 
slurm = Slurm(tin_arguments)

calc = MBDBatchCalc(struct_dir, settings=mbd_settings, Slurm=slurm, calc_dir=calc_dir)
calc.calc(overwrite=True)
