"""
If any part of this module is used for a publication please cite:

R. Tom, T. Rose, I. Bier, H. OBrien, A. Vazquez-Mayagoitia,
and N. Marom "Genarris 2.0: A Random Structure Generator for Molecular Crystals 
arXiv 1909.10629 (2019)
"""


from ibslib.io import read,write
from ibslib.molecules import UniqueMolecules

__author__ = "Rithwik Tom, Timothy Rose, Immanuel Bier, Harriet OBrien," + \
                "Xiayue Li, Farren Curtis, Alvaro Vazquez-Mayagoita"
__copyright__ = "Copyright 2019, Carnegie Mellon University"
__credits__ = ["Rithwik Tom", "Timothy Rose", "Immanuel Bier", "Harriet OBrien",\
               "Xiayue Li", "Farren Curtis", "Alvaro Vazquez-Mayagoita", "Noa Marom"]
__license__ = "BSD-3"
__version__ = "2.0_190923"
__maintainer__ = "Noa Marom"
__email__ = "nmarom@andrew.cmu.edu"
__url__ = "http://www.noamarom.com"







# Define structure file to use
struct_file = "TETCEN.cif"
# Can change output format to "cif"
output_format = "geo"
# folder for molecule outputs
molecule_folder = "molecules"
# folder for whole molecule reconstructed structure files (rstruct)
rstruct_folder = "rstruct"
# overwrite controls if structure files can be overwritten 
overwrite = True
# Read in to Structure object
struct = read(struct_file)

# Identify unique molecules and construct whole molecules
um = UniqueMolecules(struct)
# Write single molecule found in structure
um.write_unique(molecule_folder, file_format=output_format,
                overwrite=overwrite)
# Write structure with whole molecules 
um.write_rstruct(rstruct_folder, file_format=output_format,
                 overwrite=overwrite)
