"""
If any part of this module is used for a publication please cite:

R. Tom, T. Rose, I. Bier, H. OBrien, A. Vazquez-Mayagoitia,
and N. Marom "Genarris 2.0: A Random Structure Generator for Molecular Crystals 
arXiv 1909.10629 (2019)
"""


import json
from ase.calculators.aims import Aims
from ibslib.calculators import Slurm,AimsBatchCalc, \
                               tin_arguments,tier1_SPE_settings

__author__ = "Rithwik Tom, Timothy Rose, Immanuel Bier, Harriet OBrien," + \
                "Xiayue Li, Farren Curtis, Alvaro Vazquez-Mayagoita"
__copyright__ = "Copyright 2019, Carnegie Mellon University"
__credits__ = ["Rithwik Tom", "Timothy Rose", "Immanuel Bier", "Harriet OBrien",\
               "Xiayue Li", "Farren Curtis", "Alvaro Vazquez-Mayagoita", "Noa Marom"]
__license__ = "BSD-3"
__version__ = "2.0_190923"
__maintainer__ = "Noa Marom"
__email__ = "nmarom@andrew.cmu.edu"
__url__ = "http://www.noamarom.com"






# Directory with structure files to calculate
struct_dir = "struct_dir"
# Directory where calculations will take place
calc_dir = "batch_calc"
# Initialize settings for Slurm and Aims
aims = Aims(**tier1_SPE_settings)
# Setup command for calculations
#tin_arguments["pre-command"] = "module load intel"
tin_arguments["command"] = "mpirun -np 12 /opt/marom/bin/aims.171221_1.scalapack.mpi.x > aims.out"
slurm = Slurm(tin_arguments)

calc = AimsBatchCalc(struct_dir, Aims=aims, Slurm=slurm, calc_dir=calc_dir)
calc.calc(overwrite=True)
