"""
If any part of this module is used for a publication please cite:

R. Tom, T. Rose, I. Bier, H. OBrien, A. Vazquez-Mayagoitia,
and N. Marom "Genarris 2.0: A Random Structure Generator for Molecular Crystals 
arXiv 1909.10629 (2019)
"""

from ibslib.descriptor.distance_matrix import calc_euclidean_dist_vectorized_squared
from sklearn.cluster import AffinityPropagation,KMeans






__author__ = "Rithwik Tom, Timothy Rose, Immanuel Bier, Harriet OBrien," + \
                "Xiayue Li, Farren Curtis, Alvaro Vazquez-Mayagoita"
__copyright__ = "Copyright 2019, Carnegie Mellon University"
__credits__ = ["Rithwik Tom", "Timothy Rose", "Immanuel Bier", "Harriet OBrien",\
               "Xiayue Li", "Farren Curtis", "Alvaro Vazquez-Mayagoita", "Noa Marom"]
__license__ = "BSD-3"
__version__ = "2.0_190923"
__maintainer__ = "Noa Marom"
__email__ = "nmarom@andrew.cmu.edu"
__url__ = "http://www.noamarom.com"

def run_ap_dist_mat(dist_matrix,**kwargs):
    '''
    Purpose:
        Runs affinity propagation from the distance matrix. Includes all 
          standard arguments from sklearn affinity propagation.
    Returns:
        Cluster labels
    '''
    ap_obj = AffinityPropagation(**kwargs)    
    cluster_labels = ap_obj.fit_predict(dist_matrix)
    
    return cluster_labels 

def run_ap(feature_matrix,**kwargs):
    '''
    Purpose: Will call run_ap after computing the distance matrix for you
    '''
    dist_matrix = calc_euclidean_dist_vectorized_squared(feature_matrix)
    return run_ap_dist_mat(dist_matrix,**kwargs)

def run_KMeans(feature_matrix,**kwargs):
    dist_matrix = calc_euclidean_dist_vectorized_squared(feature_matrix)
    return run_KMeans_dist_mat(dist_matrix,**kwargs)

def run_KMeans_dist_mat(dist_matrix,**kwargs):
    k_model = KMeans(**kwargs)
    k_model.fit(dist_matrix)
    return k_model