"""
If any part of this module is used for a publication please cite:

R. Tom, T. Rose, I. Bier, H. OBrien, A. Vazquez-Mayagoitia,
and N. Marom "Genarris 2.0: A Random Structure Generator for Molecular Crystals 
arXiv 1909.10629 (2019)
"""

__author__ = "Rithwik Tom, Timothy Rose, Immanuel Bier, Harriet OBrien," + \
                "Xiayue Li, Farren Curtis, Alvaro Vazquez-Mayagoita"
__copyright__ = "Copyright 2019, Carnegie Mellon University"
__credits__ = ["Rithwik Tom", "Timothy Rose", "Immanuel Bier", "Harriet OBrien",\
               "Xiayue Li", "Farren Curtis", "Alvaro Vazquez-Mayagoita", "Noa Marom"]
__license__ = "BSD-3"
__version__ = "2.0_190923"
__maintainer__ = "Noa Marom"
__email__ = "nmarom@andrew.cmu.edu"
__url__ = "http://www.noamarom.com"





spglib_database = {
    "1": {
        "Coordinates": [
            [
                "(x,y,z)"
            ]
        ],
        "Multiplicity": [
            "1"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1  ",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "2": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,-z)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P -1  ",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "3": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)"
            ],
            [
                "(1/2,y,1/2)"
            ],
            [
                "(1/2,y,0)"
            ],
            [
                "(0,y,1/2)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "2",
            "2"
        ],
        "Space Group": "P 1 2 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "4": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)"
            ],
            [
                "(1/2,1/2,z)"
            ],
            [
                "(0,1/2,z)"
            ],
            [
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "2",
            "2"
        ],
        "Space Group": "P 1 1 2",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "5": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)"
            ],
            [
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,1/2)"
            ],
            [
                "(x,1/2,0)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "2",
            "2"
        ],
        "Space Group": "P 2 1 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "6": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1 2sub1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "7": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1 1 2sub1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "8": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 2sub1 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "9": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)"
            ],
            [
                "(0,y,1/2)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "C 1 2 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "10": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)"
            ],
            [
                "(1/2,y,0)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "A 1 2 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "11": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)"
            ],
            [
                "(1/2,y,0)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "I 1 2 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "12": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)"
            ],
            [
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "A 1 1 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "13": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)"
            ],
            [
                "(0,1/2,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "B 1 1 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "14": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)"
            ],
            [
                "(0,1/2,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "I 1 1 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "15": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)"
            ],
            [
                "(x,1/2,0)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "B 2 1 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "16": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)"
            ],
            [
                "(x,0,1/2)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "C 2 1 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "17": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)"
            ],
            [
                "(x,0,1/2)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2"
        ],
        "Space Group": "I 2 1 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "18": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,z)"
            ],
            [
                "(x,1/2,z)"
            ],
            [
                "(x,0,z)"
            ]
        ],
        "Multiplicity": [
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "m"
        ],
        "Space Group": "P 1 m 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "19": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,1/2)"
            ],
            [
                "(x,y,0)"
            ]
        ],
        "Multiplicity": [
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "m"
        ],
        "Space Group": "P 1 1 m",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "20": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,z)"
            ],
            [
                "(1/2,y,z)"
            ],
            [
                "(0,y,z)"
            ]
        ],
        "Multiplicity": [
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "m"
        ],
        "Space Group": "P m 1 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "21": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1 c 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "22": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1 n 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "23": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,z)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1 a 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "24": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,y,-z)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1 1 a",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "25": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1 1 n",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "26": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 1 1 b",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "27": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,z)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P b 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "28": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P n 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "29": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "2"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P c 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "30": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "C 1 m 1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "31": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "A 1 m 1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "32": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "I 1 m 1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "33": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "A 1 1 m",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "34": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "B 1 1 m",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "35": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "I 1 1 m",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "36": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "B m 1 1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "37": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "C m 1 1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "38": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m"
        ],
        "Space Group": "I m 1 1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "39": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "C 1 c 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "40": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "A 1 n 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "41": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "I 1 a 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "42": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "A 1 a 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "43": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "C 1 n 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "44": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "I 1 c 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "45": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,y,-z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "A 1 1 a",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "46": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "B 1 1 n",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "47": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "I 1 1 b",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "48": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "B 1 1 b",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "49": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "A 1 1 n",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "50": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,y,-z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "I 1 1 a",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "51": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "B b 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "52": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "C n 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "53": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "I c 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "54": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "C c 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "55": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "B n 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "56": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "I b 1 1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "57": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,-z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)"
            ],
            [
                "(1/2,y,1/2)",
                "(1/2,-y,1/2)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "m",
            "2",
            "2",
            "2",
            "2",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "P 1 2/m 1",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "58": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "m",
            "2",
            "2",
            "2",
            "2",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "P 1 1 2/m",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "59": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "m",
            "2",
            "2",
            "2",
            "2",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "P 2/m 1 1",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "60": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,3/4,-z)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 2sub1/m 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "61": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-x,-y,3/4)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 1 2sub1/m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "62": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,-y,-z)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/m 1 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "63": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "C 1 2/m 1",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "64": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(1/2,1/4,3/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,1/4,3/4)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "A 1 2/m 1",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "65": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "I 1 2/m 1",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "66": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "A 1 1 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "67": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(3/4,1/2,1/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,0,1/4)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "B 1 1 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "68": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "I 1 1 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "69": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "B 2/m 1 1",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "70": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,3/4,0)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "C 2/m 1 1",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "71": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/4,3/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m",
            "2",
            "2",
            "-1",
            "-1",
            "2/m",
            "2/m",
            "2/m",
            "2/m"
        ],
        "Space Group": "I 2/m 1 1",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "72": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)"
            ],
            [
                "(1/2,y,1/4)",
                "(1/2,-y,3/4)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 2/c 1",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "73": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z+1/2)"
            ],
            [
                "(3/4,y,1/4)",
                "(1/4,-y,3/4)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 2/n 1",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "74": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)"
            ],
            [
                "(3/4,y,1/2)",
                "(1/4,-y,1/2)"
            ],
            [
                "(3/4,y,0)",
                "(1/4,-y,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 2/a 1",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "75": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)"
            ],
            [
                "(1/4,1/2,z)",
                "(3/4,1/2,-z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 1 2/a",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "76": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)"
            ],
            [
                "(1/4,3/4,z)",
                "(3/4,1/4,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 1 2/n",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "77": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(1/2,3/4,z)",
                "(1/2,1/4,-z)"
            ],
            [
                "(0,3/4,z)",
                "(0,1/4,-z)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 1 2/b",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "78": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x,3/4,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/b 1 1",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "79": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z+1/2)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x,3/4,1/4)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,3/4)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/n 1 1",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "80": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)"
            ],
            [
                "(x,1/2,3/4)",
                "(-x,1/2,1/4)"
            ],
            [
                "(x,0,3/4)",
                "(-x,0,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/c 1 1",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "81": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 2sub1/c 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "82": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y+1/2,z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 2sub1/n 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "83": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 2sub1/a 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "84": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 1 2sub1/a",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "85": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z+1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 1 2sub1/n",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "86": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z+1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 1 1 2sub1/b",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "87": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/b 1 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "88": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x+1/2,y+1/2,z+1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/n 1 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "89": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z+1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/c 1 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "90": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "C 1 2/c 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "91": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z+1/2)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y,3/4)"
            ],
            [
                "(1/2,3/4,3/4)",
                "(0,3/4,3/4)"
            ],
            [
                "(0,3/4,1/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "A 1 2/n 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "92": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y,0)"
            ],
            [
                "(3/4,3/4,1/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(1/4,3/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "I 1 2/a 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "93": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)"
            ],
            [
                "(3/4,y,0)",
                "(1/4,-y,0)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(0,1/4,3/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,1/4,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "A 1 2/a 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "94": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z+1/2)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y,3/4)"
            ],
            [
                "(3/4,1/4,1/2)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,1/4,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "C 1 2/n 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "95": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "I 1 2/c 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "96": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "A 1 1 2/a",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "97": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z)"
            ],
            [
                "(3/4,1/2,3/4)",
                "(3/4,0,3/4)"
            ],
            [
                "(1/4,0,3/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "B 1 1 2/n",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "98": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z)"
            ],
            [
                "(1/4,3/4,3/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "I 1 1 2/b",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "99": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(0,3/4,z)",
                "(0,1/4,-z)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(3/4,0,1/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,1/2,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "B 1 1 2/b",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "100": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z)"
            ],
            [
                "(1/2,3/4,1/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,1/4,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "A 1 1 2/n",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "101": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "I 1 1 2/a",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "102": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "B 2/b 1 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "103": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z+1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,3/4)"
            ],
            [
                "(3/4,3/4,1/2)",
                "(3/4,3/4,0)"
            ],
            [
                "(3/4,1/4,0)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "C 2/n 1 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "104": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,3/4)"
            ],
            [
                "(3/4,1/4,3/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "I 2/c 1 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "105": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)"
            ],
            [
                "(x,0,3/4)",
                "(-x,0,1/4)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(1/4,3/4,0)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "C 2/c 1 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "106": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z+1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,3/4)"
            ],
            [
                "(1/4,1/2,3/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,1/2,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "B 2/n 1 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "107": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(1/4,3/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2",
            "-1",
            "-1",
            "-1",
            "-1"
        ],
        "Space Group": "I 2/b 1 1",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "108": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,y,1/2)",
                "(1/2,-y,1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2",
            "..2",
            ".2.",
            ".2.",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 2 2 2",
        "Wyckoff Letter": [
            "u",
            "t",
            "s",
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "109": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z)"
            ],
            [
                "(1/2,y,1/4)",
                "(1/2,-y,3/4)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2.",
            "2..",
            "2.."
        ],
        "Space Group": "P 2 2 2sub1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "110": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(-x+1/2,-y,z)",
                "(-x,y,-z)"
            ],
            [
                "(1/4,1/2,z)",
                "(3/4,1/2,-z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(0,y,1/2)",
                "(1/2,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2.",
            "2..",
            "2.."
        ],
        "Space Group": "P 2sub1 2 2",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "111": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(x,-y+1/2,-z)",
                "(-x,-y,z)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x,3/4,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2.",
            "2..",
            "2.."
        ],
        "Space Group": "P 2 2sub1 2",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "112": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 2sub1 2sub1 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "113": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,y+1/2,-z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 2 2sub1 2sub1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "114": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,-y,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 2sub1 2 2sub1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "115": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 2sub1 2sub1 2sub1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "116": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(x,0,0)",
                "(-x,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "2.."
        ],
        "Space Group": "C 2 2 2sub1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "117": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(-x+1/2,-y,z)",
                "(-x,y,-z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "2.."
        ],
        "Space Group": "A 2sub1 2 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "118": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(x,-y+1/2,-z)",
                "(-x,-y,z)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "2.."
        ],
        "Space Group": "B 2 2sub1 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "119": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "C 2 2 2",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "120": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y,z)",
                "(-x,y,-z)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "A 2 2 2",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "121": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x,-y,z)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y,3/4)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "B 2 2 2",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "122": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y,1/4)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/4,1/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            ".2.",
            "..2",
            "..2",
            ".2.",
            "2..",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "F 2 2 2",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "123": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "I 2 2 2",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "124": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z+1/2)"
            ],
            [
                "(1/4,y,0)",
                "(1/4,-y,1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            "2.."
        ],
        "Space Group": "I 2sub1 2sub1 2sub1",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "125": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)"
            ],
            [
                "(1/2,1/2,z)"
            ],
            [
                "(1/2,0,z)"
            ],
            [
                "(0,1/2,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            ".m.",
            "mm2",
            "mm2",
            "mm2",
            "mm2"
        ],
        "Space Group": "P m m 2",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "126": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,1/2,z)",
                "(x,1/2,-z)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z)"
            ],
            [
                "(x,y,1/2)",
                "(x,-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,1/2,1/2)"
            ],
            [
                "(x,1/2,0)"
            ],
            [
                "(x,0,1/2)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            ".m.",
            "mm2",
            "mm2",
            "mm2",
            "mm2"
        ],
        "Space Group": "P 2 m m",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "127": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,y,0)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,y,-z)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z)"
            ],
            [
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,1/2)"
            ],
            [
                "(1/2,y,0)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            ".m.",
            "mm2",
            "mm2",
            "mm2",
            "mm2"
        ],
        "Space Group": "P m 2 m",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "128": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(x,-y,z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,z+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m.."
        ],
        "Space Group": "P m c 2sub1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "129": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,z+1/2)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m.."
        ],
        "Space Group": "P c m 2sub1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "130": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(x+1/2,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,1/2,z)",
                "(x+1/2,1/2,-z)"
            ],
            [
                "(x,0,z)",
                "(x+1/2,0,-z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m.."
        ],
        "Space Group": "P 2sub1 m a",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "131": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(x+1/2,-y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,1/2)",
                "(x+1/2,-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(x+1/2,-y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m.."
        ],
        "Space Group": "P 2sub1 a m",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "132": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(-x,y+1/2,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,y+1/2,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,y+1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m.."
        ],
        "Space Group": "P b 2sub1 m",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "133": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(x,y+1/2,-z)",
                "(-x,y,z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,y+1/2,-z)"
            ],
            [
                "(0,y,z)",
                "(0,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m.."
        ],
        "Space Group": "P m 2sub1 b",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "134": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2",
            "..2"
        ],
        "Space Group": "P c c 2",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "135": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x+1/2,y,-z)",
                "(x+1/2,-y,z)"
            ],
            [
                "(x,1/2,1/2)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(x,1/2,0)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(x,0,1/2)",
                "(x+1/2,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(x+1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2",
            "..2"
        ],
        "Space Group": "P 2 a a",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "136": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y+1/2,z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(1/2,y,1/2)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(0,y,1/2)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,y+1/2,0)"
            ],
            [
                "(0,y,0)",
                "(0,y+1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2",
            "..2"
        ],
        "Space Group": "P b 2 b",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "137": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x+1/2,-y,z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,-y,z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "..2"
        ],
        "Space Group": "P m a 2",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "138": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y+1/2,z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,3/4,z)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "..2"
        ],
        "Space Group": "P b m 2",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "139": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y+1/2,-z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(x,3/4,-z)"
            ],
            [
                "(x,0,1/2)",
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(x,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "..2"
        ],
        "Space Group": "P 2 m b",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "140": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,-y,z+1/2)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(x,-y,3/4)"
            ],
            [
                "(x,1/2,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(x,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "..2"
        ],
        "Space Group": "P 2 c m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "141": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y,z+1/2)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-x,y,3/4)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,y,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "..2"
        ],
        "Space Group": "P c 2 m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "142": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,y,-z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,y,-z)"
            ],
            [
                "(0,y,1/2)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,y,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "..2"
        ],
        "Space Group": "P m 2 a",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "143": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(x+1/2,-y,z)",
                "(-x+1/2,y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P c a 2sub1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "144": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x,y+1/2,z)",
                "(x,-y+1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P b c 2sub1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "145": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(x,y+1/2,-z)",
                "(x+1/2,-y+1/2,z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 2sub1 a b",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "146": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(x,-y,z+1/2)",
                "(x+1/2,y,-z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 2sub1 c a",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "147": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(-x,y,z+1/2)",
                "(x,y+1/2,-z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P c 2sub1 b",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "148": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(x+1/2,y,-z)",
                "(-x+1/2,y+1/2,z)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P b 2sub1 a",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "149": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y+1/2,z+1/2)",
                "(-x,y+1/2,z+1/2)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P n c 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "150": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,-y,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P c n 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "151": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(x+1/2,-y,z+1/2)"
            ],
            [
                "(x,1/2,0)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(x+1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 2 n a",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "152": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x+1/2,-y+1/2,z)",
                "(x+1/2,y+1/2,-z)"
            ],
            [
                "(x,0,1/2)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(x+1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 2 a n",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "153": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x+1/2,y+1/2,z)",
                "(x+1/2,y+1/2,-z)"
            ],
            [
                "(0,y,1/2)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,y+1/2,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P b 2 n",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "154": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(-x,y+1/2,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,y+1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P n 2 b",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "155": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(1/2,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "P m n 2sub1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "156": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "P n m 2sub1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "157": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y+1/2,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(x+1/2,1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "P 2sub1 m n",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "158": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z+1/2)",
                "(x+1/2,-y,z+1/2)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(x+1/2,-y,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "P 2sub1 n m",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "159": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,y+1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "P n 2sub1 m",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "160": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,y+1/2,-z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(1/2,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "P m 2sub1 n",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "161": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P b a 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "162": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(x,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 2 c b",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "163": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(0,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,y,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P c 2 a",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "164": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P n a 2sub1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "165": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x+1/2,y+1/2,z)",
                "(x+1/2,-y+1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P b n 2sub1",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "166": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 2sub1 n b",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "167": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(x,-y+1/2,z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 2sub1 c n",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "168": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P c 2sub1 n",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "169": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(x+1/2,y,-z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P n 2sub1 a",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "170": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P n n 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "171": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(x+1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 2 n n",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "172": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,y+1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P n 2 n",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "173": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,3/4,z)"
            ],
            [
                "(0,1/2,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            "..2",
            "mm2",
            "mm2"
        ],
        "Space Group": "C m m 2",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "174": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z)"
            ],
            [
                "(x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,1/4,1/4)",
                "(x,1/4,3/4)"
            ],
            [
                "(x,0,1/2)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            "..2",
            "mm2",
            "mm2"
        ],
        "Space Group": "A 2 m m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "175": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,y,0)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,y,1/4)"
            ],
            [
                "(1/2,y,0)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            "..2",
            "mm2",
            "mm2"
        ],
        "Space Group": "B m 2 m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "176": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(x,-y,z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "C m c 2sub1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "177": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "C c m 2sub1",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "178": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(x+1/2,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(x+1/2,0,-z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "A 2sub1 m a",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "179": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(x+1/2,-y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(x+1/2,-y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "A 2sub1 a m",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "180": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(-x,y+1/2,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,y+1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "B b 2sub1 m",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "181": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(x,y+1/2,-z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,y+1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m.."
        ],
        "Space Group": "B m 2sub1 b",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "182": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,3/4,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2"
        ],
        "Space Group": "C c c 2",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "183": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x+1/2,y,-z)",
                "(x+1/2,-y,z)"
            ],
            [
                "(x,1/4,1/4)",
                "(x+1/2,1/4,3/4)"
            ],
            [
                "(x,0,1/2)",
                "(x+1/2,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(x+1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2"
        ],
        "Space Group": "A 2 a a",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "184": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y+1/2,z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,y+1/2,1/4)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,y+1/2,0)"
            ],
            [
                "(0,y,0)",
                "(0,y+1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "..2"
        ],
        "Space Group": "B b 2 b",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "185": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)"
            ],
            [
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "A m m 2",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "186": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,z)",
                "(x,-y,z)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)"
            ],
            [
                "(0,1/2,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "B m m 2",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "187": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,1/2,z)",
                "(x,1/2,-z)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z)"
            ],
            [
                "(x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,1/2,0)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "B 2 m m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "188": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,-y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,1/2)",
                "(x,-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z)"
            ],
            [
                "(x,0,1/2)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "C 2 m m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "189": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,y,0)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z)"
            ],
            [
                "(0,y,1/2)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "C m 2 m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "190": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x,y,-z)",
                "(-x,y,z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,y,-z)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,y,0)"
            ],
            [
                "(1/2,y,0)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "A m 2 m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "191": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y+1/2,z)",
                "(-x,y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,3/4,z)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "..2"
        ],
        "Space Group": "A b m 2",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "192": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x+1/2,y,z)",
                "(x+1/2,-y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,-y,z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "..2"
        ],
        "Space Group": "B m a 2",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "193": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y,-z+1/2)",
                "(x,-y,z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(x,-y,3/4)"
            ],
            [
                "(x,1/2,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(x,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "..2"
        ],
        "Space Group": "B 2 c m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "194": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,-y+1/2,z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(x,1/4,z)",
                "(x,3/4,-z)"
            ],
            [
                "(x,0,1/2)",
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(x,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "..2"
        ],
        "Space Group": "C 2 m b",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "195": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x+1/2,y,z)",
                "(x+1/2,y,-z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,y,-z)"
            ],
            [
                "(0,y,1/2)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "..2"
        ],
        "Space Group": "C m 2 a",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "196": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x,y,-z+1/2)",
                "(-x,y,z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-x,y,3/4)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,y,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "..2"
        ],
        "Space Group": "A c 2 m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "197": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x+1/2,-y,z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,-y,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "A m a 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "198": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y+1/2,z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,3/4,z)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "B b m 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "199": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y+1/2,-z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(x,3/4,-z)"
            ],
            [
                "(x,0,0)",
                "(x,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "B 2 m b",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "200": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,-y,z+1/2)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(x,-y,3/4)"
            ],
            [
                "(x,0,0)",
                "(x,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "C 2 c m",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "201": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y,z+1/2)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-x,y,3/4)"
            ],
            [
                "(0,y,0)",
                "(0,y,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "C c 2 m",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "202": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,y,-z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,y,-z)"
            ],
            [
                "(0,y,0)",
                "(1/2,y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "A m 2 a",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "203": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "A b a 2",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "204": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x+1/2,y+1/2,z)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "B b a 2",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "205": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(x,0,0)",
                "(x,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "B 2 c b",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "206": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,-y+1/2,z+1/2)",
                "(x,y+1/2,-z+1/2)"
            ],
            [
                "(x,0,0)",
                "(x,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "C 2 c b",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "207": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,y,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "C c 2 a",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "208": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(-x+1/2,y,z+1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,y,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "A c 2 a",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "209": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,3/4,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            "mm2"
        ],
        "Space Group": "F m m 2",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "210": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z)"
            ],
            [
                "(x,1/4,1/4)",
                "(x,1/4,3/4)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            "mm2"
        ],
        "Space Group": "F 2 m m",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "211": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,y,0)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,y,1/4)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            "mm2"
        ],
        "Space Group": "F m 2 m",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "212": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x+1/4,-y+1/4,z+1/4)",
                "(-x+1/4,y+1/4,z+1/4)"
            ],
            [
                "(0,0,z)",
                "(1/4,1/4,z+1/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "F d d 2",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "213": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x+1/4,y+1/4,-z+1/4)",
                "(x+1/4,-y+1/4,z+1/4)"
            ],
            [
                "(x,0,0)",
                "(x+1/4,1/4,1/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "F 2 d d",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "214": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x+1/4,y+1/4,z+1/4)",
                "(x+1/4,y+1/4,-z+1/4)"
            ],
            [
                "(0,y,0)",
                "(1/4,y+1/4,1/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "F d 2 d",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "215": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)"
            ],
            [
                "(0,1/2,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "I m m 2",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "216": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z)"
            ],
            [
                "(x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,0,1/2)"
            ],
            [
                "(x,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "I 2 m m",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "217": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,y,0)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z)"
            ],
            [
                "(1/2,y,0)"
            ],
            [
                "(0,y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            "mm2",
            "mm2"
        ],
        "Space Group": "I m 2 m",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "218": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "I b a 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "219": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(x,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "I 2 c b",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "220": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(0,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,y,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "I c 2 a",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "221": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(x+1/2,-y,z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,-y,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "I m a 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "222": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y+1/2,z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,3/4,z)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "I b m 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "223": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,y+1/2,-z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(x,3/4,-z)"
            ],
            [
                "(x,0,0)",
                "(x,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "I 2 m b",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "224": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(x,-y,z+1/2)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(x,-y,3/4)"
            ],
            [
                "(x,0,0)",
                "(x,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "I 2 c m",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "225": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(-x,y,z+1/2)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-x,y,3/4)"
            ],
            [
                "(0,y,0)",
                "(0,y,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "I c 2 m",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "226": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,y,-z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,y,-z)"
            ],
            [
                "(0,y,0)",
                "(1/2,y,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2"
        ],
        "Space Group": "I m 2 a",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "227": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,-y,1/2)",
                "(-x,y,1/2)",
                "(x,-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,z)",
                "(-x,1/2,-z)",
                "(x,1/2,-z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(-x,0,-z)",
                "(x,0,-z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,z)",
                "(1/2,y,-z)",
                "(1/2,-y,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,y,1/2)",
                "(1/2,-y,1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..m",
            ".m.",
            ".m.",
            "m..",
            "m..",
            "mm2",
            "mm2",
            "mm2",
            "mm2",
            "m2m",
            "m2m",
            "m2m",
            "m2m",
            "2mm",
            "2mm",
            "2mm",
            "2mm",
            "mmm",
            "mmm",
            "mmm",
            "mmm",
            "mmm",
            "mmm",
            "mmm",
            "mmm"
        ],
        "Space Group": "P 2/m 2/m 2/m",
        "Wyckoff Letter": [
            "A",
            "z",
            "y",
            "x",
            "w",
            "v",
            "u",
            "t",
            "s",
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "228": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)",
                "(0,-y+1/2,1/2)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)",
                "(1/2,-y+1/2,1/2)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(-x+1/2,1/2,0)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(-x+1/2,1/2,1/2)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(3/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-1",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 2/n 2/n 2/n",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "229": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y+1/2,z+1/2)"
            ],
            [
                "(1/4,3/4,z)",
                "(1/4,3/4,-z+1/2)",
                "(3/4,1/4,-z)",
                "(3/4,1/4,z+1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,1/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(3/4,3/4,z+1/2)"
            ],
            [
                "(3/4,y,1/4)",
                "(3/4,-y+1/2,1/4)",
                "(1/4,-y,3/4)",
                "(1/4,y+1/2,3/4)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y+1/2,1/4)",
                "(3/4,-y,3/4)",
                "(3/4,y+1/2,3/4)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x+1/2,1/4,3/4)",
                "(-x,3/4,1/4)",
                "(x+1/2,3/4,1/4)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,1/4,1/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,3/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(0,0,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(1/4,3/4,1/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-1",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 2/n 2/n 2/n",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "230": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x,y,1/2)",
                "(x,-y,1/2)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,-z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z+1/2)",
                "(0,1/2,-z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(1/2,y,1/4)",
                "(1/2,-y,1/4)",
                "(1/2,-y,3/4)",
                "(1/2,y,3/4)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,1/4)",
                "(0,-y,3/4)",
                "(0,y,3/4)"
            ],
            [
                "(x,1/2,1/4)",
                "(-x,1/2,1/4)",
                "(-x,1/2,3/4)",
                "(x,1/2,3/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,1/4)",
                "(-x,0,3/4)",
                "(x,0,3/4)"
            ],
            [
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2/c 2/c 2/m",
        "Wyckoff Letter": [
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "231": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x+1/2,-y,z)",
                "(-x+1/2,y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z)",
                "(x+1/2,y,-z)",
                "(x+1/2,-y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)",
                "(1/2,-y,z)",
                "(1/2,y,-z)"
            ],
            [
                "(x,1/2,0)",
                "(-x+1/2,1/2,0)",
                "(-x,1/2,0)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x+1/2,0,1/2)",
                "(-x,0,1/2)",
                "(x+1/2,0,1/2)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,0,0)",
                "(-x,0,0)",
                "(x+1/2,0,0)"
            ],
            [
                "(1/4,1/2,z)",
                "(1/4,1/2,-z)",
                "(3/4,1/2,-z)",
                "(3/4,1/2,z)"
            ],
            [
                "(1/4,0,z)",
                "(1/4,0,-z)",
                "(3/4,0,-z)",
                "(3/4,0,z)"
            ],
            [
                "(1/4,y,1/2)",
                "(1/4,-y,1/2)",
                "(3/4,-y,1/2)",
                "(3/4,y,1/2)"
            ],
            [
                "(1/4,y,0)",
                "(1/4,-y,0)",
                "(3/4,-y,0)",
                "(3/4,y,0)"
            ],
            [
                "(1/4,1/2,1/2)",
                "(3/4,1/2,1/2)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)"
            ],
            [
                "(1/4,1/2,0)",
                "(3/4,1/2,0)"
            ],
            [
                "(1/4,0,0)",
                "(3/4,0,0)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2/m 2/a 2/a",
        "Wyckoff Letter": [
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "232": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x,-y+1/2,-z)",
                "(-x,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x,-y,z)",
                "(-x,y+1/2,z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)",
                "(x,1/2,-z)",
                "(-x,1/2,z)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y+1/2,1/2)",
                "(0,-y,1/2)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y+1/2,0)",
                "(1/2,-y,0)",
                "(1/2,y+1/2,0)"
            ],
            [
                "(1/2,y,1/2)",
                "(1/2,-y+1/2,1/2)",
                "(1/2,-y,1/2)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y+1/2,0)",
                "(0,-y,0)",
                "(0,y+1/2,0)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x,1/4,1/2)",
                "(-x,3/4,1/2)",
                "(x,3/4,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x,1/4,0)",
                "(-x,3/4,0)",
                "(x,3/4,0)"
            ],
            [
                "(1/2,1/4,z)",
                "(1/2,1/4,-z)",
                "(1/2,3/4,-z)",
                "(1/2,3/4,z)"
            ],
            [
                "(0,1/4,z)",
                "(0,1/4,-z)",
                "(0,3/4,-z)",
                "(0,3/4,z)"
            ],
            [
                "(1/2,1/4,1/2)",
                "(1/2,3/4,1/2)"
            ],
            [
                "(1/2,1/4,0)",
                "(1/2,3/4,0)"
            ],
            [
                "(0,1/4,1/2)",
                "(0,3/4,1/2)"
            ],
            [
                "(0,1/4,0)",
                "(0,3/4,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2/b 2/m 2/b",
        "Wyckoff Letter": [
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "233": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x+1/2,-y+1/2,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)",
                "(1/2,-y+1/2,1/2)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)",
                "(1/2,-y+1/2,0)",
                "(1/2,y+1/2,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(-x+1/2,1/2,0)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,3/4,1/2)",
                "(3/4,1/4,1/2)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)",
                "(3/4,1/4,0)",
                "(1/4,3/4,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-1",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 2/b 2/a 2/n",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "234": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x+1/2,y,-z)",
                "(x,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x+1/2,-y,z)",
                "(-x,y+1/2,z)"
            ],
            [
                "(1/4,3/4,z)",
                "(1/4,3/4,-z)",
                "(3/4,1/4,-z)",
                "(3/4,1/4,z)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,1/4,-z)",
                "(3/4,3/4,-z)",
                "(3/4,3/4,z)"
            ],
            [
                "(1/4,y,1/2)",
                "(1/4,-y+1/2,1/2)",
                "(3/4,-y,1/2)",
                "(3/4,y+1/2,1/2)"
            ],
            [
                "(1/4,y,0)",
                "(1/4,-y+1/2,0)",
                "(3/4,-y,0)",
                "(3/4,y+1/2,0)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x+1/2,1/4,1/2)",
                "(-x,3/4,1/2)",
                "(x+1/2,3/4,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,1/4,0)",
                "(-x,3/4,0)",
                "(x+1/2,3/4,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,3/4,1/2)"
            ],
            [
                "(3/4,1/4,1/2)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(3/4,1/4,0)",
                "(1/4,3/4,0)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-1",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 2/b 2/a 2/n",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "235": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(-x,-y+1/2,-z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(x,y+1/2,-z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(-x,1/2,0)",
                "(x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(-x,1/2,1/2)",
                "(x,1/2,1/2)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(0,1/2,-z+1/2)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)",
                "(1/2,-y+1/2,1/2)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)",
                "(0,-y+1/2,1/2)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(1/2,3/4,3/4)",
                "(1/2,3/4,1/4)",
                "(1/2,1/4,3/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,3/4)",
                "(0,3/4,1/4)",
                "(0,1/4,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-1",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 2/n 2/c 2/b",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "236": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y+1/2,z)",
                "(-x,y,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z+1/2)",
                "(x,y+1/2,-z)",
                "(x,-y,z+1/2)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x,1/4,3/4)",
                "(-x,3/4,1/4)",
                "(x,3/4,1/4)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,1/4,1/4)",
                "(-x,3/4,3/4)",
                "(x,3/4,3/4)"
            ],
            [
                "(1/2,1/4,z)",
                "(1/2,1/4,-z+1/2)",
                "(1/2,3/4,-z)",
                "(1/2,3/4,z+1/2)"
            ],
            [
                "(0,1/4,z)",
                "(0,1/4,-z+1/2)",
                "(0,3/4,-z)",
                "(0,3/4,z+1/2)"
            ],
            [
                "(1/2,y,1/4)",
                "(1/2,-y+1/2,1/4)",
                "(1/2,-y,3/4)",
                "(1/2,y+1/2,3/4)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y+1/2,1/4)",
                "(0,-y,3/4)",
                "(0,y+1/2,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(1/2,3/4,3/4)"
            ],
            [
                "(1/2,3/4,1/4)",
                "(1/2,1/4,3/4)"
            ],
            [
                "(0,3/4,1/4)",
                "(0,1/4,3/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-1",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 2/n 2/c 2/b",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "237": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x,-y,z)",
                "(-x+1/2,-y,-z+1/2)",
                "(x+1/2,-y,z+1/2)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)",
                "(0,-y,1/2)",
                "(0,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)",
                "(1/2,-y,1/2)",
                "(1/2,y,1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(-x+1/2,1/2,1/2)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(-x+1/2,0,1/2)",
                "(x+1/2,0,1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(3/4,1/2,3/4)",
                "(1/4,1/2,3/4)",
                "(3/4,1/2,1/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,0,3/4)",
                "(1/4,0,3/4)",
                "(3/4,0,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-1",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 2/c 2/n 2/a",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "238": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y,-z+1/2)",
                "(-x+1/2,-y,z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(x+1/2,y,-z)"
            ],
            [
                "(3/4,y,1/4)",
                "(3/4,-y,1/4)",
                "(1/4,-y,3/4)",
                "(1/4,y,3/4)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y,1/4)",
                "(3/4,-y,3/4)",
                "(3/4,y,3/4)"
            ],
            [
                "(x,1/2,1/4)",
                "(-x+1/2,1/2,1/4)",
                "(-x,1/2,3/4)",
                "(x+1/2,1/2,3/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,1/4)",
                "(-x,0,3/4)",
                "(x+1/2,0,3/4)"
            ],
            [
                "(1/4,1/2,z)",
                "(1/4,1/2,-z+1/2)",
                "(3/4,1/2,-z)",
                "(3/4,1/2,z+1/2)"
            ],
            [
                "(1/4,0,z)",
                "(1/4,0,-z+1/2)",
                "(3/4,0,-z)",
                "(3/4,0,z+1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(0,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(3/4,1/2,3/4)"
            ],
            [
                "(1/4,1/2,3/4)",
                "(3/4,1/2,1/4)"
            ],
            [
                "(1/4,0,3/4)",
                "(3/4,0,1/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,0,3/4)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z+1/2)",
                "(0,0,-z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-1",
            "222",
            "222",
            "222",
            "222",
            "2..",
            "2/m..",
            "2/m..",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "P 2/c 2/n 2/a",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "239": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(-x,y,-z)",
                "(x+1/2,-y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)",
                "(x,-y,z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,-y,z)",
                "(3/4,y,-z)",
                "(3/4,-y,-z)"
            ],
            [
                "(x,1/2,z)",
                "(-x+1/2,1/2,z)",
                "(-x,1/2,-z)",
                "(x+1/2,1/2,-z)"
            ],
            [
                "(x,0,z)",
                "(-x+1/2,0,z)",
                "(-x,0,-z)",
                "(x+1/2,0,-z)"
            ],
            [
                "(0,y,1/2)",
                "(1/2,-y,1/2)",
                "(0,-y,1/2)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,0)",
                "(0,-y,0)",
                "(1/2,y,0)"
            ],
            [
                "(1/4,1/2,z)",
                "(3/4,1/2,-z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            ".m.",
            ".2.",
            ".2.",
            "mm2",
            "mm2",
            ".2/m.",
            ".2/m.",
            ".2/m.",
            ".2/m."
        ],
        "Space Group": "P 2sub1/m 2/m 2/a",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "240": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(x,-y,-z)",
                "(-x,y+1/2,-z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)",
                "(-x,y,z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,1/4,z)",
                "(x,3/4,-z)",
                "(-x,3/4,-z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y+1/2,z)",
                "(1/2,-y,-z)",
                "(1/2,y+1/2,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y+1/2,z)",
                "(0,-y,-z)",
                "(0,y+1/2,-z)"
            ],
            [
                "(x,0,1/2)",
                "(-x,1/2,1/2)",
                "(-x,0,1/2)",
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,0)",
                "(-x,0,0)",
                "(x,1/2,0)"
            ],
            [
                "(1/2,1/4,z)",
                "(1/2,3/4,-z)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            ".m.",
            ".2.",
            ".2.",
            "mm2",
            "mm2",
            ".2/m.",
            ".2/m.",
            ".2/m.",
            ".2/m."
        ],
        "Space Group": "P 2/m 2sub1/m 2/b",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "241": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x,-y,z)",
                "(-x,y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)",
                "(x,y,-z)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,1/4,z)",
                "(x,1/4,-z)",
                "(-x,3/4,z)",
                "(-x,3/4,-z)"
            ],
            [
                "(x,y,1/2)",
                "(x,-y+1/2,1/2)",
                "(-x,-y,1/2)",
                "(-x,y+1/2,1/2)"
            ],
            [
                "(x,y,0)",
                "(x,-y+1/2,0)",
                "(-x,-y,0)",
                "(-x,y+1/2,0)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,1/2,-z)",
                "(1/2,0,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z)",
                "(0,0,-z)",
                "(0,1/2,z)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x,3/4,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            ".m.",
            ".2.",
            ".2.",
            "mm2",
            "mm2",
            ".2/m.",
            ".2/m.",
            ".2/m.",
            ".2/m."
        ],
        "Space Group": "P 2/b 2sub1/m 2/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "242": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x,y,-z)",
                "(-x,-y,z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)",
                "(x,-y,z)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(x,-y,1/4)",
                "(-x,y,3/4)",
                "(-x,-y,3/4)"
            ],
            [
                "(x,1/2,z)",
                "(x,1/2,-z+1/2)",
                "(-x,1/2,-z)",
                "(-x,1/2,z+1/2)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z+1/2)",
                "(-x,0,-z)",
                "(-x,0,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,1/2)",
                "(1/2,-y,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,1/2)",
                "(0,-y,0)",
                "(0,y,1/2)"
            ],
            [
                "(x,1/2,1/4)",
                "(-x,1/2,3/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,3/4)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            ".m.",
            ".2.",
            ".2.",
            "mm2",
            "mm2",
            ".2/m.",
            ".2/m.",
            ".2/m.",
            ".2/m."
        ],
        "Space Group": "P 2/c 2/m 2sub1/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "243": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z)",
                "(-x,-y,z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z)",
                "(x,y,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-x,y,1/4)",
                "(x,-y,3/4)",
                "(-x,-y,3/4)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,y,-z+1/2)",
                "(1/2,-y,-z)",
                "(1/2,-y,z+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z+1/2)",
                "(0,-y,-z)",
                "(0,-y,z+1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,1/2)",
                "(-x,1/2,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,1/2)",
                "(-x,0,0)",
                "(x,0,1/2)"
            ],
            [
                "(1/2,y,1/4)",
                "(1/2,-y,3/4)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            ".m.",
            ".2.",
            ".2.",
            "mm2",
            "mm2",
            ".2/m.",
            ".2/m.",
            ".2/m.",
            ".2/m."
        ],
        "Space Group": "P 2/m 2/c 2sub1/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "244": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(-x,-y,z)",
                "(x+1/2,-y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)",
                "(x,y,-z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,y,-z)",
                "(3/4,-y,z)",
                "(3/4,-y,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-x+1/2,y,1/2)",
                "(-x,-y,1/2)",
                "(x+1/2,-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x+1/2,y,0)",
                "(-x,-y,0)",
                "(x+1/2,-y,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,1/2,-z)",
                "(0,1/2,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,-z)",
                "(0,0,-z)",
                "(1/2,0,z)"
            ],
            [
                "(1/4,y,1/2)",
                "(3/4,-y,1/2)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y,0)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            ".m.",
            ".2.",
            ".2.",
            "mm2",
            "mm2",
            ".2/m.",
            ".2/m.",
            ".2/m.",
            ".2/m."
        ],
        "Space Group": "P 2sub1/m 2/a 2/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "245": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x,y+1/2,z+1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,3/4,1/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,1/4,3/4)"
            ],
            [
                "(1/4,0,z)",
                "(1/4,1/2,-z+1/2)",
                "(3/4,0,-z)",
                "(3/4,1/2,z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)",
                "(1/2,1/2,1/2)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/n 2sub1/n 2/a",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "246": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x+1/2,y,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(x+1/2,-y,z+1/2)"
            ],
            [
                "(1/4,y,3/4)",
                "(3/4,-y+1/2,3/4)",
                "(3/4,-y,1/4)",
                "(1/4,y+1/2,1/4)"
            ],
            [
                "(0,1/4,z)",
                "(1/2,1/4,-z+1/2)",
                "(0,3/4,-z)",
                "(1/2,3/4,z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/n 2/n 2/b",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "247": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y,z+1/2)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y+1/2,3/4)",
                "(3/4,-y,3/4)",
                "(3/4,y+1/2,1/4)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,1/4,1/2)",
                "(-x,3/4,0)",
                "(x+1/2,3/4,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/b 2/n 2sub1/n",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "248": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(x+1/2,y+1/2,-z)"
            ],
            [
                "(3/4,1/4,z)",
                "(3/4,3/4,-z+1/2)",
                "(1/4,3/4,-z)",
                "(1/4,1/4,z+1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,1/2,1/4)",
                "(-x,0,3/4)",
                "(x+1/2,1/2,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/c 2sub1/n 2/n",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "249": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(x+1/2,y+1/2,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(1/4,3/4,z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(1/2,-y+1/2,1/4)",
                "(0,-y,3/4)",
                "(1/2,y+1/2,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/n 2/c 2/n",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "250": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(-x,y+1/2,z+1/2)"
            ],
            [
                "(x,3/4,1/4)",
                "(-x+1/2,3/4,3/4)",
                "(-x,1/4,3/4)",
                "(x+1/2,1/4,1/4)"
            ],
            [
                "(1/4,y,0)",
                "(1/4,-y+1/2,1/2)",
                "(3/4,-y,0)",
                "(3/4,y+1/2,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)",
                "(1/2,1/2,1/2)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/n 2/a 2sub1/n",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "251": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(1/2,-y,z+1/2)",
                "(1/2,y,-z+1/2)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y,3/4)",
                "(3/4,-y,3/4)",
                "(3/4,y,1/4)"
            ],
            [
                "(x,1/2,0)",
                "(-x+1/2,1/2,1/2)",
                "(-x,1/2,0)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,0,1/2)",
                "(-x,0,0)",
                "(x+1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "2..",
            "2/m..",
            "2/m..",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "P 2/m 2/n 2sub1/a",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "252": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,1/2,z+1/2)",
                "(x,1/2,-z+1/2)",
                "(-x,0,-z)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(x,3/4,3/4)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y+1/2,1/2)",
                "(1/2,-y,0)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y+1/2,1/2)",
                "(0,-y,0)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "2..",
            "2/m..",
            "2/m..",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "P 2/n 2/m 2sub1/b",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "253": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(-x+1/2,y+1/2,z)",
                "(x+1/2,y+1/2,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(x+1/2,1/2,-z)",
                "(-x+1/2,1/2,z)",
                "(-x,0,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z)",
                "(3/4,3/4,-z)",
                "(1/4,3/4,z)"
            ],
            [
                "(0,y,1/2)",
                "(1/2,-y+1/2,1/2)",
                "(0,-y,1/2)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y+1/2,0)",
                "(0,-y,0)",
                "(1/2,y+1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "2..",
            "2/m..",
            "2/m..",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "P 2sub1/b 2/m 2/n",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "254": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,y,-z+1/2)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,-y,z+1/2)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(x+1/2,-y,1/2)",
                "(-x+1/2,y,1/2)",
                "(-x,-y,0)"
            ],
            [
                "(3/4,y,1/4)",
                "(1/4,-y,1/4)",
                "(1/4,-y,3/4)",
                "(3/4,y,3/4)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,1/2,-z+1/2)",
                "(0,1/2,-z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,-z+1/2)",
                "(0,0,-z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "2..",
            "2/m..",
            "2/m..",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "P 2sub1/c 2/n 2/m",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "255": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,y+1/2,1/2)",
                "(x,-y+1/2,1/2)",
                "(-x,-y,0)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(-x,3/4,3/4)",
                "(x,1/4,3/4)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,0,-z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z+1/2)",
                "(0,0,-z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "2..",
            "2/m..",
            "2/m..",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "P 2/n 2sub1/c 2/m",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "256": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(-x+1/2,-y+1/2,z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,-y+1/2,z)",
                "(x+1/2,y+1/2,-z)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(1/2,y+1/2,-z)",
                "(1/2,-y+1/2,z)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,3/4,z)",
                "(1/4,1/4,-z)",
                "(3/4,1/4,-z)",
                "(3/4,3/4,z)"
            ],
            [
                "(x,0,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(-x,0,1/2)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,1/2,0)",
                "(-x,0,0)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "2..",
            "2/m..",
            "2/m..",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "P 2/m 2sub1/a 2/n",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "257": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(-x,y,-z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)",
                "(x,-y,z+1/2)",
                "(-x+1/2,y,z+1/2)"
            ],
            [
                "(1/4,1/2,z)",
                "(3/4,1/2,-z+1/2)",
                "(3/4,1/2,-z)",
                "(1/4,1/2,z+1/2)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z+1/2)",
                "(3/4,0,-z)",
                "(1/4,0,z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(1/2,-y,1/4)",
                "(0,-y,3/4)",
                "(1/2,y,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)",
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/c 2/c 2/a",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "258": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(x,-y,-z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)",
                "(-x,y,z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(1/2,1/4,z)",
                "(1/2,3/4,-z+1/2)",
                "(1/2,3/4,-z)",
                "(1/2,1/4,z+1/2)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z+1/2)",
                "(0,3/4,-z)",
                "(0,1/4,z+1/2)"
            ],
            [
                "(x,0,3/4)",
                "(-x,1/2,3/4)",
                "(-x,0,1/4)",
                "(x,1/2,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/c 2sub1/c 2/b",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "259": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x+1/2,-y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)",
                "(x+1/2,y,-z)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x+1/2,3/4,1/2)",
                "(-x,3/4,1/2)",
                "(x+1/2,1/4,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,3/4,0)",
                "(-x,3/4,0)",
                "(x+1/2,1/4,0)"
            ],
            [
                "(1/4,0,z)",
                "(1/4,1/2,-z)",
                "(3/4,0,-z)",
                "(3/4,1/2,z)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/b 2sub1/a 2/a",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "260": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x+1/2,y,-z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)",
                "(x+1/2,-y,z)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(x,1/2,1/4)",
                "(-x+1/2,1/2,3/4)",
                "(-x,1/2,3/4)",
                "(x+1/2,1/2,1/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(-x,0,3/4)",
                "(x+1/2,0,1/4)"
            ],
            [
                "(3/4,y,0)",
                "(3/4,-y,1/2)",
                "(1/4,-y,0)",
                "(1/4,y,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/c 2/a 2sub1/a",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "261": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(x,-y+1/2,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)",
                "(-x,y+1/2,z)",
                "(x,y+1/2,-z+1/2)"
            ],
            [
                "(1/2,y,1/4)",
                "(1/2,-y+1/2,3/4)",
                "(1/2,-y,3/4)",
                "(1/2,y+1/2,1/4)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y+1/2,3/4)",
                "(0,-y,3/4)",
                "(0,y+1/2,1/4)"
            ],
            [
                "(x,1/4,0)",
                "(-x,1/4,1/2)",
                "(-x,3/4,0)",
                "(x,3/4,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/b 2/c 2sub1/b",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "262": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(-x,-y+1/2,z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)",
                "(x,y+1/2,-z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(1/4,y,1/2)",
                "(3/4,-y+1/2,1/2)",
                "(3/4,-y,1/2)",
                "(1/4,y+1/2,1/2)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y+1/2,0)",
                "(3/4,-y,0)",
                "(1/4,y+1/2,0)"
            ],
            [
                "(0,3/4,z)",
                "(1/2,3/4,-z)",
                "(0,1/4,-z)",
                "(1/2,1/4,z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/b 2/a 2/b",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "263": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,-y,1/2)",
                "(-x+1/2,y+1/2,1/2)",
                "(x+1/2,-y+1/2,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x+1/2,y+1/2,0)",
                "(x+1/2,-y+1/2,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z)",
                "(0,0,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..m",
            "..2",
            "..2",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2sub1/b 2sub1/a 2/m",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "264": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z)",
                "(x,y+1/2,-z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,-z)",
                "(1/2,-y+1/2,z+1/2)",
                "(1/2,y+1/2,-z+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)",
                "(0,-y+1/2,z+1/2)",
                "(0,y+1/2,-z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,1/2,0)",
                "(-x,0,1/2)",
                "(x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)",
                "(-x,0,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..m",
            "..2",
            "..2",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2/m 2sub1/c 2sub1/b",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "265": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,-z)",
                "(x+1/2,1/2,-z+1/2)",
                "(-x+1/2,1/2,z+1/2)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)",
                "(x+1/2,0,-z+1/2)",
                "(-x+1/2,0,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(0,-y,1/2)",
                "(1/2,-y,0)",
                "(0,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,1/2)",
                "(0,-y,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..m",
            "..2",
            "..2",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2sub1/c 2/m 2sub1/a",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "266": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y,z+1/2)"
            ],
            [
                "(1/4,3/4,z)",
                "(3/4,1/4,-z+1/2)",
                "(3/4,1/4,-z)",
                "(1/4,3/4,z+1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(1/4,1/4,z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/c 2sub1/c 2/n",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "267": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z+1/2)",
                "(-x+1/2,-y,z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z+1/2)",
                "(x+1/2,y,-z+1/2)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x+1/2,3/4,1/4)",
                "(-x,3/4,1/4)",
                "(x+1/2,1/4,3/4)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,3/4,3/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,1/4,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)",
                "(0,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/n 2sub1/a 2sub1/a",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "268": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z+1/2)",
                "(-x+1/2,y+1/2,z)",
                "(x,y+1/2,-z+1/2)"
            ],
            [
                "(3/4,y,1/4)",
                "(1/4,-y+1/2,3/4)",
                "(1/4,-y,3/4)",
                "(3/4,y+1/2,1/4)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y+1/2,3/4)",
                "(3/4,-y,3/4)",
                "(1/4,y+1/2,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)",
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/b 2/n 2sub1/b",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "269": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x,y,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x,y+1/2,z)"
            ],
            [
                "(x,y,1/4)",
                "(-x,-y,3/4)",
                "(-x,y+1/2,1/4)",
                "(x,-y+1/2,3/4)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,1/2)",
                "(-x,3/4,0)",
                "(x,1/4,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(0,1/2,1/2)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/b 2sub1/c 2sub1/m",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "270": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,y,-z)",
                "(-x,-y,-z)",
                "(x,y,-z+1/2)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,-y,z)"
            ],
            [
                "(x,y,3/4)",
                "(-x,-y,1/4)",
                "(x+1/2,-y,3/4)",
                "(-x+1/2,y,1/4)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y,1/2)",
                "(3/4,-y,0)",
                "(1/4,y,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(1/2,0,1/2)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/c 2/a 2sub1/m",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "271": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y,-z+1/2)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y,z+1/2)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,-y,-z)",
                "(1/4,-y,z+1/2)",
                "(3/4,y,-z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(1/2,-y,3/4)",
                "(0,-y,3/4)",
                "(1/2,y,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)",
                "(1/2,0,1/2)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/m 2/c 2sub1/a",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "272": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y+1/2,z)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z)",
                "(x+1/2,-y+1/2,z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(3/4,y,z)",
                "(1/4,-y,-z)",
                "(3/4,y+1/2,-z)",
                "(1/4,-y+1/2,z)"
            ],
            [
                "(0,1/4,z)",
                "(1/2,3/4,-z)",
                "(0,3/4,-z)",
                "(1/2,1/4,z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)",
                "(1/2,1/2,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/m 2sub1/a 2/b",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "273": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x+1/2,-y,z)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(x+1/2,y,-z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,3/4,-z)",
                "(x+1/2,1/4,-z)",
                "(-x+1/2,3/4,z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,1/2,-z)",
                "(3/4,0,-z)",
                "(1/4,1/2,z)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/b 2sub1/m 2/a",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "274": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(x,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z)",
                "(x,y+1/2,-z+1/2)",
                "(-x,y,z+1/2)"
            ],
            [
                "(x,3/4,z)",
                "(-x,1/4,-z)",
                "(-x,3/4,z+1/2)",
                "(x,1/4,-z+1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x,1/2,3/4)",
                "(-x,0,3/4)",
                "(x,1/2,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/c 2sub1/m 2sub1/b",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "275": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x+1/2,y+1/2,1/2)",
                "(x+1/2,-y+1/2,1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z+1/2)",
                "(0,1/2,-z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2sub1/n 2sub1/n 2/m",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "276": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)",
                "(1/2,-y+1/2,z+1/2)",
                "(1/2,y+1/2,-z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x+1/2,1/2,0)",
                "(-x,0,1/2)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,1/2,1/2)",
                "(-x,0,0)",
                "(x+1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2/m 2sub1/n 2sub1/n",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "277": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)",
                "(x+1/2,1/2,-z+1/2)",
                "(-x+1/2,1/2,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(0,-y+1/2,1/2)",
                "(1/2,-y,0)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y+1/2,1/2)",
                "(0,-y,0)",
                "(1/2,y+1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m"
        ],
        "Space Group": "P 2sub1/n 2/m 2sub1/n",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "278": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x+1/2,-y+1/2,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(-x+1/2,1/2,-z)",
                "(x+1/2,1/2,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(1/2,y+1/2,-z)",
                "(1/2,-y+1/2,-z)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,3/4,1/2)",
                "(1/4,3/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)",
                "(1/4,3/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "-1",
            "-1",
            "mm2",
            "mm2"
        ],
        "Space Group": "P 2sub1/m 2sub1/m 2/n",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "279": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,y+1/2,-z)",
                "(x+1/2,-y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x,-y+1/2,z)",
                "(-x+1/2,y,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x+1/2,1/4,z)",
                "(-x,3/4,-z)",
                "(x+1/2,3/4,-z)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,-y+1/2,z)",
                "(3/4,y+1/2,-z)",
                "(3/4,-y,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(1/4,3/4,z)",
                "(3/4,1/4,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "-1",
            "-1",
            "mm2",
            "mm2"
        ],
        "Space Group": "P 2sub1/m 2sub1/m 2/n",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "280": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,-y+1/2,-z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(x,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(x,y,0)",
                "(x,-y,0)",
                "(-x,-y+1/2,1/2)",
                "(-x,y+1/2,1/2)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z)",
                "(-x,1/2,z+1/2)",
                "(-x,1/2,-z+1/2)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(1/2,3/4,3/4)",
                "(1/2,1/4,3/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,3/4)",
                "(0,1/4,3/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(x,0,1/2)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "-1",
            "-1",
            "mm2",
            "mm2"
        ],
        "Space Group": "P 2/n 2sub1/m 2sub1/m",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "281": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,z+1/2)",
                "(-x,y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z+1/2)",
                "(x,y,-z+1/2)",
                "(x,-y+1/2,z)"
            ],
            [
                "(x,y,1/4)",
                "(x,-y+1/2,1/4)",
                "(-x,-y,3/4)",
                "(-x,y+1/2,3/4)"
            ],
            [
                "(x,1/4,z)",
                "(x,1/4,-z+1/2)",
                "(-x,3/4,z+1/2)",
                "(-x,3/4,-z)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)",
                "(0,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x,3/4,1/4)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "-1",
            "-1",
            "mm2",
            "mm2"
        ],
        "Space Group": "P 2/n 2sub1/m 2sub1/m",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "282": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,-y,z+1/2)",
                "(-x+1/2,-y,-z+1/2)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z)",
                "(1/2,-y,-z+1/2)",
                "(1/2,-y,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,y,0)",
                "(x+1/2,-y,1/2)",
                "(-x+1/2,-y,1/2)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(3/4,1/2,3/4)",
                "(3/4,1/2,1/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,0,3/4)",
                "(3/4,0,1/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(1/2,y,0)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "-1",
            "-1",
            "mm2",
            "mm2"
        ],
        "Space Group": "P 2sub1/m 2/n 2sub1/m",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "283": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x+1/2,-y,-z)",
                "(-x,-y,z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z+1/2)",
                "(-x+1/2,y,z)",
                "(x,y,-z+1/2)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,y,-z+1/2)",
                "(3/4,-y,-z)",
                "(3/4,-y,z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-x+1/2,y,1/4)",
                "(x+1/2,-y,3/4)",
                "(-x,-y,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(3/4,y,1/4)",
                "(1/4,-y,3/4)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "-1",
            "-1",
            "mm2",
            "mm2"
        ],
        "Space Group": "P 2sub1/m 2/n 2sub1/m",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "284": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-x,y,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(x,-y,z+1/2)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(0,y,1/4)",
                "(1/2,-y+1/2,3/4)",
                "(0,-y,3/4)",
                "(1/2,y+1/2,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)",
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/b 2/c 2sub1/n",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "285": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(x,-y,-z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(-x,y,z+1/2)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(x,0,3/4)",
                "(-x+1/2,1/2,1/4)",
                "(-x,0,1/4)",
                "(x+1/2,1/2,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)",
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/c 2sub1/a 2sub1/n",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "286": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x+1/2,-y,z)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(x+1/2,y,-z)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,1/2,-z+1/2)",
                "(3/4,0,-z)",
                "(1/4,1/2,z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)",
                "(1/2,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/n 2sub1/c 2/a",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "287": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x+1/2,y,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(x+1/2,-y,z)",
                "(x,y+1/2,-z+1/2)"
            ],
            [
                "(3/4,y,0)",
                "(1/4,-y+1/2,1/2)",
                "(1/4,-y,0)",
                "(3/4,y+1/2,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)",
                "(1/2,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/n 2/a 2sub1/b",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "288": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x,-y+1/2,-z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x,y+1/2,z)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,3/4,1/2)",
                "(-x,3/4,0)",
                "(x+1/2,1/4,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)",
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2/b 2sub1/n 2sub1/a",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "289": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(-x,-y+1/2,z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(x,y+1/2,-z)",
                "(-x+1/2,y,z+1/2)"
            ],
            [
                "(0,3/4,z)",
                "(1/2,1/4,-z+1/2)",
                "(0,1/4,-z)",
                "(1/2,3/4,z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)",
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/c 2sub1/n 2/b",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "290": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/b 2sub1/c 2sub1/a",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "291": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/c 2sub1/a 2sub1/b",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "292": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y+1/2,z)",
                "(-x+1/2,y+1/2,z+1/2)"
            ],
            [
                "(x,1/4,z)",
                "(-x+1/2,3/4,z+1/2)",
                "(-x,3/4,-z)",
                "(x+1/2,1/4,-z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/n 2sub1/m 2sub1/a",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "293": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(x+1/2,-y,-z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(-x+1/2,y,z)",
                "(x+1/2,-y+1/2,z+1/2)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,-y+1/2,z+1/2)",
                "(3/4,-y,-z)",
                "(1/4,y+1/2,-z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,0)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)",
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/m 2sub1/n 2sub1/b",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "294": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,z+1/2)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x+1/2,y+1/2,z)",
                "(x,y,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(x+1/2,-y+1/2,3/4)",
                "(-x,-y,3/4)",
                "(-x+1/2,y+1/2,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/b 2sub1/n 2sub1/m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "295": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x,y+1/2,-z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z+1/2)",
                "(x,-y+1/2,z)",
                "(x+1/2,y+1/2,-z+1/2)"
            ],
            [
                "(x,1/4,z)",
                "(x+1/2,3/4,-z+1/2)",
                "(-x,3/4,-z)",
                "(-x+1/2,1/4,z+1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,0,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/c 2sub1/m 2sub1/n",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "296": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y,-z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y,z)",
                "(x+1/2,y+1/2,-z+1/2)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,y+1/2,-z+1/2)",
                "(3/4,-y,-z)",
                "(1/4,-y+1/2,z+1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,0,1/2)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)",
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/m 2sub1/c 2sub1/n",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "297": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,-y+1/2,z)",
                "(x,y,-z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-x+1/2,y+1/2,3/4)",
                "(-x,-y,3/4)",
                "(x+1/2,-y+1/2,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "-1",
            "-1"
        ],
        "Space Group": "P 2sub1/n 2sub1/a 2sub1/m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "298": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,y,-z+1/2)",
                "(x,-y,z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(x,y,1/4)",
                "(-x,-y,3/4)",
                "(-x,y,1/4)",
                "(x,-y,3/4)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z+1/2)",
                "(0,y,-z+1/2)",
                "(0,-y,-z)"
            ],
            [
                "(x,0,0)",
                "(-x,0,1/2)",
                "(-x,0,0)",
                "(x,0,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,1/2)",
                "(3/4,1/4,1/2)",
                "(1/4,3/4,0)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "2..",
            "-1",
            "m2m",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "C 2/m 2/c 2sub1/m",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "299": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(x,-y,-z+1/2)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x,y,-z+1/2)",
                "(-x,y,z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,y,3/4)",
                "(-x,-y,1/4)",
                "(x,-y,3/4)",
                "(-x,y,1/4)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z+1/2)",
                "(x,0,-z+1/2)",
                "(-x,0,-z)"
            ],
            [
                "(0,y,0)",
                "(0,-y,1/2)",
                "(0,-y,0)",
                "(0,y,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,1/2)",
                "(1/4,3/4,1/2)",
                "(3/4,1/4,0)"
            ],
            [
                "(x,0,3/4)",
                "(-x,0,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "2..",
            "-1",
            "m2m",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "C 2/c 2/m 2sub1/m",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "300": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(-x+1/2,-y,z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z)",
                "(x+1/2,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(1/4,y,z)",
                "(3/4,-y,-z)",
                "(1/4,-y,z)",
                "(3/4,y,-z)"
            ],
            [
                "(x,0,z)",
                "(x+1/2,0,-z)",
                "(-x+1/2,0,z)",
                "(-x,0,-z)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,0)",
                "(0,-y,0)",
                "(1/2,y,0)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,3/4,3/4)",
                "(1/2,3/4,1/4)",
                "(0,1/4,3/4)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "2..",
            "-1",
            "m2m",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "A 2sub1/m 2/m 2/a",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "301": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z)",
                "(-x+1/2,y,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z)",
                "(x+1/2,-y,z)",
                "(x,y,-z)"
            ],
            [
                "(3/4,y,z)",
                "(1/4,-y,-z)",
                "(3/4,y,-z)",
                "(1/4,-y,z)"
            ],
            [
                "(x,y,0)",
                "(x+1/2,-y,0)",
                "(-x+1/2,y,0)",
                "(-x,-y,0)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,-z)",
                "(0,0,-z)",
                "(1/2,0,z)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,3/4,3/4)",
                "(1/2,1/4,3/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(3/4,y,0)",
                "(1/4,-y,0)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "2..",
            "-1",
            "m2m",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "A 2sub1/m 2/a 2/m",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "302": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(x,-y+1/2,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z)",
                "(-x,y+1/2,z)",
                "(x,y,-z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,3/4,-z)",
                "(x,1/4,-z)",
                "(-x,3/4,z)"
            ],
            [
                "(x,y,0)",
                "(-x,y+1/2,0)",
                "(x,-y+1/2,0)",
                "(-x,-y,0)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z)",
                "(0,0,-z)",
                "(0,1/2,z)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,1/2,3/4)",
                "(1/4,1/2,3/4)",
                "(3/4,0,1/4)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "2..",
            "-1",
            "m2m",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "B 2/b 2sub1/m 2/m",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "303": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z)",
                "(-x,-y+1/2,z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z)",
                "(x,y+1/2,-z)",
                "(-x,y,z)"
            ],
            [
                "(x,3/4,z)",
                "(-x,1/4,-z)",
                "(-x,3/4,z)",
                "(x,1/4,-z)"
            ],
            [
                "(0,y,z)",
                "(0,y+1/2,-z)",
                "(0,-y+1/2,z)",
                "(0,-y,-z)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,0)",
                "(-x,0,0)",
                "(x,1/2,0)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,1/2,3/4)",
                "(3/4,1/2,1/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(0,3/4,z)",
                "(0,1/4,-z)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "2..",
            "-1",
            "m2m",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "B 2/m 2sub1/m 2/b",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "304": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y+1/2,z+1/2)",
                "(0,y+1/2,-z+1/2)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y+1/2,3/4)",
                "(3/4,-y,3/4)",
                "(1/4,y+1/2,1/4)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)",
                "(-x,0,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,1/4,1/2)",
                "(3/4,3/4,1/2)",
                "(1/4,3/4,0)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "-1",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "C 2/m 2/c 2sub1/a",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "305": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(-x+1/2,y,z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(-x+1/2,0,z+1/2)",
                "(x+1/2,0,-z+1/2)",
                "(-x,0,-z)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x+1/2,3/4,1/4)",
                "(-x,3/4,1/4)",
                "(x+1/2,1/4,3/4)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,1/2)",
                "(0,-y,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,3/4,1/2)",
                "(3/4,3/4,1/2)",
                "(3/4,1/4,0)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "-1",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "C 2/c 2/m 2sub1/b",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "306": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,0,z)",
                "(x+1/2,0,-z+1/2)",
                "(-x+1/2,0,z+1/2)",
                "(-x,0,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(1/4,1/4,z+1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,1/2)",
                "(0,-y,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,3/4,1/4)",
                "(1/2,3/4,3/4)",
                "(0,1/4,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "-1",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "A 2sub1/b 2/m a",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "307": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(-x+1/2,y+1/2,z)",
                "(x+1/2,-y+1/2,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(x+1/2,-y+1/2,0)",
                "(-x+1/2,y+1/2,0)",
                "(-x,-y,0)"
            ],
            [
                "(3/4,y,1/4)",
                "(1/4,-y+1/2,3/4)",
                "(1/4,-y,3/4)",
                "(3/4,y+1/2,1/4)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z)",
                "(0,0,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,1/4,3/4)",
                "(1/2,3/4,3/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "-1",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "A 2sub1/c 2/a 2/m",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "308": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(x,y,-z)"
            ],
            [
                "(x,y,0)",
                "(-x+1/2,y+1/2,0)",
                "(x+1/2,-y+1/2,0)",
                "(-x,-y,0)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,3/4,3/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,1/4,1/4)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z)",
                "(0,0,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,1/2,3/4)",
                "(3/4,1/2,3/4)",
                "(3/4,0,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "-1",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "B 2/b 2sub1/c 2/m",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "309": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,-y+1/2,z+1/2)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,-y+1/2,z+1/2)",
                "(x,y+1/2,-z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(0,y,z)",
                "(0,y+1/2,-z+1/2)",
                "(0,-y+1/2,z+1/2)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,3/4,z)",
                "(3/4,1/4,-z+1/2)",
                "(3/4,1/4,-z)",
                "(1/4,3/4,z+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)",
                "(-x,0,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,1/2,1/4)",
                "(3/4,1/2,3/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "2..",
            "-1",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "B 2/m 2sub1/a 2/b",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "310": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,-y,1/2)",
                "(-x,y,1/2)",
                "(x,-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(-x,0,-z)",
                "(x,0,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z)",
                "(3/4,3/4,-z)",
                "(1/4,3/4,z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..m",
            ".m.",
            "m..",
            "..2",
            "mm2",
            "mm2",
            "m2m",
            "m2m",
            "2mm",
            "2mm",
            "..2/m",
            "..2/m",
            "mmm",
            "mmm",
            "mmm",
            "mmm"
        ],
        "Space Group": "C 2/m 2/m 2/m",
        "Wyckoff Letter": [
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "311": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z)",
                "(x,y,-z)",
                "(x,-y,z)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,-z)",
                "(1/2,-y,z)",
                "(1/2,y,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)",
                "(0,-y,z)",
                "(0,y,-z)"
            ],
            [
                "(x,y,0)",
                "(x,-y,0)",
                "(-x,-y,0)",
                "(-x,y,0)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z)",
                "(-x,0,z)",
                "(-x,0,-z)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(-x,3/4,3/4)",
                "(x,1/4,3/4)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(1/2,0,0)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..m",
            ".m.",
            "m..",
            "..2",
            "mm2",
            "mm2",
            "m2m",
            "m2m",
            "2mm",
            "2mm",
            "..2/m",
            "..2/m",
            "mmm",
            "mmm",
            "mmm",
            "mmm"
        ],
        "Space Group": "A 2/m 2/m 2/m",
        "Wyckoff Letter": [
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "312": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(x,y,-z)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,-z)",
                "(x,1/2,-z)",
                "(-x,1/2,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)",
                "(x,0,-z)",
                "(-x,0,z)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(0,-y,z)"
            ],
            [
                "(x,y,0)",
                "(-x,y,0)",
                "(x,-y,0)",
                "(-x,-y,0)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y,3/4)",
                "(3/4,-y,3/4)",
                "(3/4,y,1/4)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(0,1/2,0)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..m",
            ".m.",
            "m..",
            "..2",
            "mm2",
            "mm2",
            "m2m",
            "m2m",
            "2mm",
            "2mm",
            "..2/m",
            "..2/m",
            "mmm",
            "mmm",
            "mmm",
            "mmm"
        ],
        "Space Group": "B 2/m 2/m 2/m",
        "Wyckoff Letter": [
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "313": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x,y,1/2)",
                "(x,-y,1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(1/4,3/4,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,-z+1/2)",
                "(0,1/2,-z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,1/4)",
                "(0,-y,3/4)",
                "(0,y,3/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,1/4)",
                "(-x,0,3/4)",
                "(x,0,3/4)"
            ],
            [
                "(1/4,3/4,0)",
                "(3/4,3/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2",
            ".2.",
            "2..",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m",
            "222",
            "222"
        ],
        "Space Group": "C 2/c 2/c 2/m",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "314": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x+1/2,-y,z)",
                "(-x+1/2,y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z)",
                "(x+1/2,y,-z)",
                "(x+1/2,-y,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)",
                "(1/2,-y,z)",
                "(1/2,y,-z)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,3/4,1/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,1/4,3/4)"
            ],
            [
                "(x,0,1/2)",
                "(-x+1/2,0,1/2)",
                "(-x,0,1/2)",
                "(x+1/2,0,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,0,0)",
                "(-x,0,0)",
                "(x+1/2,0,0)"
            ],
            [
                "(1/4,0,z)",
                "(1/4,0,-z)",
                "(3/4,0,-z)",
                "(3/4,0,z)"
            ],
            [
                "(1/4,y,0)",
                "(1/4,-y,0)",
                "(3/4,-y,0)",
                "(3/4,y,0)"
            ],
            [
                "(0,1/4,3/4)",
                "(1/2,3/4,3/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)"
            ],
            [
                "(1/4,0,0)",
                "(3/4,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2",
            ".2.",
            "2..",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m",
            "222",
            "222"
        ],
        "Space Group": "A 2/m 2/a 2/a",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "315": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x,-y+1/2,-z)",
                "(-x,-y+1/2,z)",
                "(-x,-y,-z)",
                "(x,-y,z)",
                "(-x,y+1/2,z)",
                "(x,y+1/2,-z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)",
                "(x,1/2,-z)",
                "(-x,1/2,z)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y+1/2,3/4)",
                "(3/4,-y,3/4)",
                "(3/4,y+1/2,1/4)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y+1/2,0)",
                "(1/2,-y,0)",
                "(1/2,y+1/2,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y+1/2,0)",
                "(0,-y,0)",
                "(0,y+1/2,0)"
            ],
            [
                "(x,1/4,0)",
                "(-x,1/4,0)",
                "(-x,3/4,0)",
                "(x,3/4,0)"
            ],
            [
                "(0,1/4,z)",
                "(0,1/4,-z)",
                "(0,3/4,-z)",
                "(0,3/4,z)"
            ],
            [
                "(3/4,0,1/4)",
                "(3/4,1/2,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/2,1/4,0)",
                "(1/2,3/4,0)"
            ],
            [
                "(0,1/4,0)",
                "(0,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "..2",
            ".2.",
            "2..",
            "..2/m",
            "..2/m",
            "..2/m",
            "..2/m",
            "222",
            "222"
        ],
        "Space Group": "B 2/b 2/m 2/b",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "316": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(-x,y+1/2,-z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)",
                "(x,-y+1/2,z)",
                "(-x,y,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,1/4,z)",
                "(-x,3/4,-z)",
                "(x,3/4,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y+1/2,z)",
                "(0,y+1/2,-z)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,1/2,-z)",
                "(3/4,0,-z)",
                "(1/4,1/2,z)"
            ],
            [
                "(1/4,y,1/2)",
                "(3/4,-y+1/2,1/2)",
                "(3/4,-y,1/2)",
                "(1/4,y+1/2,1/2)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y+1/2,0)",
                "(3/4,-y,0)",
                "(1/4,y+1/2,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,1/2,1/2)",
                "(-x,0,1/2)",
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,0)",
                "(-x,0,0)",
                "(x,1/2,0)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)"
            ],
            [
                "(1/4,0,0)",
                "(3/4,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m..",
            "222",
            "222"
        ],
        "Space Group": "C 2/m 2/m 2/a",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "317": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(x+1/2,-y,-z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)",
                "(-x+1/2,y,z)",
                "(x,-y,z)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,-y,z)",
                "(3/4,-y,-z)",
                "(3/4,y,-z)"
            ],
            [
                "(x,0,z)",
                "(-x+1/2,0,z)",
                "(x+1/2,0,-z)",
                "(-x,0,-z)"
            ],
            [
                "(0,1/4,z)",
                "(1/2,3/4,-z)",
                "(0,3/4,-z)",
                "(1/2,1/4,z)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x+1/2,3/4,1/2)",
                "(-x,3/4,1/2)",
                "(x+1/2,1/4,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,3/4,0)",
                "(-x,3/4,0)",
                "(x+1/2,1/4,0)"
            ],
            [
                "(0,y,1/2)",
                "(1/2,-y,1/2)",
                "(0,-y,1/2)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,0)",
                "(0,-y,0)",
                "(1/2,y,0)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,3/4,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/4,1/2)",
                "(0,3/4,1/2)"
            ],
            [
                "(0,1/4,0)",
                "(0,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m..",
            "222",
            "222"
        ],
        "Space Group": "C 2/m 2/m 2/b",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "318": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x,-y,z+1/2)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)",
                "(x,y,-z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,y,1/4)",
                "(x,-y,1/4)",
                "(-x,-y,3/4)",
                "(-x,y,3/4)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z+1/2)",
                "(-x,0,z+1/2)",
                "(-x,0,-z)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,1/2)",
                "(-x,3/4,0)",
                "(x,1/4,1/2)"
            ],
            [
                "(1/2,1/4,z)",
                "(1/2,3/4,-z+1/2)",
                "(1/2,3/4,-z)",
                "(1/2,1/4,z+1/2)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z+1/2)",
                "(0,3/4,-z)",
                "(0,1/4,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,1/2)",
                "(1/2,-y,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,1/2)",
                "(0,-y,0)",
                "(0,y,1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,3/4)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(1/2,1/4,0)",
                "(1/2,3/4,0)"
            ],
            [
                "(0,1/4,0)",
                "(0,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m..",
            "222",
            "222"
        ],
        "Space Group": "A 2/b 2/m 2/m",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "319": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x,y+1/2,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)",
                "(x,-y+1/2,z)",
                "(x,y,-z)"
            ],
            [
                "(x,1/4,z)",
                "(x,1/4,-z)",
                "(-x,3/4,-z)",
                "(-x,3/4,z)"
            ],
            [
                "(x,y,0)",
                "(x,-y+1/2,0)",
                "(-x,y+1/2,0)",
                "(-x,-y,0)"
            ],
            [
                "(x,0,1/4)",
                "(-x,1/2,3/4)",
                "(-x,0,3/4)",
                "(x,1/2,1/4)"
            ],
            [
                "(1/2,y,1/4)",
                "(1/2,-y+1/2,3/4)",
                "(1/2,-y,3/4)",
                "(1/2,y+1/2,1/4)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y+1/2,3/4)",
                "(0,-y,3/4)",
                "(0,y+1/2,1/4)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,1/2,-z)",
                "(1/2,0,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z)",
                "(0,0,-z)",
                "(0,1/2,z)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(1/2,1/4,1/4)",
                "(1/2,1/4,3/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,1/4,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m..",
            "222",
            "222"
        ],
        "Space Group": "A 2/c 2/m 2/m",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "320": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(x+1/2,-y,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)",
                "(-x+1/2,y,z)",
                "(x,y,-z)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,y,-z)",
                "(3/4,-y,-z)",
                "(3/4,-y,z)"
            ],
            [
                "(x,y,0)",
                "(-x+1/2,y,0)",
                "(x+1/2,-y,0)",
                "(-x,-y,0)"
            ],
            [
                "(0,y,1/4)",
                "(1/2,-y,3/4)",
                "(0,-y,3/4)",
                "(1/2,y,1/4)"
            ],
            [
                "(x,1/2,1/4)",
                "(-x+1/2,1/2,3/4)",
                "(-x,1/2,3/4)",
                "(x+1/2,1/2,1/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(-x,0,3/4)",
                "(x+1/2,0,1/4)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,1/2,-z)",
                "(0,1/2,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,-z)",
                "(0,0,-z)",
                "(1/2,0,z)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y,0)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m..",
            "222",
            "222"
        ],
        "Space Group": "B 2/m 2/c 2/m",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "321": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(-x,-y,z+1/2)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)",
                "(x,y,-z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(x,y,1/4)",
                "(-x,y,1/4)",
                "(-x,-y,3/4)",
                "(x,-y,3/4)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z+1/2)",
                "(0,-y,z+1/2)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y,1/2)",
                "(3/4,-y,0)",
                "(1/4,y,1/2)"
            ],
            [
                "(1/4,1/2,z)",
                "(3/4,1/2,-z+1/2)",
                "(3/4,1/2,-z)",
                "(1/4,1/2,z+1/2)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z+1/2)",
                "(3/4,0,-z)",
                "(1/4,0,z+1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,1/2)",
                "(-x,1/2,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,1/2)",
                "(-x,0,0)",
                "(x,0,1/2)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(1/4,1/2,1/4)",
                "(3/4,1/2,1/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,0,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(1/4,1/2,0)",
                "(3/4,1/2,0)"
            ],
            [
                "(1/4,0,0)",
                "(3/4,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m..",
            "222",
            "222"
        ],
        "Space Group": "B 2/m 2/a 2/m",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "322": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x,y,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y+1/2,-z+1/2)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y,z+1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z)",
                "(3/4,1/4,-z+1/2)",
                "(1/4,1/4,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(0,1/2,-z+1/2)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y+1/2,0)",
                "(0,-y+1/2,1/2)",
                "(1/2,y,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,1/2,0)",
                "(-x,1/2,1/2)",
                "(x+1/2,0,1/2)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,1/4,1/4)",
                "(0,1/4,3/4)",
                "(1/2,1/4,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,1/2,1/4)",
                "(3/4,0,3/4)",
                "(3/4,1/2,3/4)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "C 2/c 2/c 2/a",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "323": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(-x,y,-z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)",
                "(x,-y,z+1/2)",
                "(-x+1/2,y,z+1/2)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z+1/2)",
                "(3/4,0,-z)",
                "(1/4,0,z+1/2)"
            ],
            [
                "(0,1/4,z)",
                "(0,1/4,-z+1/2)",
                "(0,3/4,-z)",
                "(0,3/4,z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(1/2,-y,1/4)",
                "(0,-y,3/4)",
                "(1/2,y,3/4)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,3/4,1/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,1/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)",
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(1/4,3/4,0)",
                "(1/4,1/4,0)",
                "(3/4,3/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(0,1/4,3/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "C 2/c 2/c 2/a",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "324": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(x,-y,-z)",
                "(-x+1/2,y+1/2,-z)",
                "(-x+1/2,-y,-z+1/2)",
                "(x,y+1/2,-z+1/2)",
                "(-x+1/2,y,z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,3/4,-z)",
                "(1/4,3/4,-z+1/2)",
                "(1/4,1/4,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,1/2,0)",
                "(-x+1/2,0,1/2)",
                "(x,1/2,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y+1/2,0)",
                "(1/2,-y,1/2)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(1/4,0,3/4)",
                "(1/4,1/2,3/4)",
                "(1/4,0,1/4)",
                "(1/4,1/2,1/4)"
            ],
            [
                "(0,1/4,3/4)",
                "(1/2,1/4,3/4)",
                "(0,3/4,1/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "C 2/c 2/c 2/b",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "325": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(x,-y,-z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)",
                "(-x,y,z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z+1/2)",
                "(0,3/4,-z)",
                "(0,1/4,z+1/2)"
            ],
            [
                "(1/4,0,z)",
                "(1/4,0,-z+1/2)",
                "(3/4,0,-z)",
                "(3/4,0,z+1/2)"
            ],
            [
                "(x,0,3/4)",
                "(-x,1/2,3/4)",
                "(-x,0,1/4)",
                "(x,1/2,1/4)"
            ],
            [
                "(1/4,y,3/4)",
                "(3/4,-y+1/2,3/4)",
                "(3/4,-y,1/4)",
                "(1/4,y+1/2,1/4)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(3/4,1/4,0)",
                "(1/4,1/4,0)",
                "(3/4,3/4,1/2)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,0,3/4)"
            ],
            [
                "(1/4,0,3/4)",
                "(3/4,0,1/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "C 2/c 2/c 2/b",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "326": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,-y,z)",
                "(-x,y+1/2,-z+1/2)",
                "(-x+1/2,-y,-z+1/2)",
                "(-x+1/2,y+1/2,z)",
                "(x+1/2,y,-z+1/2)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(-x+1/2,3/4,1/4)",
                "(x+1/2,1/4,1/4)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(-x+1/2,0,1/2)",
                "(x+1/2,0,1/2)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,-z+1/2)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,y,0)",
                "(0,-y+1/2,1/2)",
                "(1/2,-y,1/2)",
                "(1/2,y+1/2,0)"
            ],
            [
                "(1/4,0,1/4)",
                "(1/4,1/2,1/4)",
                "(3/4,0,1/4)",
                "(3/4,1/2,1/4)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,1/4,1/2)",
                "(3/4,3/4,0)",
                "(3/4,3/4,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "A 2/b 2/a 2/a",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "327": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x+1/2,-y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)",
                "(x+1/2,y,-z)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,3/4,0)",
                "(-x,3/4,0)",
                "(x+1/2,1/4,0)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,1/4)",
                "(-x,0,3/4)",
                "(x+1/2,0,3/4)"
            ],
            [
                "(1/4,0,z)",
                "(1/4,1/2,-z)",
                "(3/4,0,-z)",
                "(3/4,1/2,z)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y+1/2,3/4)",
                "(3/4,-y,3/4)",
                "(3/4,y+1/2,1/4)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,1/4,3/4)",
                "(0,1/4,1/4)",
                "(1/2,3/4,3/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(3/4,0,1/4)",
                "(1/4,0,3/4)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "A 2/b 2/a 2/a",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "328": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z+1/2)",
                "(-x,y,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x+1/2,-y+1/2,-z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,-y+1/2,z)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,1/4,3/4)",
                "(-x+1/2,1/4,3/4)",
                "(x+1/2,1/4,1/4)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(-x+1/2,1/2,0)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(0,y,0)",
                "(0,-y+1/2,1/2)",
                "(1/2,-y+1/2,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z+1/2)",
                "(1/2,1/2,-z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(3/4,1/4,0)",
                "(3/4,1/4,1/2)",
                "(1/4,1/4,0)",
                "(1/4,1/4,1/2)"
            ],
            [
                "(3/4,0,1/4)",
                "(3/4,1/2,1/4)",
                "(1/4,0,3/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "A 2/c 2/a 2/a",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "329": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x+1/2,y,-z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)",
                "(x+1/2,-y,z)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(-x,0,3/4)",
                "(x+1/2,0,1/4)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,1/4,0)",
                "(-x,3/4,0)",
                "(x+1/2,3/4,0)"
            ],
            [
                "(3/4,y,0)",
                "(3/4,-y,1/2)",
                "(1/4,-y,0)",
                "(1/4,y,1/2)"
            ],
            [
                "(3/4,1/4,z)",
                "(3/4,3/4,-z+1/2)",
                "(1/4,3/4,-z)",
                "(1/4,1/4,z+1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,3/4,1/4)",
                "(0,1/4,1/4)",
                "(1/2,3/4,3/4)",
                "(1/2,1/4,3/4)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)"
            ],
            [
                "(3/4,1/4,0)",
                "(1/4,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "A 2/c 2/a 2/a",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "330": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y,-z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x+1/2,-y+1/2,-z)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z)",
                "(x,y+1/2,-z+1/2)"
            ],
            [
                "(1/4,y,1/4)",
                "(1/4,-y,3/4)",
                "(1/4,-y+1/2,3/4)",
                "(1/4,y+1/2,1/4)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)",
                "(1/2,-y+1/2,0)",
                "(1/2,y+1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,0,1/2)",
                "(-x+1/2,1/2,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,-z+1/2)",
                "(1/2,1/2,-z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,1/4,1/2)",
                "(1/4,3/4,0)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(0,1/4,1/4)",
                "(1/2,1/4,1/4)",
                "(0,3/4,3/4)",
                "(1/2,3/4,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "B 2/b 2/c 2/b",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "331": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(x,-y+1/2,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)",
                "(-x,y+1/2,z)",
                "(x,y+1/2,-z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y+1/2,3/4)",
                "(0,-y,3/4)",
                "(0,y+1/2,1/4)"
            ],
            [
                "(1/4,y,0)",
                "(1/4,-y+1/2,0)",
                "(3/4,-y,0)",
                "(3/4,y+1/2,0)"
            ],
            [
                "(x,1/4,0)",
                "(-x,1/4,1/2)",
                "(-x,3/4,0)",
                "(x,3/4,1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(1/4,3/4,z+1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(3/4,0,1/4)",
                "(1/4,0,1/4)",
                "(3/4,1/2,3/4)",
                "(1/4,1/2,3/4)"
            ],
            [
                "(1/4,3/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "B 2/b 2/c 2/b",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "332": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z+1/2)",
                "(-x,-y,z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x,-y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z)",
                "(x,y+1/2,-z+1/2)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y,1/4)",
                "(3/4,-y+1/2,1/4)",
                "(1/4,y+1/2,1/4)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)",
                "(0,-y+1/2,1/2)",
                "(0,y+1/2,1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,-z+1/2)",
                "(0,1/2,-z+1/2)",
                "(1/2,1/2,z)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,0,1/2)",
                "(-x,1/2,1/2)",
                "(x+1/2,1/2,0)"
            ],
            [
                "(0,3/4,1/4)",
                "(1/2,3/4,1/4)",
                "(0,1/4,1/4)",
                "(1/2,1/4,1/4)"
            ],
            [
                "(1/4,3/4,0)",
                "(1/4,3/4,1/2)",
                "(3/4,1/4,0)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "B 2/b 2/a 2/b",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "333": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(-x,-y+1/2,z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)",
                "(x,y+1/2,-z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y+1/2,0)",
                "(3/4,-y,0)",
                "(1/4,y+1/2,0)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y+1/2,1/4)",
                "(0,-y,3/4)",
                "(0,y+1/2,3/4)"
            ],
            [
                "(0,3/4,z)",
                "(1/2,3/4,-z)",
                "(0,1/4,-z)",
                "(1/2,1/4,z)"
            ],
            [
                "(x,3/4,1/4)",
                "(-x+1/2,3/4,3/4)",
                "(-x,1/4,3/4)",
                "(x+1/2,1/4,1/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/4,0,3/4)",
                "(1/4,0,1/4)",
                "(3/4,1/2,3/4)",
                "(3/4,1/2,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,3/4)"
            ],
            [
                "(0,3/4,1/4)",
                "(0,1/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "B 2/b 2/a 2/b",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "334": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(-x,0,-z)",
                "(x,0,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(-x,3/4,3/4)",
                "(x,1/4,3/4)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y,1/4)",
                "(3/4,-y,3/4)",
                "(1/4,y,3/4)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,1/4,-z)",
                "(3/4,3/4,-z)",
                "(1/4,3/4,z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,0,1/4)",
                "(3/4,0,1/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "16",
            "16",
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            ".m.",
            "m..",
            "2..",
            ".2.",
            "..2",
            "mm2",
            "m2m",
            "2mm",
            "222",
            "..2/m",
            ".2/m.",
            "2/m..",
            "mmm",
            "mmm"
        ],
        "Space Group": "F 2/m 2/m 2/m",
        "Wyckoff Letter": [
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "335": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x+1/4,-y+1/4,-z+1/4)",
                "(x+1/4,y+1/4,-z+1/4)",
                "(x+1/4,-y+1/4,z+1/4)",
                "(-x+1/4,y+1/4,z+1/4)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/4,1/4,-z+1/4)",
                "(1/4,1/4,z+1/4)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)",
                "(1/4,-y+1/4,1/4)",
                "(1/4,y+1/4,1/4)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(-x+1/4,1/4,1/4)",
                "(x+1/4,1/4,1/4)"
            ],
            [
                "(5/8,5/8,5/8)",
                "(3/8,3/8,5/8)",
                "(3/8,5/8,3/8)",
                "(5/8,3/8,3/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,7/8,1/8)",
                "(7/8,1/8,7/8)",
                "(1/8,7/8,7/8)"
            ],
            [
                "(0,0,1/2)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/4,1/4,1/4)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "F 2/d 2/d 2/d",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "336": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+3/4,-y+3/4,z)",
                "(-x+3/4,y,-z+3/4)",
                "(x,-y+3/4,-z+3/4)",
                "(-x,-y,-z)",
                "(x+1/4,y+1/4,-z)",
                "(x+1/4,-y,z+1/4)",
                "(-x,y+1/4,z+1/4)"
            ],
            [
                "(1/8,1/8,z)",
                "(5/8,1/8,-z+3/4)",
                "(7/8,7/8,-z)",
                "(3/8,7/8,z+1/4)"
            ],
            [
                "(1/8,y,1/8)",
                "(5/8,-y+3/4,1/8)",
                "(7/8,-y,7/8)",
                "(3/8,y+1/4,7/8)"
            ],
            [
                "(x,1/8,1/8)",
                "(-x+3/4,5/8,1/8)",
                "(-x,7/8,7/8)",
                "(x+1/4,3/8,7/8)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(1/4,1/4,1/2)",
                "(1/4,1/2,1/4)",
                "(1/2,1/4,1/4)"
            ],
            [
                "(0,0,0)",
                "(3/4,3/4,0)",
                "(3/4,0,3/4)",
                "(0,3/4,3/4)"
            ],
            [
                "(1/8,1/8,5/8)",
                "(7/8,7/8,3/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,7/8,7/8)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1",
            "222",
            "222"
        ],
        "Space Group": "F 2/d 2/d 2/d",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "337": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x,y,0)",
                "(x,-y,0)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(-x,0,-z)",
                "(x,0,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(1/2,0,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,y,1/2)",
                "(0,-y,1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,0)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)"
            ],
            [
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            ".m.",
            "m..",
            "-1",
            "mm2",
            "mm2",
            "m2m",
            "m2m",
            "2mm",
            "2mm",
            "mmm",
            "mmm",
            "mmm",
            "mmm"
        ],
        "Space Group": "I 2/m 2/m 2/m",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "338": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-x+1/2,y+1/2,0)",
                "(x+1/2,-y+1/2,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z)",
                "(0,0,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,1/4)",
                "(0,-y,3/4)",
                "(0,y,3/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,1/4)",
                "(-x,0,3/4)",
                "(x,0,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "..2/m",
            "..2/m",
            "222",
            "222"
        ],
        "Space Group": "I 2/b 2/a 2/m",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "339": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(-x,y,z)",
                "(x,y+1/2,-z+1/2)",
                "(x,-y+1/2,z+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,-z)",
                "(0,-y+1/2,z+1/2)",
                "(0,y+1/2,-z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,1/2,0)",
                "(-x,0,1/2)",
                "(x,1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)",
                "(-x,0,0)",
                "(x,1/2,1/2)"
            ],
            [
                "(1/4,0,z)",
                "(1/4,0,-z)",
                "(3/4,0,-z)",
                "(3/4,0,z)"
            ],
            [
                "(1/4,y,0)",
                "(1/4,-y,0)",
                "(3/4,-y,0)",
                "(3/4,y,0)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/4,1/2,0)",
                "(3/4,1/2,0)"
            ],
            [
                "(1/4,0,0)",
                "(3/4,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "..2/m",
            "..2/m",
            "222",
            "222"
        ],
        "Space Group": "I 2/m 2/c 2/b",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "340": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,-y,-z)",
                "(x,-y,z)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,y,-z+1/2)"
            ],
            [
                "(x,0,z)",
                "(-x,0,-z)",
                "(x+1/2,0,-z+1/2)",
                "(-x+1/2,0,z+1/2)"
            ],
            [
                "(1/2,y,0)",
                "(0,-y,1/2)",
                "(1/2,-y,0)",
                "(0,y,1/2)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,1/2)",
                "(0,-y,0)",
                "(1/2,y,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x,1/4,0)",
                "(-x,3/4,0)",
                "(x,3/4,0)"
            ],
            [
                "(0,1/4,z)",
                "(0,1/4,-z)",
                "(0,3/4,-z)",
                "(0,3/4,z)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,1/4,3/4)",
                "(3/4,3/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/4,1/2)",
                "(0,3/4,1/2)"
            ],
            [
                "(0,1/4,0)",
                "(0,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            ".2.",
            "2..",
            "-1",
            "..2/m",
            "..2/m",
            "222",
            "222"
        ],
        "Space Group": "I 2/c 2/m 2/a",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "341": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z+1/2)",
                "(0,3/4,-z)",
                "(0,1/4,z+1/2)"
            ],
            [
                "(1/4,y,0)",
                "(1/4,-y,1/2)",
                "(3/4,-y,0)",
                "(3/4,y,1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(-x,0,3/4)",
                "(x+1/2,0,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1"
        ],
        "Space Group": "I 2/b 2/c 2/a",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "342": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z+1/2)",
                "(-x+1/2,y,z+1/2)",
                "(x+1/2,-y+1/2,z)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z+1/2)",
                "(3/4,0,-z)",
                "(1/4,0,z+1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x,1/4,1/2)",
                "(-x,3/4,0)",
                "(x,3/4,1/2)"
            ],
            [
                "(0,y,3/4)",
                "(0,-y+1/2,1/4)",
                "(0,-y,1/4)",
                "(0,y+1/2,3/4)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,1/4,1/4)",
                "(3/4,3/4,3/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            "2..",
            "-1",
            "-1"
        ],
        "Space Group": "I 2/c 2/a 2/b",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "343": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z)",
                "(-x,y+1/2,-z)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,y+1/2,-z)",
                "(x,-y+1/2,z)",
                "(-x,y,z)"
            ],
            [
                "(x,1/4,z)",
                "(-x,1/4,z)",
                "(-x,3/4,-z)",
                "(x,3/4,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y+1/2,z)",
                "(0,y+1/2,-z)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,y,1/4)",
                "(3/4,-y+1/2,1/4)",
                "(3/4,-y,3/4)",
                "(1/4,y+1/2,3/4)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,0)",
                "(-x,0,0)",
                "(x,1/2,0)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,-z)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            ".2.",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "I 2/m 2/m 2/a",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "344": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z)",
                "(x+1/2,-y,-z)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z)",
                "(-x+1/2,y,z)",
                "(x,-y,z)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,-y,z)",
                "(3/4,-y,-z)",
                "(3/4,y,-z)"
            ],
            [
                "(x,0,z)",
                "(-x+1/2,0,z)",
                "(x+1/2,0,-z)",
                "(-x,0,-z)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x+1/2,3/4,3/4)",
                "(-x,3/4,1/4)",
                "(x+1/2,1/4,1/4)"
            ],
            [
                "(0,y,0)",
                "(1/2,-y,0)",
                "(0,-y,0)",
                "(1/2,y,0)"
            ],
            [
                "(1/4,0,z)",
                "(3/4,0,-z)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            ".2.",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "I 2/m 2/m 2/b",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "345": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y,-z+1/2)",
                "(-x,-y,z+1/2)",
                "(-x,y,-z)",
                "(-x,-y,-z)",
                "(-x,y,z+1/2)",
                "(x,y,-z+1/2)",
                "(x,-y,z)"
            ],
            [
                "(x,y,1/4)",
                "(x,-y,1/4)",
                "(-x,-y,3/4)",
                "(-x,y,3/4)"
            ],
            [
                "(x,0,z)",
                "(x,0,-z+1/2)",
                "(-x,0,z+1/2)",
                "(-x,0,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,3/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(3/4,1/4,z+1/2)"
            ],
            [
                "(0,y,0)",
                "(0,-y,1/2)",
                "(0,-y,0)",
                "(0,y,1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,3/4)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            ".2.",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "I 2/b 2/m 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "346": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(x,-y+1/2,-z)",
                "(-x,y+1/2,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(-x,y+1/2,z)",
                "(x,-y+1/2,z)",
                "(x,y,-z)"
            ],
            [
                "(x,1/4,z)",
                "(x,1/4,-z)",
                "(-x,3/4,-z)",
                "(-x,3/4,z)"
            ],
            [
                "(x,y,0)",
                "(x,-y+1/2,0)",
                "(-x,y+1/2,0)",
                "(-x,-y,0)"
            ],
            [
                "(3/4,y,1/4)",
                "(3/4,-y+1/2,3/4)",
                "(1/4,-y,3/4)",
                "(1/4,y+1/2,1/4)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,-z)",
                "(0,0,-z)",
                "(0,1/2,z)"
            ],
            [
                "(x,1/4,0)",
                "(-x,3/4,0)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            ".2.",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "I 2/c 2/m 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "347": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,y,-z)",
                "(x+1/2,-y,-z)",
                "(-x,-y,z)",
                "(-x,-y,-z)",
                "(x+1/2,-y,z)",
                "(-x+1/2,y,z)",
                "(x,y,-z)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,y,-z)",
                "(3/4,-y,-z)",
                "(3/4,-y,z)"
            ],
            [
                "(x,y,0)",
                "(-x+1/2,y,0)",
                "(x+1/2,-y,0)",
                "(-x,-y,0)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,1/4,3/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,3/4,1/4)"
            ],
            [
                "(0,0,z)",
                "(1/2,0,-z)",
                "(0,0,-z)",
                "(1/2,0,z)"
            ],
            [
                "(1/4,y,0)",
                "(3/4,-y,0)"
            ],
            [
                "(1/4,3/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            ".2.",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "I 2/m 2/c 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "348": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,y,-z+1/2)",
                "(-x,-y,z+1/2)",
                "(x,-y,-z)",
                "(-x,-y,-z)",
                "(x,-y,z+1/2)",
                "(x,y,-z+1/2)",
                "(-x,y,z)"
            ],
            [
                "(x,y,1/4)",
                "(-x,y,1/4)",
                "(-x,-y,3/4)",
                "(x,-y,3/4)"
            ],
            [
                "(0,y,z)",
                "(0,y,-z+1/2)",
                "(0,-y,z+1/2)",
                "(0,-y,-z)"
            ],
            [
                "(1/4,3/4,z)",
                "(3/4,3/4,-z+1/2)",
                "(3/4,1/4,-z)",
                "(1/4,1/4,z+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,1/2)",
                "(-x,0,0)",
                "(x,0,1/2)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,3/4,1/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            ".2.",
            "2..",
            "mm2",
            ".2/m.",
            ".2/m.",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "I 2/m 2/a 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "349": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)"
            ],
            [
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "4..",
            "4.."
        ],
        "Space Group": "P 4  ",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "350": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-y,x,z+1/4)",
                "(y,-x,z+3/4)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 4sub1  ",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "351": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "2.."
        ],
        "Space Group": "P 4sub2  ",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "352": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-y,x,z+3/4)",
                "(y,-x,z+1/4)"
            ]
        ],
        "Multiplicity": [
            "4"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 4sub3  ",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "353": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "4.."
        ],
        "Space Group": "I 4  ",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "354": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-y,x+1/2,z+1/4)",
                "(y+1/2,-x,z+3/4)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z+1/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2.."
        ],
        "Space Group": "I 4sub1  ",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "355": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "4",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "2..",
            "-4..",
            "-4..",
            "-4..",
            "-4.."
        ],
        "Space Group": "P -4  ",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "356": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,3/4)"
            ],
            [
                "(0,1/2,1/4)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "-4..",
            "-4..",
            "-4..",
            "-4.."
        ],
        "Space Group": "I -4  ",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "357": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,-y,1/2)",
                "(-y,x,1/2)",
                "(y,-x,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,0)",
                "(y,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            "2..",
            "4..",
            "4..",
            "2/m..",
            "2/m..",
            "4/m..",
            "4/m..",
            "4/m..",
            "4/m.."
        ],
        "Space Group": "P 4/m  ",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "358": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z+1/2)",
                "(-y,x,-z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,1/2)",
                "(y,-x,1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)",
                "(0,1/2,-z)",
                "(1/2,0,-z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(0,0,-z)",
                "(0,0,-z+1/2)"
            ],
            [
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "2..",
            "2..",
            "2..",
            "-4..",
            "-4..",
            "2/m..",
            "2/m..",
            "2/m..",
            "2/m.."
        ],
        "Space Group": "P 4sub2/m  ",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "359": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z)",
                "(y+1/2,-x+1/2,z)",
                "(-x+1/2,-y+1/2,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)",
                "(0,0,-z)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,3/4,1/2)",
                "(1/4,3/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)",
                "(1/4,3/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "-1",
            "-1",
            "4..",
            "-4..",
            "-4.."
        ],
        "Space Group": "P 4/n  ",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "360": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z)",
                "(y,-x+1/2,z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z)",
                "(-y,x+1/2,-z)"
            ],
            [
                "(1/4,3/4,z)",
                "(3/4,1/4,z)",
                "(3/4,1/4,-z)",
                "(1/4,3/4,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z)"
            ],
            [
                "(1/4,3/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,3/4,0)",
                "(3/4,1/4,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "-1",
            "-1",
            "4..",
            "-4..",
            "-4.."
        ],
        "Space Group": "P 4/n  ",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "361": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(y,-x,-z)",
                "(-y,x,-z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,-z)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,3/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "-1",
            "-1",
            "-4..",
            "-4.."
        ],
        "Space Group": "P 4sub2/n  ",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "362": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y,x+1/2,z+1/2)",
                "(y+1/2,-x,z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y,-x+1/2,-z+1/2)",
                "(-y+1/2,x,-z+1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,z+1/2)",
                "(3/4,3/4,-z)",
                "(1/4,1/4,-z+1/2)"
            ],
            [
                "(3/4,1/4,z)",
                "(3/4,1/4,z+1/2)",
                "(1/4,3/4,-z)",
                "(1/4,3/4,-z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "-1",
            "-1",
            "-4..",
            "-4.."
        ],
        "Space Group": "P 4sub2/n  ",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "363": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,0)",
                "(y,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "2..",
            "-1",
            "4..",
            "-4..",
            "2/m..",
            "4/m..",
            "4/m.."
        ],
        "Space Group": "I 4/m  ",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "364": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-y,x+1/2,z+1/4)",
                "(y+1/2,-x,z+3/4)",
                "(-x,-y+1/2,-z+1/4)",
                "(x+1/2,y,-z+3/4)",
                "(y,-x,-z)",
                "(-y+1/2,x+1/2,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z+1/4)",
                "(0,1/2,-z+1/4)",
                "(0,0,-z)"
            ],
            [
                "(0,1/4,5/8)",
                "(1/2,1/4,1/8)",
                "(3/4,1/2,7/8)",
                "(3/4,0,3/8)"
            ],
            [
                "(0,1/4,1/8)",
                "(1/2,1/4,5/8)",
                "(3/4,1/2,3/8)",
                "(3/4,0,7/8)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "-1",
            "-1",
            "-4..",
            "-4.."
        ],
        "Space Group": "I 4sub1/a  ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "365": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-y+3/4,x+1/4,z+1/4)",
                "(y+3/4,-x+3/4,z+3/4)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(y+1/4,-x+3/4,-z+3/4)",
                "(-y+1/4,x+1/4,-z+1/4)"
            ],
            [
                "(0,1/4,z)",
                "(1/2,1/4,z+1/4)",
                "(0,3/4,-z)",
                "(1/2,3/4,-z+3/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)",
                "(3/4,1/4,3/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(3/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(0,1/4,5/8)",
                "(1/2,1/4,7/8)"
            ],
            [
                "(0,1/4,1/8)",
                "(1/2,1/4,3/8)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "-1",
            "-1",
            "-4..",
            "-4.."
        ],
        "Space Group": "I 4sub1/a  ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "366": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y,x,-z)",
                "(-y,-x,-z)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(1/2,x,0)",
                "(1/2,-x,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,x,1/2)",
                "(0,-x,1/2)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)"
            ],
            [
                "(x,x,1/2)",
                "(-x,-x,1/2)",
                "(-x,x,1/2)",
                "(x,-x,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x,x,0)",
                "(x,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2.",
            ".2.",
            ".2.",
            "..2",
            "..2",
            "2..",
            "4..",
            "4..",
            "222.",
            "222.",
            "422",
            "422",
            "422",
            "422"
        ],
        "Space Group": "P 4 2 2",
        "Wyckoff Letter": [
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "367": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z)",
                "(y+1/2,-x+1/2,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(y,x,-z)",
                "(-y,-x,-z)"
            ],
            [
                "(x,x,1/2)",
                "(-x,-x,1/2)",
                "(-x+1/2,x+1/2,1/2)",
                "(x+1/2,-x+1/2,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x+1/2,x+1/2,0)",
                "(x+1/2,-x+1/2,0)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "2..",
            "4..",
            "2.22",
            "2.22"
        ],
        "Space Group": "P 4 2sub1 2",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "368": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-y,x,z+1/4)",
                "(y,-x,z+3/4)",
                "(-x,y,-z)",
                "(x,-y,-z+1/2)",
                "(y,x,-z+3/4)",
                "(-y,-x,-z+1/4)"
            ],
            [
                "(x,x,3/8)",
                "(-x,-x,7/8)",
                "(-x,x,5/8)",
                "(x,-x,1/8)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,1/2)",
                "(-y,1/2,1/4)",
                "(y,1/2,3/4)"
            ],
            [
                "(0,y,0)",
                "(0,-y,1/2)",
                "(-y,0,1/4)",
                "(y,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            ".2."
        ],
        "Space Group": "P 4sub1 2 2",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "369": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-y+1/2,x+1/2,z+1/4)",
                "(y+1/2,-x+1/2,z+3/4)",
                "(-x+1/2,y+1/2,-z+1/4)",
                "(x+1/2,-y+1/2,-z+3/4)",
                "(y,x,-z)",
                "(-y,-x,-z+1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,1/2)",
                "(-x+1/2,x+1/2,1/4)",
                "(x+1/2,-x+1/2,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "P 4sub1 2sub1 2",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "370": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y,x,-z+1/2)",
                "(-y,-x,-z+1/2)"
            ],
            [
                "(x,x,3/4)",
                "(-x,-x,3/4)",
                "(-x,x,1/4)",
                "(x,-x,1/4)"
            ],
            [
                "(x,x,1/4)",
                "(-x,-x,1/4)",
                "(-x,x,3/4)",
                "(x,-x,3/4)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,x,0)",
                "(0,-x,0)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)",
                "(0,1/2,-z)",
                "(1/2,0,-z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(0,0,-z)",
                "(0,0,-z+1/2)"
            ],
            [
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "2..",
            "2.22",
            "2.22",
            "222.",
            "222.",
            "222.",
            "222."
        ],
        "Space Group": "P 4sub2 2 2",
        "Wyckoff Letter": [
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "371": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(y,x,-z)",
                "(-y,-x,-z)"
            ],
            [
                "(x,x,1/2)",
                "(-x,-x,1/2)",
                "(-x+1/2,x+1/2,0)",
                "(x+1/2,-x+1/2,0)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x+1/2,x+1/2,1/2)",
                "(x+1/2,-x+1/2,1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "2..",
            "2..",
            "2.22",
            "2.22"
        ],
        "Space Group": "P 4sub2 2sub1 2",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "372": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-y,x,z+3/4)",
                "(y,-x,z+1/4)",
                "(-x,y,-z)",
                "(x,-y,-z+1/2)",
                "(y,x,-z+1/4)",
                "(-y,-x,-z+3/4)"
            ],
            [
                "(x,x,5/8)",
                "(-x,-x,1/8)",
                "(-x,x,3/8)",
                "(x,-x,7/8)"
            ],
            [
                "(1/2,y,0)",
                "(1/2,-y,1/2)",
                "(-y,1/2,3/4)",
                "(y,1/2,1/4)"
            ],
            [
                "(0,y,0)",
                "(0,-y,1/2)",
                "(-y,0,3/4)",
                "(y,0,1/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            ".2."
        ],
        "Space Group": "P 4sub3 2 2",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "373": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z+1/2)",
                "(-y+1/2,x+1/2,z+3/4)",
                "(y+1/2,-x+1/2,z+1/4)",
                "(-x+1/2,y+1/2,-z+3/4)",
                "(x+1/2,-y+1/2,-z+1/4)",
                "(y,x,-z)",
                "(-y,-x,-z+1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,1/2)",
                "(-x+1/2,x+1/2,3/4)",
                "(x+1/2,-x+1/2,1/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2"
        ],
        "Space Group": "P 4sub3 2sub1 2",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "374": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y,x,-z)",
                "(-y,-x,-z)"
            ],
            [
                "(x,x+1/2,1/4)",
                "(-x,-x+1/2,1/4)",
                "(-x+1/2,x,1/4)",
                "(x+1/2,-x,1/4)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,x,1/2)",
                "(0,-x,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x,x,0)",
                "(x,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            ".2.",
            "..2",
            "2..",
            "4..",
            "2.22",
            "222.",
            "422",
            "422"
        ],
        "Space Group": "I 4 2 2",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "375": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-y,x+1/2,z+1/4)",
                "(y+1/2,-x,z+3/4)",
                "(-x+1/2,y,-z+3/4)",
                "(x,-y+1/2,-z+1/4)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y,-x,-z)"
            ],
            [
                "(x,1/4,1/8)",
                "(-x+1/2,1/4,5/8)",
                "(3/4,x+1/2,3/8)",
                "(3/4,-x,7/8)"
            ],
            [
                "(x,-x,0)",
                "(-x+1/2,x+1/2,1/2)",
                "(x,x+1/2,1/4)",
                "(-x+1/2,-x,3/4)"
            ],
            [
                "(x,x,0)",
                "(-x+1/2,-x+1/2,1/2)",
                "(-x,x+1/2,1/4)",
                "(x+1/2,-x,3/4)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z+1/4)",
                "(1/2,0,-z+3/4)",
                "(1/2,1/2,-z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "..2",
            "..2",
            "2..",
            "2.22",
            "2.22"
        ],
        "Space Group": "I 4sub1 2 2",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "376": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,z)",
                "(1/2,x,z)",
                "(1/2,-x,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(0,x,z)",
                "(0,-x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,z)",
                "(x,-x,z)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)"
            ],
            [
                "(1/2,1/2,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            ".m.",
            "..m",
            "2mm.",
            "4mm",
            "4mm"
        ],
        "Space Group": "P 4 m m",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "377": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(-y+1/2,-x+1/2,z)",
                "(y+1/2,x+1/2,z)"
            ],
            [
                "(x,x+1/2,z)",
                "(-x,-x+1/2,z)",
                "(-x+1/2,x,z)",
                "(x+1/2,-x,z)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2.mm",
            "4.."
        ],
        "Space Group": "P 4 b m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "378": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,z+1/2)",
                "(x,-x,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)",
                "(0,1/2,z+1/2)",
                "(1/2,0,z)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "2.mm",
            "2.mm"
        ],
        "Space Group": "P 4sub2 c m",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "379": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x+1/2,x+1/2,z+1/2)",
                "(x+1/2,-x+1/2,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,z+1/2)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "2.mm"
        ],
        "Space Group": "P 4sub2 n m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "380": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(-y,-x,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "4..",
            "4.."
        ],
        "Space Group": "P 4 c c",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "381": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(1/2,0,z+1/2)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "4.."
        ],
        "Space Group": "P 4 n c",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "382": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-y,-x,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,z)",
                "(1/2,x,z+1/2)",
                "(1/2,-x,z+1/2)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(0,x,z+1/2)",
                "(0,-x,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            ".m.",
            "2mm.",
            "2mm.",
            "2mm."
        ],
        "Space Group": "P 4sub2 m c",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "383": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)",
                "(1/2,0,z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2.."
        ],
        "Space Group": "P 4sub2 b c",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "384": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(0,x,z)",
                "(0,-x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,z)",
                "(x,-x,z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..m",
            "2mm.",
            "4mm"
        ],
        "Space Group": "I 4 m m",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "385": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(-y,-x,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(x,x+1/2,z)",
                "(-x,-x+1/2,z)",
                "(-x+1/2,x,z)",
                "(x+1/2,-x,z)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2.mm",
            "4.."
        ],
        "Space Group": "I 4 c m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "386": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-y,x+1/2,z+1/4)",
                "(y+1/2,-x,z+3/4)",
                "(x,-y,z)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-y,-x+1/2,z+1/4)",
                "(y+1/2,x,z+3/4)"
            ],
            [
                "(0,y,z)",
                "(1/2,-y+1/2,z+1/2)",
                "(-y,1/2,z+1/4)",
                "(y+1/2,0,z+3/4)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z+1/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "2mm."
        ],
        "Space Group": "I 4sub1 m d",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "387": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-y,x+1/2,z+1/4)",
                "(y+1/2,-x,z+3/4)",
                "(x,-y,z+1/2)",
                "(-x+1/2,y+1/2,z)",
                "(-y,-x+1/2,z+3/4)",
                "(y+1/2,x,z+1/4)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z+1/4)",
                "(0,0,z+1/2)",
                "(0,1/2,z+3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "2.."
        ],
        "Space Group": "I 4sub1 c d",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "388": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(x,-x,-z)",
                "(-x,x,-z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(1/2,0,z)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(1/2,-x,0)",
                "(1/2,x,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,-x,1/2)",
                "(0,x,1/2)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,-x,1/2)",
                "(1/2,x,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,-x,0)",
                "(0,x,0)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            ".2.",
            ".2.",
            ".2.",
            ".2.",
            "2.mm",
            "2.mm",
            "222.",
            "222.",
            "-42m",
            "-42m",
            "-42m",
            "-42m"
        ],
        "Space Group": "P -4 2 m",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "389": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z+1/2)",
                "(-y,-x,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(0,0,-z+1/2)",
                "(0,0,z+1/2)"
            ],
            [
                "(0,y,1/4)",
                "(0,-y,1/4)",
                "(y,0,3/4)",
                "(-y,0,3/4)"
            ],
            [
                "(x,1/2,1/4)",
                "(-x,1/2,1/4)",
                "(1/2,-x,3/4)",
                "(1/2,x,3/4)"
            ],
            [
                "(1/2,y,1/4)",
                "(1/2,-y,1/4)",
                "(y,1/2,3/4)",
                "(-y,1/2,3/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,1/4)",
                "(0,-x,3/4)",
                "(0,x,3/4)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(1/2,0,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "2..",
            ".2.",
            ".2.",
            ".2.",
            ".2.",
            "-4..",
            "-4..",
            "222.",
            "222.",
            "222.",
            "222."
        ],
        "Space Group": "P -4 2 c",
        "Wyckoff Letter": [
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "390": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(-y+1/2,-x+1/2,z)",
                "(y+1/2,x+1/2,z)"
            ],
            [
                "(x,x+1/2,z)",
                "(-x,-x+1/2,z)",
                "(x+1/2,-x,-z)",
                "(-x+1/2,x,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "2.mm",
            "-4..",
            "-4.."
        ],
        "Space Group": "P -4 2sub1 m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "391": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(1/2,0,-z+1/2)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "-4..",
            "-4.."
        ],
        "Space Group": "P -4 2sub1 c",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "392": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(y,x,-z)",
                "(-y,-x,-z)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,z)",
                "(1/2,-x,-z)",
                "(1/2,x,-z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(0,-x,-z)",
                "(0,x,-z)"
            ],
            [
                "(x,x,1/2)",
                "(-x,-x,1/2)",
                "(x,-x,1/2)",
                "(-x,x,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(x,-x,0)",
                "(-x,x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            ".m.",
            "..2",
            "..2",
            "2mm.",
            "2mm.",
            "2mm.",
            "-4m2",
            "-4m2",
            "-4m2",
            "-4m2"
        ],
        "Space Group": "P -4 m 2",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "393": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(y,x,-z+1/2)",
                "(-y,-x,-z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,-z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(0,0,z+1/2)",
                "(0,0,-z+1/2)"
            ],
            [
                "(x,x,3/4)",
                "(-x,-x,3/4)",
                "(x,-x,1/4)",
                "(-x,x,1/4)"
            ],
            [
                "(x,x,1/4)",
                "(-x,-x,1/4)",
                "(x,-x,3/4)",
                "(-x,x,3/4)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "2..",
            "..2",
            "..2",
            "-4..",
            "-4..",
            "2.22",
            "2.22"
        ],
        "Space Group": "P -4 c 2",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "394": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(y+1/2,x+1/2,-z)",
                "(-y+1/2,-x+1/2,-z)"
            ],
            [
                "(x,x+1/2,1/2)",
                "(-x,-x+1/2,1/2)",
                "(x+1/2,-x,1/2)",
                "(-x+1/2,x,1/2)"
            ],
            [
                "(x,x+1/2,0)",
                "(-x,-x+1/2,0)",
                "(x+1/2,-x,0)",
                "(-x+1/2,x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(1/2,0,z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "2..",
            "2..",
            "2.22",
            "2.22",
            "-4..",
            "-4.."
        ],
        "Space Group": "P -4 b 2",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "395": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(1/2,0,z+1/2)",
                "(0,1/2,-z+1/2)"
            ],
            [
                "(x,x+1/2,1/4)",
                "(-x,-x+1/2,1/4)",
                "(x+1/2,-x,3/4)",
                "(-x+1/2,x,3/4)"
            ],
            [
                "(x,-x+1/2,1/4)",
                "(-x,x+1/2,1/4)",
                "(-x+1/2,-x,3/4)",
                "(x+1/2,x,3/4)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z+1/2)"
            ],
            [
                "(0,1/2,3/4)",
                "(1/2,0,1/4)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "..2",
            "..2",
            "2..",
            "2.22",
            "2.22",
            "-4..",
            "-4.."
        ],
        "Space Group": "P -4 n 2",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "396": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(y,x,-z)",
                "(-y,-x,-z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(0,-x,-z)",
                "(0,x,-z)"
            ],
            [
                "(x,x+1/2,1/4)",
                "(-x,-x+1/2,1/4)",
                "(x+1/2,-x,3/4)",
                "(-x+1/2,x,3/4)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(x,-x,0)",
                "(-x,x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,3/4)"
            ],
            [
                "(0,1/2,1/4)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "..2",
            "2mm.",
            "2mm.",
            "-4m2",
            "-4m2",
            "-4m2",
            "-4m2"
        ],
        "Space Group": "I -4 m 2",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "397": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(y,x,-z+1/2)",
                "(-y,-x,-z+1/2)"
            ],
            [
                "(x,x+1/2,0)",
                "(-x,-x+1/2,0)",
                "(x+1/2,-x,0)",
                "(-x+1/2,x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(0,0,z+1/2)",
                "(0,0,-z+1/2)"
            ],
            [
                "(x,x,1/4)",
                "(-x,-x,1/4)",
                "(x,-x,3/4)",
                "(-x,x,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            "2..",
            "..2",
            "2.22",
            "-4..",
            "-4..",
            "2.22"
        ],
        "Space Group": "I -4 c 2",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "398": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(x,-x,-z)",
                "(-x,x,-z)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(1/2,0,z)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,-x,1/2)",
                "(0,x,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,-x,0)",
                "(0,x,0)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            ".2.",
            ".2.",
            "2.mm",
            "-4..",
            "222.",
            "-42m",
            "-42m"
        ],
        "Space Group": "I -4 2 m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "399": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(-x+1/2,y,-z+3/4)",
                "(x+1/2,-y,-z+3/4)",
                "(-y+1/2,-x,z+3/4)",
                "(y+1/2,x,z+3/4)"
            ],
            [
                "(x,1/4,1/8)",
                "(-x,3/4,1/8)",
                "(1/4,-x,7/8)",
                "(3/4,x,7/8)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,0,-z+3/4)",
                "(1/2,0,z+3/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,1/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "2..",
            "-4..",
            "-4.."
        ],
        "Space Group": "I -4 2 d",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "400": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,1/2,z)",
                "(-x,1/2,z)",
                "(1/2,x,z)",
                "(1/2,-x,z)",
                "(-x,1/2,-z)",
                "(x,1/2,-z)",
                "(1/2,x,-z)",
                "(1/2,-x,-z)"
            ],
            [
                "(x,0,z)",
                "(-x,0,z)",
                "(0,x,z)",
                "(0,-x,z)",
                "(-x,0,-z)",
                "(x,0,-z)",
                "(0,x,-z)",
                "(0,-x,-z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,z)",
                "(x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(x,x,-z)",
                "(-x,-x,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,-y,1/2)",
                "(-y,x,1/2)",
                "(y,-x,1/2)",
                "(-x,y,1/2)",
                "(x,-y,1/2)",
                "(y,x,1/2)",
                "(-y,-x,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,0)",
                "(y,-x,0)",
                "(-x,y,0)",
                "(x,-y,0)",
                "(y,x,0)",
                "(-y,-x,0)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(1/2,x,0)",
                "(1/2,-x,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,x,1/2)",
                "(0,-x,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)"
            ],
            [
                "(x,x,1/2)",
                "(-x,-x,1/2)",
                "(-x,x,1/2)",
                "(x,-x,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x,x,0)",
                "(x,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            ".m.",
            "..m",
            "m..",
            "m..",
            "m2m.",
            "m2m.",
            "m2m.",
            "m2m.",
            "m.2m",
            "m.2m",
            "2mm.",
            "4mm",
            "4mm",
            "mmm.",
            "mmm.",
            "4/mmm",
            "4/mmm",
            "4/mmm",
            "4/mmm"
        ],
        "Space Group": "P 4/m 2/m 2/m",
        "Wyckoff Letter": [
            "u",
            "t",
            "s",
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "401": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z+1/2)",
                "(y,x,-z+1/2)",
                "(-y,-x,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(-y,-x,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,0)",
                "(y,-x,0)",
                "(-x,y,1/2)",
                "(x,-y,1/2)",
                "(y,x,1/2)",
                "(-y,-x,1/2)"
            ],
            [
                "(x,1/2,1/4)",
                "(-x,1/2,1/4)",
                "(1/2,x,1/4)",
                "(1/2,-x,1/4)",
                "(-x,1/2,3/4)",
                "(x,1/2,3/4)",
                "(1/2,-x,3/4)",
                "(1/2,x,3/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,1/4)",
                "(0,x,1/4)",
                "(0,-x,1/4)",
                "(-x,0,3/4)",
                "(x,0,3/4)",
                "(0,-x,3/4)",
                "(0,x,3/4)"
            ],
            [
                "(x,x,1/4)",
                "(-x,-x,1/4)",
                "(-x,x,1/4)",
                "(x,-x,1/4)",
                "(-x,-x,3/4)",
                "(x,x,3/4)",
                "(x,-x,3/4)",
                "(-x,x,3/4)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,-z+1/2)",
                "(0,1/2,-z)",
                "(1/2,0,-z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,1/4)",
                "(0,1/2,3/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            ".2.",
            "..2",
            "2..",
            "4..",
            "4..",
            "222.",
            "2/m..",
            "4/m..",
            "422",
            "4/m..",
            "422"
        ],
        "Space Group": "P 4/m 2/c 2/c",
        "Wyckoff Letter": [
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "402": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(-x+1/2,-y+1/2,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x+1/2,-z)",
                "(-y+1/2,x+1/2,-z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(-y+1/2,-x+1/2,z)",
                "(y+1/2,x+1/2,z)"
            ],
            [
                "(x,x+1/2,z)",
                "(-x,-x+1/2,z)",
                "(-x+1/2,x,z)",
                "(x+1/2,-x,z)",
                "(-x,x+1/2,-z)",
                "(x,-x+1/2,-z)",
                "(x+1/2,x,-z)",
                "(-x+1/2,-x,-z)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(1/2,x+1/2,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(-x+1/2,1/2,0)",
                "(x+1/2,1/2,0)",
                "(1/2,-x+1/2,0)",
                "(1/2,x+1/2,0)"
            ],
            [
                "(x,x,1/2)",
                "(-x,-x,1/2)",
                "(-x,x,1/2)",
                "(x,-x,1/2)",
                "(-x+1/2,-x+1/2,1/2)",
                "(x+1/2,x+1/2,1/2)",
                "(x+1/2,-x+1/2,1/2)",
                "(-x+1/2,x+1/2,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x,x,0)",
                "(x,-x,0)",
                "(-x+1/2,-x+1/2,0)",
                "(x+1/2,x+1/2,0)",
                "(x+1/2,-x+1/2,0)",
                "(-x+1/2,x+1/2,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,3/4,1/2)",
                "(3/4,1/4,1/2)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)",
                "(3/4,1/4,0)",
                "(1/4,3/4,0)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            ".2.",
            ".2.",
            "..2",
            "..2",
            "2.mm",
            "4..",
            "..2/m",
            "..2/m",
            "-42m",
            "-42m",
            "422",
            "422"
        ],
        "Space Group": "P 4/n 2/b 2/m",
        "Wyckoff Letter": [
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "403": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z)",
                "(y,-x+1/2,z)",
                "(-x+1/2,y,-z)",
                "(x,-y+1/2,-z)",
                "(y,x,-z)",
                "(-y+1/2,-x+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z)",
                "(-y,x+1/2,-z)",
                "(x+1/2,-y,z)",
                "(-x,y+1/2,z)",
                "(-y,-x,z)",
                "(y+1/2,x+1/2,z)"
            ],
            [
                "(x,-x,z)",
                "(-x+1/2,x+1/2,z)",
                "(x+1/2,x,z)",
                "(-x,-x+1/2,z)",
                "(-x+1/2,-x,-z)",
                "(x,x+1/2,-z)",
                "(-x,x,-z)",
                "(x+1/2,-x+1/2,-z)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x+1/2,1/4,1/2)",
                "(1/4,x,1/2)",
                "(1/4,-x+1/2,1/2)",
                "(-x,3/4,1/2)",
                "(x+1/2,3/4,1/2)",
                "(3/4,-x,1/2)",
                "(3/4,x+1/2,1/2)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,1/4,0)",
                "(1/4,x,0)",
                "(1/4,-x+1/2,0)",
                "(-x,3/4,0)",
                "(x+1/2,3/4,0)",
                "(3/4,-x,0)",
                "(3/4,x+1/2,0)"
            ],
            [
                "(x,x,1/2)",
                "(-x+1/2,-x+1/2,1/2)",
                "(-x+1/2,x,1/2)",
                "(x,-x+1/2,1/2)",
                "(-x,-x,1/2)",
                "(x+1/2,x+1/2,1/2)",
                "(x+1/2,-x,1/2)",
                "(-x,x+1/2,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x+1/2,-x+1/2,0)",
                "(-x+1/2,x,0)",
                "(x,-x+1/2,0)",
                "(-x,-x,0)",
                "(x+1/2,x+1/2,0)",
                "(x+1/2,-x,0)",
                "(-x,x+1/2,0)"
            ],
            [
                "(3/4,1/4,z)",
                "(1/4,3/4,z)",
                "(3/4,1/4,-z)",
                "(1/4,3/4,-z)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,1/4,-z)",
                "(3/4,3/4,-z)",
                "(3/4,3/4,z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(3/4,1/4,1/2)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(3/4,1/4,0)",
                "(1/4,3/4,0)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,3/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            ".2.",
            ".2.",
            "..2",
            "..2",
            "2.mm",
            "4..",
            "..2/m",
            "..2/m",
            "-42m",
            "-42m",
            "422",
            "422"
        ],
        "Space Group": "P 4/n 2/b 2/m",
        "Wyckoff Letter": [
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "404": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,-z+1/2)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(-x+1/2,1/2,0)",
                "(x+1/2,1/2,0)",
                "(1/2,-x+1/2,0)",
                "(1/2,x+1/2,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(-x+1/2,1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(1/2,x+1/2,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x,x,0)",
                "(x,-x,0)",
                "(-x+1/2,-x+1/2,1/2)",
                "(x+1/2,x+1/2,1/2)",
                "(x+1/2,-x+1/2,1/2)",
                "(-x+1/2,x+1/2,1/2)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,-z+1/2)",
                "(0,1/2,z+1/2)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(1/2,0,1/4)",
                "(0,1/2,1/4)",
                "(1/2,0,3/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2.",
            "..2",
            "2..",
            "-1",
            "4..",
            "-4..",
            "222.",
            "422",
            "422"
        ],
        "Space Group": "P 4/n 2/n 2/c",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "405": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z)",
                "(y,-x+1/2,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(y,x,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z)",
                "(-y,x+1/2,-z)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(-y,-x,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(x,3/4,1/4)",
                "(-x+1/2,3/4,1/4)",
                "(3/4,x,1/4)",
                "(3/4,-x+1/2,1/4)",
                "(-x,1/4,3/4)",
                "(x+1/2,1/4,3/4)",
                "(1/4,-x,3/4)",
                "(1/4,x+1/2,3/4)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,1/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x+1/2,1/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,3/4,3/4)",
                "(3/4,-x,3/4)",
                "(3/4,x+1/2,3/4)"
            ],
            [
                "(x,x,1/4)",
                "(-x+1/2,-x+1/2,1/4)",
                "(-x+1/2,x,1/4)",
                "(x,-x+1/2,1/4)",
                "(-x,-x,3/4)",
                "(x+1/2,x+1/2,3/4)",
                "(x+1/2,-x,3/4)",
                "(-x,x+1/2,3/4)"
            ],
            [
                "(1/4,3/4,z)",
                "(3/4,1/4,z)",
                "(1/4,3/4,-z+1/2)",
                "(3/4,1/4,-z+1/2)",
                "(3/4,1/4,-z)",
                "(1/4,3/4,-z)",
                "(3/4,1/4,z+1/2)",
                "(1/4,3/4,z+1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,1/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(3/4,3/4,z+1/2)"
            ],
            [
                "(1/4,3/4,0)",
                "(3/4,1/4,0)",
                "(1/4,3/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2.",
            "..2",
            "2..",
            "-1",
            "4..",
            "-4..",
            "222.",
            "422",
            "422"
        ],
        "Space Group": "P 4/n 2/n 2/c",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "406": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(y+1/2,x+1/2,-z)",
                "(-y+1/2,-x+1/2,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(-y+1/2,-x+1/2,z)",
                "(y+1/2,x+1/2,z)"
            ],
            [
                "(x,x+1/2,z)",
                "(-x,-x+1/2,z)",
                "(-x+1/2,x,z)",
                "(x+1/2,-x,z)",
                "(-x+1/2,x,-z)",
                "(x+1/2,-x,-z)",
                "(x,x+1/2,-z)",
                "(-x,-x+1/2,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-x,-y,1/2)",
                "(-y,x,1/2)",
                "(y,-x,1/2)",
                "(-x+1/2,y+1/2,1/2)",
                "(x+1/2,-y+1/2,1/2)",
                "(y+1/2,x+1/2,1/2)",
                "(-y+1/2,-x+1/2,1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,0)",
                "(y,-x,0)",
                "(-x+1/2,y+1/2,0)",
                "(x+1/2,-y+1/2,0)",
                "(y+1/2,x+1/2,0)",
                "(-y+1/2,-x+1/2,0)"
            ],
            [
                "(x,x+1/2,1/2)",
                "(-x,-x+1/2,1/2)",
                "(-x+1/2,x,1/2)",
                "(x+1/2,-x,1/2)"
            ],
            [
                "(x,x+1/2,0)",
                "(-x,-x+1/2,0)",
                "(-x+1/2,x,0)",
                "(x+1/2,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(1/2,0,-z)",
                "(0,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z)",
                "(0,0,-z)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "m..",
            "m.2m",
            "m.2m",
            "2.mm",
            "4..",
            "m.mm",
            "m.mm",
            "4/m..",
            "4/m.."
        ],
        "Space Group": "P 4/m 2sub1/b m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "407": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,0)",
                "(y,-x,0)",
                "(-x+1/2,y+1/2,1/2)",
                "(x+1/2,-y+1/2,1/2)",
                "(y+1/2,x+1/2,1/2)",
                "(-y+1/2,-x+1/2,1/2)"
            ],
            [
                "(x,x+1/2,1/4)",
                "(-x,-x+1/2,1/4)",
                "(-x+1/2,x,1/4)",
                "(x+1/2,-x,1/4)",
                "(-x,-x+1/2,3/4)",
                "(x,x+1/2,3/4)",
                "(x+1/2,-x,3/4)",
                "(-x+1/2,x,3/4)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(1/2,0,-z+1/2)",
                "(0,1/2,-z+1/2)",
                "(0,1/2,-z)",
                "(1/2,0,-z)",
                "(1/2,0,z+1/2)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,1/4)",
                "(0,1/2,3/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "2..",
            "4..",
            "2.22",
            "2/m..",
            "4/m..",
            "4/m.."
        ],
        "Space Group": "P 4/m 2sub1/n c",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "408": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z)",
                "(y+1/2,-x+1/2,z)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(-x+1/2,-y+1/2,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-y+1/2,-x+1/2,z)",
                "(y+1/2,x+1/2,z)"
            ],
            [
                "(x,x+1/2,z)",
                "(-x,-x+1/2,z)",
                "(-x,x+1/2,z)",
                "(x,-x+1/2,z)",
                "(-x+1/2,x,-z)",
                "(x+1/2,-x,-z)",
                "(x+1/2,x,-z)",
                "(-x+1/2,-x,-z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(-y+1/2,1/2,z)",
                "(y+1/2,1/2,z)",
                "(1/2,y+1/2,-z)",
                "(1/2,-y+1/2,-z)",
                "(y,0,-z)",
                "(-y,0,-z)"
            ],
            [
                "(x,x,1/2)",
                "(-x,-x,1/2)",
                "(-x+1/2,x+1/2,1/2)",
                "(x+1/2,-x+1/2,1/2)",
                "(-x+1/2,-x+1/2,1/2)",
                "(x+1/2,x+1/2,1/2)",
                "(x,-x,1/2)",
                "(-x,x,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x+1/2,x+1/2,0)",
                "(x+1/2,-x+1/2,0)",
                "(-x+1/2,-x+1/2,0)",
                "(x+1/2,x+1/2,0)",
                "(x,-x,0)",
                "(-x,x,0)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)",
                "(1/2,1/2,-z)",
                "(0,0,-z)"
            ],
            [
                "(1/4,1/4,1/2)",
                "(3/4,3/4,1/2)",
                "(1/4,3/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)",
                "(1/4,3/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            ".m.",
            "..2",
            "..2",
            "2mm.",
            "..2/m",
            "..2/m",
            "4mm",
            "-4m2",
            "-4m2"
        ],
        "Space Group": "P 4/n 2sub1/m m",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "409": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z)",
                "(y,-x+1/2,z)",
                "(-x,y+1/2,-z)",
                "(x+1/2,-y,-z)",
                "(y+1/2,x+1/2,-z)",
                "(-y,-x,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z)",
                "(-y,x+1/2,-z)",
                "(x,-y+1/2,z)",
                "(-x+1/2,y,z)",
                "(-y+1/2,-x+1/2,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x+1/2,-x+1/2,z)",
                "(-x+1/2,x,z)",
                "(x,-x+1/2,z)",
                "(-x,x+1/2,-z)",
                "(x+1/2,-x,-z)",
                "(x+1/2,x+1/2,-z)",
                "(-x,-x,-z)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,-y+1/2,z)",
                "(-y+1/2,1/4,z)",
                "(y,1/4,z)",
                "(3/4,y+1/2,-z)",
                "(3/4,-y,-z)",
                "(y+1/2,3/4,-z)",
                "(-y,3/4,-z)"
            ],
            [
                "(x,-x,1/2)",
                "(-x+1/2,x+1/2,1/2)",
                "(x+1/2,x,1/2)",
                "(-x,-x+1/2,1/2)",
                "(-x,x,1/2)",
                "(x+1/2,-x+1/2,1/2)",
                "(-x+1/2,-x,1/2)",
                "(x,x+1/2,1/2)"
            ],
            [
                "(x,-x,0)",
                "(-x+1/2,x+1/2,0)",
                "(x+1/2,x,0)",
                "(-x,-x+1/2,0)",
                "(-x,x,0)",
                "(x+1/2,-x+1/2,0)",
                "(-x+1/2,-x,0)",
                "(x,x+1/2,0)"
            ],
            [
                "(3/4,1/4,z)",
                "(1/4,3/4,z)",
                "(1/4,3/4,-z)",
                "(3/4,1/4,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z)"
            ],
            [
                "(3/4,1/4,1/2)",
                "(1/4,3/4,1/2)"
            ],
            [
                "(3/4,1/4,0)",
                "(1/4,3/4,0)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            ".m.",
            "..2",
            "..2",
            "2mm.",
            "..2/m",
            "..2/m",
            "4mm",
            "-4m2",
            "-4m2"
        ],
        "Space Group": "P 4/n 2sub1/m m",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "410": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z)",
                "(y+1/2,-x+1/2,z)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(y,x,-z+1/2)",
                "(-y,-x,-z+1/2)",
                "(-x+1/2,-y+1/2,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(x,x,1/4)",
                "(-x,-x,1/4)",
                "(-x+1/2,x+1/2,1/4)",
                "(x+1/2,-x+1/2,1/4)",
                "(-x+1/2,-x+1/2,3/4)",
                "(x+1/2,x+1/2,3/4)",
                "(x,-x,3/4)",
                "(-x,x,3/4)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z+1/2)",
                "(1/2,1/2,-z)",
                "(0,0,-z)",
                "(0,0,z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(3/4,3/4,0)",
                "(1/4,3/4,0)",
                "(3/4,1/4,0)",
                "(1/4,3/4,1/2)",
                "(3/4,1/4,1/2)",
                "(1/4,1/4,1/2)",
                "(3/4,3/4,1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,-z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            "-1",
            "4..",
            "-4..",
            "2.22"
        ],
        "Space Group": "P 4/n 2sub1/c c",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "411": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z)",
                "(y,-x+1/2,z)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y,-x,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z)",
                "(-y,x+1/2,-z)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y,z+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(x,-x,1/4)",
                "(-x+1/2,x+1/2,1/4)",
                "(x+1/2,x,1/4)",
                "(-x,-x+1/2,1/4)",
                "(-x,x,3/4)",
                "(x+1/2,-x+1/2,3/4)",
                "(-x+1/2,-x,3/4)",
                "(x,x+1/2,3/4)"
            ],
            [
                "(3/4,1/4,z)",
                "(1/4,3/4,z)",
                "(1/4,3/4,-z+1/2)",
                "(3/4,1/4,-z+1/2)",
                "(1/4,3/4,-z)",
                "(3/4,1/4,-z)",
                "(3/4,1/4,z+1/2)",
                "(1/4,3/4,z+1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)",
                "(0,0,1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(3/4,3/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(1/4,1/4,z+1/2)"
            ],
            [
                "(3/4,1/4,0)",
                "(1/4,3/4,0)",
                "(1/4,3/4,1/2)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            "-1",
            "4..",
            "-4..",
            "2.22"
        ],
        "Space Group": "P 4/n 2sub1/c c",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "412": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y,x,-z+1/2)",
                "(-y,-x,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z+1/2)",
                "(-y,x,-z+1/2)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-y,-x,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,1/2)",
                "(y,-x,1/2)",
                "(-x,y,0)",
                "(x,-y,0)",
                "(y,x,1/2)",
                "(-y,-x,1/2)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,z)",
                "(-y,1/2,z+1/2)",
                "(y,1/2,z+1/2)",
                "(1/2,y,-z)",
                "(1/2,-y,-z)",
                "(y,1/2,-z+1/2)",
                "(-y,1/2,-z+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(-y,0,z+1/2)",
                "(y,0,z+1/2)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(y,0,-z+1/2)",
                "(-y,0,-z+1/2)"
            ],
            [
                "(x,x,1/4)",
                "(-x,-x,1/4)",
                "(-x,x,3/4)",
                "(x,-x,3/4)",
                "(-x,-x,3/4)",
                "(x,x,3/4)",
                "(x,-x,1/4)",
                "(-x,x,1/4)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,x,0)",
                "(0,-x,0)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)",
                "(0,1/2,-z)",
                "(1/2,0,-z+1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(0,0,-z)",
                "(0,0,-z+1/2)"
            ],
            [
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".m.",
            ".m.",
            "..2",
            "m2m.",
            "m2m.",
            "m2m.",
            "m2m.",
            "2mm.",
            "2mm.",
            "2mm.",
            "-4m2",
            "-4m2",
            "mmm.",
            "mmm.",
            "mmm.",
            "mmm."
        ],
        "Space Group": "P 4sub2/m 2/m 2/c",
        "Wyckoff Letter": [
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "413": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z+1/2)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z+1/2)",
                "(-y,x,-z+1/2)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,z+1/2)",
                "(x,-x,z+1/2)",
                "(-x,x,-z+1/2)",
                "(x,-x,-z+1/2)",
                "(x,x,-z)",
                "(-x,-x,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,1/2)",
                "(y,-x,1/2)",
                "(-x,y,1/2)",
                "(x,-y,1/2)",
                "(y,x,0)",
                "(-y,-x,0)"
            ],
            [
                "(x,1/2,1/4)",
                "(-x,1/2,1/4)",
                "(1/2,x,3/4)",
                "(1/2,-x,3/4)",
                "(-x,1/2,3/4)",
                "(x,1/2,3/4)",
                "(1/2,-x,1/4)",
                "(1/2,x,1/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,1/4)",
                "(0,x,3/4)",
                "(0,-x,3/4)",
                "(-x,0,3/4)",
                "(x,0,3/4)",
                "(0,-x,1/4)",
                "(0,x,1/4)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(1/2,0,-z+1/2)",
                "(0,1/2,z+1/2)",
                "(1/2,0,z)"
            ],
            [
                "(x,x,1/2)",
                "(-x,-x,1/2)",
                "(-x,x,0)",
                "(x,-x,0)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x,x,1/2)",
                "(x,-x,1/2)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,1/2,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(0,0,-z+1/2)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,3/4)",
                "(0,1/2,3/4)",
                "(1/2,0,1/4)"
            ],
            [
                "(1/2,1/2,1/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            ".2.",
            ".2.",
            "2..",
            "m.2m",
            "m.2m",
            "2.mm",
            "2.mm",
            "2/m..",
            "222.",
            "-42m",
            "m.mm",
            "-42m",
            "m.mm"
        ],
        "Space Group": "P 4sub2/m 2/c 2/m",
        "Wyckoff Letter": [
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "414": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z+1/2)",
                "(y+1/2,x+1/2,-z)",
                "(-y+1/2,-x+1/2,-z)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(-y,-x,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(x,x+1/2,0)",
                "(-x,-x+1/2,0)",
                "(-x,x+1/2,1/2)",
                "(x,-x+1/2,1/2)",
                "(-x+1/2,-x,1/2)",
                "(x+1/2,x,1/2)",
                "(x+1/2,-x,0)",
                "(-x+1/2,x,0)"
            ],
            [
                "(x,0,3/4)",
                "(-x,0,3/4)",
                "(1/2,x+1/2,1/4)",
                "(1/2,-x+1/2,1/4)",
                "(-x+1/2,1/2,3/4)",
                "(x+1/2,1/2,3/4)",
                "(0,-x,1/4)",
                "(0,x,1/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,1/4)",
                "(1/2,x+1/2,3/4)",
                "(1/2,-x+1/2,3/4)",
                "(-x+1/2,1/2,1/4)",
                "(x+1/2,1/2,1/4)",
                "(0,-x,3/4)",
                "(0,x,3/4)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)",
                "(0,0,-z+1/2)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)",
                "(1/2,1/2,z)",
                "(0,0,z+1/2)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)",
                "(0,1/2,-z+1/2)",
                "(0,1/2,-z)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,-z)",
                "(1/2,0,z)",
                "(1/2,0,z+1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)",
                "(3/4,3/4,3/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)",
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/4)",
                "(1/2,1/2,3/4)",
                "(1/2,1/2,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,1/2,1/4)",
                "(0,1/2,3/4)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-4..",
            "2.22",
            "222.",
            "222."
        ],
        "Space Group": "P 4sub2/n 2/b 2/c",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "415": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z+1/2)",
                "(y,-x+1/2,z+1/2)",
                "(-x+1/2,y,-z)",
                "(x,-y+1/2,-z)",
                "(y,x,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z+1/2)",
                "(-y,x+1/2,-z+1/2)",
                "(x+1/2,-y,z)",
                "(-x,y+1/2,z)",
                "(-y,-x,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(x,x,1/4)",
                "(-x+1/2,-x+1/2,1/4)",
                "(-x+1/2,x,3/4)",
                "(x,-x+1/2,3/4)",
                "(-x,-x,3/4)",
                "(x+1/2,x+1/2,3/4)",
                "(x+1/2,-x,1/4)",
                "(-x,x+1/2,1/4)"
            ],
            [
                "(x,1/4,1/2)",
                "(-x+1/2,1/4,1/2)",
                "(1/4,x,0)",
                "(1/4,-x+1/2,0)",
                "(-x,3/4,1/2)",
                "(x+1/2,3/4,1/2)",
                "(3/4,-x,0)",
                "(3/4,x+1/2,0)"
            ],
            [
                "(x,1/4,0)",
                "(-x+1/2,1/4,0)",
                "(1/4,x,1/2)",
                "(1/4,-x+1/2,1/2)",
                "(-x,3/4,0)",
                "(x+1/2,3/4,0)",
                "(3/4,-x,1/2)",
                "(3/4,x+1/2,1/2)"
            ],
            [
                "(3/4,1/4,z)",
                "(1/4,3/4,z+1/2)",
                "(3/4,1/4,-z)",
                "(1/4,3/4,-z+1/2)",
                "(1/4,3/4,-z)",
                "(3/4,1/4,-z+1/2)",
                "(1/4,3/4,z)",
                "(3/4,1/4,z+1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,1/4,z+1/2)",
                "(1/4,1/4,-z)",
                "(1/4,1/4,-z+1/2)",
                "(3/4,3/4,-z)",
                "(3/4,3/4,-z+1/2)",
                "(3/4,3/4,z)",
                "(3/4,3/4,z+1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(3/4,1/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,1/4,3/4)",
                "(3/4,3/4,3/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(3/4,1/4,0)",
                "(1/4,3/4,1/2)",
                "(1/4,3/4,0)",
                "(3/4,1/4,1/2)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,1/4,1/2)",
                "(3/4,3/4,0)",
                "(3/4,3/4,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "-1",
            "-4..",
            "2.22",
            "222.",
            "222."
        ],
        "Space Group": "P 4sub2/n 2/b 2/c",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "416": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x+1/2,x+1/2,z+1/2)",
                "(x+1/2,-x+1/2,z+1/2)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(x+1/2,x+1/2,-z+1/2)",
                "(-x+1/2,-x+1/2,-z+1/2)"
            ],
            [
                "(x,x+1/2,3/4)",
                "(-x,-x+1/2,3/4)",
                "(-x,x+1/2,1/4)",
                "(x,-x+1/2,1/4)",
                "(-x+1/2,-x,3/4)",
                "(x+1/2,x,3/4)",
                "(x+1/2,-x,1/4)",
                "(-x+1/2,x,1/4)"
            ],
            [
                "(x,x+1/2,1/4)",
                "(-x,-x+1/2,1/4)",
                "(-x,x+1/2,3/4)",
                "(x,-x+1/2,3/4)",
                "(-x+1/2,-x,1/4)",
                "(x+1/2,x,1/4)",
                "(x+1/2,-x,3/4)",
                "(-x+1/2,x,3/4)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x+1/2,0)",
                "(1/2,-x+1/2,0)",
                "(-x+1/2,1/2,0)",
                "(x+1/2,1/2,0)",
                "(0,-x,1/2)",
                "(0,x,1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(1/2,x+1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(0,-x,0)",
                "(0,x,0)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)",
                "(0,1/2,-z)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,-z)",
                "(1/2,0,z+1/2)",
                "(1/2,0,z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)",
                "(0,0,-z)",
                "(1/2,1/2,-z+1/2)"
            ],
            [
                "(3/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(0,1/2,1/4)",
                "(0,1/2,3/4)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2.mm",
            "..2/m",
            "..2/m",
            "2.22",
            "222.",
            "-42m",
            "-42m"
        ],
        "Space Group": "P 4sub2/n 2/n 2/m",
        "Wyckoff Letter": [
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "417": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z+1/2)",
                "(y,-x+1/2,z+1/2)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(y,x,-z)",
                "(-y+1/2,-x+1/2,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z+1/2)",
                "(-y,x+1/2,-z+1/2)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(-y,-x,z)",
                "(y+1/2,x+1/2,z)"
            ],
            [
                "(x,-x,z)",
                "(-x+1/2,x+1/2,z)",
                "(x+1/2,x,z+1/2)",
                "(-x,-x+1/2,z+1/2)",
                "(-x+1/2,-x,-z+1/2)",
                "(x,x+1/2,-z+1/2)",
                "(-x,x,-z)",
                "(x+1/2,-x+1/2,-z)"
            ],
            [
                "(x,x,1/2)",
                "(-x+1/2,-x+1/2,1/2)",
                "(-x+1/2,x,0)",
                "(x,-x+1/2,0)",
                "(-x,-x,1/2)",
                "(x+1/2,x+1/2,1/2)",
                "(x+1/2,-x,0)",
                "(-x,x+1/2,0)"
            ],
            [
                "(x,x,0)",
                "(-x+1/2,-x+1/2,0)",
                "(-x+1/2,x,1/2)",
                "(x,-x+1/2,1/2)",
                "(-x,-x,0)",
                "(x+1/2,x+1/2,0)",
                "(x+1/2,-x,1/2)",
                "(-x,x+1/2,1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,1/4,1/4)",
                "(1/4,x,3/4)",
                "(1/4,-x+1/2,3/4)",
                "(-x,3/4,3/4)",
                "(x+1/2,3/4,3/4)",
                "(3/4,-x,1/4)",
                "(3/4,x+1/2,1/4)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x+1/2,1/4,3/4)",
                "(1/4,x,1/4)",
                "(1/4,-x+1/2,1/4)",
                "(-x,3/4,1/4)",
                "(x+1/2,3/4,1/4)",
                "(3/4,-x,3/4)",
                "(3/4,x+1/2,3/4)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,1/4,z+1/2)",
                "(1/4,1/4,-z+1/2)",
                "(1/4,1/4,-z)",
                "(3/4,3/4,-z)",
                "(3/4,3/4,-z+1/2)",
                "(3/4,3/4,z+1/2)",
                "(3/4,3/4,z)"
            ],
            [
                "(3/4,1/4,z)",
                "(1/4,3/4,z+1/2)",
                "(3/4,1/4,-z+1/2)",
                "(1/4,3/4,-z)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(1/4,1/4,0)",
                "(1/4,1/4,1/2)",
                "(3/4,3/4,0)",
                "(3/4,3/4,1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,1/4,3/4)",
                "(3/4,3/4,3/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(1/4,3/4,1/4)",
                "(3/4,1/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2.mm",
            "..2/m",
            "..2/m",
            "2.22",
            "222.",
            "-42m",
            "-42m"
        ],
        "Space Group": "P 4sub2/n 2/n 2/m",
        "Wyckoff Letter": [
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "418": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z+1/2)",
                "(y,-x,z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z+1/2)",
                "(-y,x,-z+1/2)",
                "(x+1/2,-y+1/2,z)",
                "(-x+1/2,y+1/2,z)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,1/2)",
                "(y,-x,1/2)",
                "(-x+1/2,y+1/2,0)",
                "(x+1/2,-y+1/2,0)",
                "(y+1/2,x+1/2,1/2)",
                "(-y+1/2,-x+1/2,1/2)"
            ],
            [
                "(x,x+1/2,1/4)",
                "(-x,-x+1/2,1/4)",
                "(-x+1/2,x,3/4)",
                "(x+1/2,-x,3/4)",
                "(-x,-x+1/2,3/4)",
                "(x,x+1/2,3/4)",
                "(x+1/2,-x,1/4)",
                "(-x+1/2,x,1/4)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z+1/2)",
                "(1/2,0,-z)",
                "(0,1/2,-z+1/2)",
                "(0,1/2,-z)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,z)",
                "(0,1/2,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(1/2,1/2,-z)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)",
                "(0,0,-z+1/2)",
                "(1/2,1/2,z)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,3/4)",
                "(0,1/2,3/4)",
                "(1/2,0,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,1/2)",
                "(1/2,0,0)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)",
                "(1/2,1/2,3/4)",
                "(1/2,1/2,1/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)",
                "(1/2,1/2,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "2..",
            "2..",
            "2.22",
            "2/m..",
            "-4..",
            "2/m.."
        ],
        "Space Group": "P 4sub2/m 2sub1/b 2/c",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "419": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y+1/2,-x+1/2,-z+1/2)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x+1/2,x+1/2,z+1/2)",
                "(x+1/2,-x+1/2,z+1/2)",
                "(-x+1/2,x+1/2,-z+1/2)",
                "(x+1/2,-x+1/2,-z+1/2)",
                "(x,x,-z)",
                "(-x,-x,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y+1/2,x+1/2,1/2)",
                "(y+1/2,-x+1/2,1/2)",
                "(-x+1/2,y+1/2,1/2)",
                "(x+1/2,-y+1/2,1/2)",
                "(y,x,0)",
                "(-y,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,z+1/2)",
                "(1/2,0,z)"
            ],
            [
                "(x,-x,0)",
                "(-x,x,0)",
                "(x+1/2,x+1/2,1/2)",
                "(-x+1/2,-x+1/2,1/2)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x+1/2,x+1/2,1/2)",
                "(x+1/2,-x+1/2,1/2)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,1/4)",
                "(0,1/2,3/4)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "2..",
            "m.2m",
            "m.2m",
            "2.mm",
            "-4..",
            "2/m..",
            "m.mm",
            "m.mm"
        ],
        "Space Group": "P 4sub2/m 2sub1/n 2/m",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "420": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z+1/2)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(-y+1/2,1/2,z+1/2)",
                "(y+1/2,1/2,z+1/2)",
                "(1/2,y+1/2,-z+1/2)",
                "(1/2,-y+1/2,-z+1/2)",
                "(y,0,-z)",
                "(-y,0,-z)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x+1/2,x+1/2,1/2)",
                "(x+1/2,-x+1/2,1/2)",
                "(-x+1/2,-x+1/2,1/2)",
                "(x+1/2,x+1/2,1/2)",
                "(x,-x,0)",
                "(-x,x,0)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)",
                "(1/4,1/4,3/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,-z+1/2)",
                "(1/2,0,-z)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "-1",
            "2mm.",
            "2mm.",
            "-4m2",
            "-4m2"
        ],
        "Space Group": "P 4sub2/n 2sub1/m 2/c",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "421": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z+1/2)",
                "(y,-x+1/2,z+1/2)",
                "(-x,y+1/2,-z)",
                "(x+1/2,-y,-z)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y,-x,-z+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z+1/2)",
                "(-y,x+1/2,-z+1/2)",
                "(x,-y+1/2,z)",
                "(-x+1/2,y,z)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(1/4,y,z)",
                "(1/4,-y+1/2,z)",
                "(-y+1/2,1/4,z+1/2)",
                "(y,1/4,z+1/2)",
                "(3/4,y+1/2,-z)",
                "(3/4,-y,-z)",
                "(y+1/2,3/4,-z+1/2)",
                "(-y,3/4,-z+1/2)"
            ],
            [
                "(x,-x,1/4)",
                "(-x+1/2,x+1/2,1/4)",
                "(x+1/2,x,3/4)",
                "(-x,-x+1/2,3/4)",
                "(-x,x,3/4)",
                "(x+1/2,-x+1/2,3/4)",
                "(-x+1/2,-x,1/4)",
                "(x,x+1/2,1/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(1/2,1/2,1/2)",
                "(0,0,1/2)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,1/4,z+1/2)",
                "(3/4,3/4,-z)",
                "(3/4,3/4,-z+1/2)"
            ],
            [
                "(3/4,1/4,z)",
                "(1/4,3/4,z+1/2)",
                "(1/4,3/4,-z)",
                "(3/4,1/4,-z+1/2)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(3/4,1/4,3/4)",
                "(1/4,3/4,1/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            "-1",
            "2mm.",
            "2mm.",
            "-4m2",
            "-4m2"
        ],
        "Space Group": "P 4sub2/n 2sub1/m 2/c",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "422": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y+1/2,-z)",
                "(y,x,-z+1/2)",
                "(-y,-x,-z+1/2)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(-y+1/2,-x+1/2,z)",
                "(y+1/2,x+1/2,z)"
            ],
            [
                "(x,x+1/2,z)",
                "(-x,-x+1/2,z)",
                "(-x,x+1/2,z+1/2)",
                "(x,-x+1/2,z+1/2)",
                "(-x+1/2,x,-z)",
                "(x+1/2,-x,-z)",
                "(x+1/2,x,-z+1/2)",
                "(-x+1/2,-x,-z+1/2)"
            ],
            [
                "(x,x,3/4)",
                "(-x,-x,3/4)",
                "(-x+1/2,x+1/2,1/4)",
                "(x+1/2,-x+1/2,1/4)",
                "(-x+1/2,-x+1/2,3/4)",
                "(x+1/2,x+1/2,3/4)",
                "(x,-x,1/4)",
                "(-x,x,1/4)"
            ],
            [
                "(x,x,1/4)",
                "(-x,-x,1/4)",
                "(-x+1/2,x+1/2,3/4)",
                "(x+1/2,-x+1/2,3/4)",
                "(-x+1/2,-x+1/2,1/4)",
                "(x+1/2,x+1/2,1/4)",
                "(x,-x,3/4)",
                "(-x,x,3/4)"
            ],
            [
                "(0,0,z)",
                "(1/2,1/2,z+1/2)",
                "(1/2,1/2,-z)",
                "(0,0,-z+1/2)",
                "(1/2,1/2,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)",
                "(1/2,1/2,z)"
            ],
            [
                "(0,1/2,z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,-z)",
                "(1/2,0,-z+1/2)"
            ],
            [
                "(1/4,1/4,3/4)",
                "(3/4,3/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)",
                "(1/2,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(1/2,1/2,3/4)",
                "(1/2,1/2,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "2..",
            "2.mm",
            "..2/m",
            "..2/m",
            "-4..",
            "2.22"
        ],
        "Space Group": "P 4sub2/n 2sub1/c 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "423": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-y+1/2,x,z+1/2)",
                "(y,-x+1/2,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y,-z+1/2)",
                "(y+1/2,x+1/2,-z)",
                "(-y,-x,-z)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(y+1/2,-x,-z+1/2)",
                "(-y,x+1/2,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y,z+1/2)",
                "(-y+1/2,-x+1/2,z)",
                "(y,x,z)"
            ],
            [
                "(x,x,z)",
                "(-x+1/2,-x+1/2,z)",
                "(-x+1/2,x,z+1/2)",
                "(x,-x+1/2,z+1/2)",
                "(-x,x+1/2,-z+1/2)",
                "(x+1/2,-x,-z+1/2)",
                "(x+1/2,x+1/2,-z)",
                "(-x,-x,-z)"
            ],
            [
                "(x,-x,0)",
                "(-x+1/2,x+1/2,0)",
                "(x+1/2,x,1/2)",
                "(-x,-x+1/2,1/2)",
                "(-x,x,0)",
                "(x+1/2,-x+1/2,0)",
                "(-x+1/2,-x,1/2)",
                "(x,x+1/2,1/2)"
            ],
            [
                "(x,-x,1/2)",
                "(-x+1/2,x+1/2,1/2)",
                "(x+1/2,x,0)",
                "(-x,-x+1/2,0)",
                "(-x,x,1/2)",
                "(x+1/2,-x+1/2,1/2)",
                "(-x+1/2,-x,0)",
                "(x,x+1/2,0)"
            ],
            [
                "(3/4,1/4,z)",
                "(1/4,3/4,z+1/2)",
                "(1/4,3/4,-z+1/2)",
                "(3/4,1/4,-z)",
                "(1/4,3/4,-z)",
                "(3/4,1/4,-z+1/2)",
                "(3/4,1/4,z+1/2)",
                "(1/4,3/4,z)"
            ],
            [
                "(1/4,1/4,z)",
                "(1/4,1/4,z+1/2)",
                "(3/4,3/4,-z+1/2)",
                "(3/4,3/4,-z)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(1/2,0,0)",
                "(0,1/2,0)"
            ],
            [
                "(3/4,1/4,3/4)",
                "(1/4,3/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(3/4,1/4,0)",
                "(1/4,3/4,1/2)",
                "(1/4,3/4,0)",
                "(3/4,1/4,1/2)"
            ]
        ],
        "Multiplicity": [
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "2..",
            "2.mm",
            "..2/m",
            "..2/m",
            "-4..",
            "2.22"
        ],
        "Space Group": "P 4sub2/n 2sub1/c 2/m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "424": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-y,-x,z)",
                "(y,x,z)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(-y,0,z)",
                "(y,0,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(y,0,-z)",
                "(-y,0,-z)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,z)",
                "(x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(x,x,-z)",
                "(-x,-x,-z)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,0)",
                "(y,-x,0)",
                "(-x,y,0)",
                "(x,-y,0)",
                "(y,x,0)",
                "(-y,-x,0)"
            ],
            [
                "(x,x+1/2,1/4)",
                "(-x,-x+1/2,1/4)",
                "(-x+1/2,x,1/4)",
                "(x+1/2,-x,1/4)",
                "(-x,-x+1/2,3/4)",
                "(x,x+1/2,3/4)",
                "(x+1/2,-x,3/4)",
                "(-x+1/2,x,3/4)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(1/2,x,0)",
                "(1/2,-x,0)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)"
            ],
            [
                "(x,x,0)",
                "(-x,-x,0)",
                "(-x,x,0)",
                "(x,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "16",
            "8",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..m",
            "m..",
            "..2",
            "m2m.",
            "m2m.",
            "m.2m",
            "2mm.",
            "..2/m",
            "4mm",
            "-4m2",
            "mmm.",
            "4/mmm",
            "4/mmm"
        ],
        "Space Group": "I 4/m 2/m 2/m",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "425": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-y,x,z)",
                "(y,-x,z)",
                "(-x,y,-z+1/2)",
                "(x,-y,-z+1/2)",
                "(y,x,-z+1/2)",
                "(-y,-x,-z+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,-y,z+1/2)",
                "(-x,y,z+1/2)",
                "(-y,-x,z+1/2)",
                "(y,x,z+1/2)"
            ],
            [
                "(x,x+1/2,z)",
                "(-x,-x+1/2,z)",
                "(-x+1/2,x,z)",
                "(x+1/2,-x,z)",
                "(-x,x+1/2,-z+1/2)",
                "(x,-x+1/2,-z+1/2)",
                "(x+1/2,x,-z+1/2)",
                "(-x+1/2,-x,-z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-x,-y,0)",
                "(-y,x,0)",
                "(y,-x,0)",
                "(-x,y,1/2)",
                "(x,-y,1/2)",
                "(y,x,1/2)",
                "(-y,-x,1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x,0,1/4)",
                "(0,x,1/4)",
                "(0,-x,1/4)",
                "(-x,0,3/4)",
                "(x,0,3/4)",
                "(0,-x,3/4)",
                "(0,x,3/4)"
            ],
            [
                "(x,x,1/4)",
                "(-x,-x,1/4)",
                "(-x,x,1/4)",
                "(x,-x,1/4)",
                "(-x,-x,3/4)",
                "(x,x,3/4)",
                "(x,-x,3/4)",
                "(-x,x,3/4)"
            ],
            [
                "(x,x+1/2,0)",
                "(-x,-x+1/2,0)",
                "(-x+1/2,x,0)",
                "(x+1/2,-x,0)"
            ],
            [
                "(0,1/2,z)",
                "(1/2,0,z)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,1/4)",
                "(1/2,0,1/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "16",
            "8",
            "8",
            "8",
            "8",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            ".2.",
            "..2",
            "m.2m",
            "2.mm",
            "4..",
            "..2/m",
            "m.mm",
            "4/m..",
            "-42m",
            "422"
        ],
        "Space Group": "I 4/m 2/c 2/m",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "426": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-y,x+1/2,z+1/4)",
                "(y+1/2,-x,z+3/4)",
                "(-x+1/2,y,-z+3/4)",
                "(x,-y+1/2,-z+1/4)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y,-x,-z)",
                "(-x,-y+1/2,-z+1/4)",
                "(x+1/2,y,-z+3/4)",
                "(y,-x,-z)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x,y,z)",
                "(-y+1/2,-x,z+3/4)",
                "(y,x+1/2,z+1/4)"
            ],
            [
                "(0,y,z)",
                "(1/2,-y+1/2,z+1/2)",
                "(-y,1/2,z+1/4)",
                "(y+1/2,0,z+3/4)",
                "(1/2,y,-z+3/4)",
                "(0,-y+1/2,-z+1/4)",
                "(y+1/2,1/2,-z+1/2)",
                "(-y,0,-z)"
            ],
            [
                "(x,x,0)",
                "(-x+1/2,-x+1/2,1/2)",
                "(-x,x+1/2,1/4)",
                "(x+1/2,-x,3/4)",
                "(-x,-x+1/2,1/4)",
                "(x+1/2,x,3/4)",
                "(x,-x,0)",
                "(-x+1/2,x+1/2,1/2)"
            ],
            [
                "(x,1/4,1/8)",
                "(-x+1/2,1/4,5/8)",
                "(3/4,x+1/2,3/8)",
                "(3/4,-x,7/8)",
                "(-x,1/4,1/8)",
                "(x+1/2,1/4,5/8)",
                "(1/4,-x,7/8)",
                "(1/4,x+1/2,3/8)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z+1/4)",
                "(1/2,0,-z+3/4)",
                "(1/2,1/2,-z+1/2)"
            ],
            [
                "(0,1/4,5/8)",
                "(1/2,1/4,1/8)",
                "(3/4,1/2,7/8)",
                "(3/4,0,3/8)"
            ],
            [
                "(0,1/4,1/8)",
                "(1/2,1/4,5/8)",
                "(3/4,1/2,3/8)",
                "(3/4,0,7/8)"
            ],
            [
                "(0,0,1/2)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/4)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            ".2.",
            "2mm.",
            ".2/m.",
            ".2/m.",
            "-4m2",
            "-4m2"
        ],
        "Space Group": "I 4sub1/a 2/m 2/d",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "427": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-y+1/4,x+3/4,z+1/4)",
                "(y+1/4,-x+1/4,z+3/4)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y,-z)",
                "(y+1/4,x+3/4,-z+1/4)",
                "(-y+1/4,-x+1/4,-z+3/4)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(y+3/4,-x+1/4,-z+3/4)",
                "(-y+3/4,x+3/4,-z+1/4)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y,z)",
                "(-y+3/4,-x+1/4,z+3/4)",
                "(y+3/4,x+3/4,z+1/4)"
            ],
            [
                "(0,y,z)",
                "(1/2,-y,z+1/2)",
                "(-y+1/4,3/4,z+1/4)",
                "(y+1/4,1/4,z+3/4)",
                "(1/2,y,-z+1/2)",
                "(0,-y,-z)",
                "(y+1/4,3/4,-z+1/4)",
                "(-y+1/4,1/4,-z+3/4)"
            ],
            [
                "(x,x+1/4,7/8)",
                "(-x+1/2,-x+3/4,3/8)",
                "(-x,x+3/4,1/8)",
                "(x+1/2,-x+1/4,5/8)",
                "(-x,-x+3/4,1/8)",
                "(x+1/2,x+1/4,5/8)",
                "(x,-x+1/4,7/8)",
                "(-x+1/2,x+3/4,3/8)"
            ],
            [
                "(x,0,0)",
                "(-x+1/2,0,1/2)",
                "(1/4,x+3/4,1/4)",
                "(1/4,-x+1/4,3/4)",
                "(-x,0,0)",
                "(x+1/2,0,1/2)",
                "(3/4,-x+1/4,3/4)",
                "(3/4,x+3/4,1/4)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,z+1/4)",
                "(1/2,1/4,-z+1/2)",
                "(1/2,3/4,-z+1/4)"
            ],
            [
                "(0,0,1/2)",
                "(1/2,0,0)",
                "(1/4,3/4,3/4)",
                "(1/4,1/4,1/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(1/4,3/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(0,1/4,3/8)",
                "(0,3/4,5/8)"
            ],
            [
                "(0,3/4,1/8)",
                "(1/2,3/4,3/8)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "8",
            "8",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..2",
            ".2.",
            "2mm.",
            ".2/m.",
            ".2/m.",
            "-4m2",
            "-4m2"
        ],
        "Space Group": "I 4sub1/a 2/m 2/d",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "428": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z+1/2)",
                "(-y,x+1/2,z+1/4)",
                "(y+1/2,-x,z+3/4)",
                "(-x+1/2,y,-z+1/4)",
                "(x,-y+1/2,-z+3/4)",
                "(y+1/2,x+1/2,-z)",
                "(-y,-x,-z+1/2)",
                "(-x,-y+1/2,-z+1/4)",
                "(x+1/2,y,-z+3/4)",
                "(y,-x,-z)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z)",
                "(-x,y,z+1/2)",
                "(-y+1/2,-x,z+1/4)",
                "(y,x+1/2,z+3/4)"
            ],
            [
                "(x,x,1/4)",
                "(-x+1/2,-x+1/2,3/4)",
                "(-x,x+1/2,1/2)",
                "(x+1/2,-x,0)",
                "(-x,-x+1/2,0)",
                "(x+1/2,x,1/2)",
                "(x,-x,3/4)",
                "(-x+1/2,x+1/2,1/4)"
            ],
            [
                "(1/4,y,1/8)",
                "(1/4,-y+1/2,5/8)",
                "(-y,3/4,3/8)",
                "(y+1/2,3/4,7/8)",
                "(3/4,-y+1/2,1/8)",
                "(3/4,y,5/8)",
                "(y,3/4,7/8)",
                "(-y+1/2,3/4,3/8)"
            ],
            [
                "(0,0,z)",
                "(0,1/2,z+1/4)",
                "(1/2,0,-z+1/4)",
                "(1/2,1/2,-z)",
                "(0,1/2,-z+1/4)",
                "(0,0,-z)",
                "(1/2,1/2,z)",
                "(1/2,0,z+1/4)"
            ],
            [
                "(0,1/4,1/8)",
                "(1/2,1/4,5/8)",
                "(3/4,1/2,3/8)",
                "(3/4,0,7/8)",
                "(1/2,1/4,1/8)",
                "(0,1/4,5/8)",
                "(3/4,1/2,7/8)",
                "(3/4,0,3/8)"
            ],
            [
                "(0,0,1/4)",
                "(0,1/2,1/2)",
                "(0,1/2,0)",
                "(0,0,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,1/2,1/4)",
                "(1/2,0,1/4)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            "2..",
            "-1",
            "2.22",
            "-4.."
        ],
        "Space Group": "I 4sub1/a 2/c 2/d",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "429": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-y+1/4,x+3/4,z+1/4)",
                "(y+1/4,-x+1/4,z+3/4)",
                "(-x+1/2,y,-z)",
                "(x,-y,-z+1/2)",
                "(y+1/4,x+3/4,-z+3/4)",
                "(-y+1/4,-x+1/4,-z+1/4)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(y+3/4,-x+1/4,-z+3/4)",
                "(-y+3/4,x+3/4,-z+1/4)",
                "(x+1/2,-y,z)",
                "(-x,y,z+1/2)",
                "(-y+3/4,-x+1/4,z+1/4)",
                "(y+3/4,x+3/4,z+3/4)"
            ],
            [
                "(x,x+1/4,1/8)",
                "(-x+1/2,-x+3/4,5/8)",
                "(-x,x+3/4,3/8)",
                "(x+1/2,-x+1/4,7/8)",
                "(-x,-x+3/4,7/8)",
                "(x+1/2,x+1/4,3/8)",
                "(x,-x+1/4,5/8)",
                "(-x+1/2,x+3/4,1/8)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(1/4,x+3/4,1/2)",
                "(1/4,-x+1/4,0)",
                "(-x,0,3/4)",
                "(x+1/2,0,1/4)",
                "(3/4,-x+1/4,1/2)",
                "(3/4,x+3/4,0)"
            ],
            [
                "(0,1/4,z)",
                "(0,3/4,z+1/4)",
                "(1/2,1/4,-z)",
                "(1/2,3/4,-z+3/4)",
                "(0,3/4,-z)",
                "(0,1/4,-z+3/4)",
                "(1/2,3/4,z)",
                "(1/2,1/4,z+1/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(1/4,3/4,1/4)",
                "(1/4,1/4,3/4)",
                "(1/2,0,0)",
                "(0,0,1/2)",
                "(1/4,3/4,3/4)",
                "(1/4,1/4,1/4)"
            ],
            [
                "(0,1/4,1/8)",
                "(0,3/4,3/8)",
                "(0,3/4,7/8)",
                "(0,1/4,5/8)"
            ],
            [
                "(0,1/4,3/8)",
                "(0,3/4,5/8)",
                "(1/2,1/4,5/8)",
                "(1/2,3/4,3/8)"
            ]
        ],
        "Multiplicity": [
            "32",
            "16",
            "16",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            "2..",
            "-1",
            "2.22",
            "-4.."
        ],
        "Space Group": "I 4sub1/a 2/c 2/d",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "430": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)"
            ],
            [
                "(2/3,1/3,z)"
            ],
            [
                "(1/3,2/3,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "3",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "3..",
            "3..",
            "3.."
        ],
        "Space Group": "P 3  ",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "431": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+1/3)",
                "(-x+y,-x,z+2/3)"
            ]
        ],
        "Multiplicity": [
            "3"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 3sub1  ",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "432": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+2/3)",
                "(-x+y,-x,z+1/3)"
            ]
        ],
        "Multiplicity": [
            "3"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 3sub2  ",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "433": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "9",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "3"
        ],
        "Space Group": "R 3  ",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "434": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(z,x,y)",
                "(y,z,x)"
            ],
            [
                "(x,x,x)"
            ]
        ],
        "Multiplicity": [
            "3",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "3"
        ],
        "Space Group": "R 3  ",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "435": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3",
            "2",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "3..",
            "3..",
            "-3..",
            "-3.."
        ],
        "Space Group": "P -3  ",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "436": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "18",
            "9",
            "9",
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "3",
            "-3",
            "-3"
        ],
        "Space Group": "R -3  ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "437": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(z,x,y)",
                "(y,z,x)",
                "(-x,-y,-z)",
                "(-z,-x,-y)",
                "(-y,-z,-x)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,-x)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "-1",
            "-1",
            "3",
            "-3",
            "-3"
        ],
        "Space Group": "R -3  ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "438": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-y,-x,-z)",
                "(-x+y,y,-z)",
                "(x,x-y,-z)"
            ],
            [
                "(x,-x,1/2)",
                "(x,2x,1/2)",
                "(-2x,-x,1/2)"
            ],
            [
                "(x,-x,0)",
                "(x,2x,0)",
                "(-2x,-x,0)"
            ],
            [
                "(2/3,1/3,z)",
                "(2/3,1/3,-z)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(2/3,1/3,1/2)"
            ],
            [
                "(2/3,1/3,0)"
            ],
            [
                "(1/3,2/3,1/2)"
            ],
            [
                "(1/3,2/3,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "3..",
            "3..",
            "3..",
            "3.2",
            "3.2",
            "3.2",
            "3.2",
            "3.2",
            "3.2"
        ],
        "Space Group": "P 3 1 2",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "439": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3",
            "2",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2.",
            "3..",
            "3..",
            "32",
            "32"
        ],
        "Space Group": "P 3 2 1",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "440": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+1/3)",
                "(-x+y,-x,z+2/3)",
                "(-y,-x,-z+2/3)",
                "(-x+y,y,-z+1/3)",
                "(x,x-y,-z)"
            ],
            [
                "(x,-x,5/6)",
                "(x,2x,1/6)",
                "(-2x,-x,1/2)"
            ],
            [
                "(x,-x,1/3)",
                "(x,2x,2/3)",
                "(-2x,-x,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 3sub1 1 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "441": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+1/3)",
                "(-x+y,-x,z+2/3)",
                "(y,x,-z)",
                "(x-y,-y,-z+2/3)",
                "(-x,-x+y,-z+1/3)"
            ],
            [
                "(x,0,5/6)",
                "(0,x,1/6)",
                "(-x,-x,1/2)"
            ],
            [
                "(x,0,1/3)",
                "(0,x,2/3)",
                "(-x,-x,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2."
        ],
        "Space Group": "P 3sub1 2 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "442": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+2/3)",
                "(-x+y,-x,z+1/3)",
                "(-y,-x,-z+1/3)",
                "(-x+y,y,-z+2/3)",
                "(x,x-y,-z)"
            ],
            [
                "(x,-x,1/6)",
                "(x,2x,5/6)",
                "(-2x,-x,1/2)"
            ],
            [
                "(x,-x,2/3)",
                "(x,2x,1/3)",
                "(-2x,-x,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2"
        ],
        "Space Group": "P 3sub2 1 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "443": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+2/3)",
                "(-x+y,-x,z+1/3)",
                "(y,x,-z)",
                "(x-y,-y,-z+1/3)",
                "(-x,-x+y,-z+2/3)"
            ],
            [
                "(x,0,1/6)",
                "(0,x,5/6)",
                "(-x,-x,1/2)"
            ],
            [
                "(x,0,2/3)",
                "(0,x,1/3)",
                "(-x,-x,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            ".2."
        ],
        "Space Group": "P 3sub2 2 1",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "444": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "18",
            "9",
            "9",
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "0.2",
            "0.2",
            "3",
            "32",
            "32"
        ],
        "Space Group": "R 3 2 ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "445": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(z,x,y)",
                "(y,z,x)",
                "(-y,-x,-z)",
                "(-x,-z,-y)",
                "(-z,-y,-x)"
            ],
            [
                "(1/2,y,-y)",
                "(-y,1/2,y)",
                "(y,-y,1/2)"
            ],
            [
                "(0,y,-y)",
                "(-y,0,y)",
                "(y,-y,0)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,-x)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "0.2",
            "0.2",
            "3",
            "32",
            "32"
        ],
        "Space Group": "R 3 2 ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "446": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)"
            ],
            [
                "(x,-x,z)",
                "(x,2x,z)",
                "(-2x,-x,z)"
            ],
            [
                "(2/3,1/3,z)"
            ],
            [
                "(1/3,2/3,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "3m.",
            "3m.",
            "3m."
        ],
        "Space Group": "P 3 m 1",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "447": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(y,x,z)",
                "(x-y,-y,z)",
                "(-x,-x+y,z)"
            ],
            [
                "(x,0,z)",
                "(0,x,z)",
                "(-x,-x,z)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "2",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "3..",
            "3.m"
        ],
        "Space Group": "P 3 1 m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "448": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)"
            ],
            [
                "(2/3,1/3,z)",
                "(2/3,1/3,z+1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "6",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "3..",
            "3..",
            "3.."
        ],
        "Space Group": "P 3 c 1",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "449": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(y,x,z+1/2)",
                "(x-y,-y,z+1/2)",
                "(-x,-x+y,z+1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "6",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "3..",
            "3.."
        ],
        "Space Group": "P 3 1 c",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "450": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)"
            ],
            [
                "(x,-x,z)",
                "(x,2x,z)",
                "(-2x,-x,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "18",
            "9",
            "3"
        ],
        "Site Symmetry": [
            "1",
            ".m",
            "3m"
        ],
        "Space Group": "R 3 m ",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "451": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(z,x,y)",
                "(y,z,x)",
                "(y,x,z)",
                "(x,z,y)",
                "(z,y,x)"
            ],
            [
                "(x,x,z)",
                "(z,x,x)",
                "(x,z,x)"
            ],
            [
                "(x,x,x)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m",
            "3m"
        ],
        "Space Group": "R 3 m ",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "452": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "18",
            "6"
        ],
        "Site Symmetry": [
            "1",
            "3"
        ],
        "Space Group": "R 3 c ",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "453": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(z,x,y)",
                "(y,z,x)",
                "(y+1/2,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,y+1/2)",
                "(z+1/2,y+1/2,x+1/2)"
            ],
            [
                "(x,x,x)",
                "(x+1/2,x+1/2,x+1/2)"
            ]
        ],
        "Multiplicity": [
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "3"
        ],
        "Space Group": "R 3 c ",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "454": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-y,-x,-z)",
                "(-x+y,y,-z)",
                "(x,x-y,-z)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(y,x,z)",
                "(x-y,-y,z)",
                "(-x,-x+y,z)"
            ],
            [
                "(x,0,z)",
                "(0,x,z)",
                "(-x,-x,z)",
                "(0,-x,-z)",
                "(-x,0,-z)",
                "(x,x,-z)"
            ],
            [
                "(x,-x,1/2)",
                "(x,2x,1/2)",
                "(-2x,-x,1/2)",
                "(-x,x,1/2)",
                "(-x,-2x,1/2)",
                "(2x,x,1/2)"
            ],
            [
                "(x,-x,0)",
                "(x,2x,0)",
                "(-2x,-x,0)",
                "(-x,x,0)",
                "(-x,-2x,0)",
                "(2x,x,0)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,-z)",
                "(2/3,1/3,-z)",
                "(2/3,1/3,z)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/3,2/3,1/2)",
                "(2/3,1/3,1/2)"
            ],
            [
                "(1/3,2/3,0)",
                "(2/3,1/3,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "4",
            "3",
            "3",
            "2",
            "2",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "3..",
            "..2/m",
            "..2/m",
            "3.m",
            "3.2",
            "3.2",
            "-3.m",
            "-3.m"
        ],
        "Space Group": "P -3 1 2/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "455": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-y,-x,-z+1/2)",
                "(-x+y,y,-z+1/2)",
                "(x,x-y,-z+1/2)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(y,x,z+1/2)",
                "(x-y,-y,z+1/2)",
                "(-x,-x+y,z+1/2)"
            ],
            [
                "(x,-x,1/4)",
                "(x,2x,1/4)",
                "(-2x,-x,1/4)",
                "(-x,x,3/4)",
                "(-x,-2x,3/4)",
                "(2x,x,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,-z+1/2)",
                "(2/3,1/3,-z)",
                "(2/3,1/3,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(2/3,1/3,1/4)",
                "(1/3,2/3,3/4)"
            ],
            [
                "(1/3,2/3,1/4)",
                "(2/3,1/3,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "-1",
            "3..",
            "3..",
            "3.2",
            "3.2",
            "-3..",
            "3.2"
        ],
        "Space Group": "P -3 1 2/c",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "456": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)"
            ],
            [
                "(x,-x,z)",
                "(x,2x,z)",
                "(-2x,-x,z)",
                "(-x,x,-z)",
                "(2x,x,-z)",
                "(-x,-2x,-z)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)",
                "(-x,0,1/2)",
                "(0,-x,1/2)",
                "(x,x,1/2)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)",
                "(-x,0,0)",
                "(0,-x,0)",
                "(x,x,0)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "3",
            "3",
            "2",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            ".2.",
            ".2.",
            ".2/m.",
            ".2/m.",
            "3m.",
            "3m.",
            "-3m.",
            "-3m."
        ],
        "Space Group": "P -3 2/m 1",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "457": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(y,x,-z+1/2)",
                "(x-y,-y,-z+1/2)",
                "(-x,-x+y,-z+1/2)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)"
            ],
            [
                "(x,0,1/4)",
                "(0,x,1/4)",
                "(-x,-x,1/4)",
                "(-x,0,3/4)",
                "(0,-x,3/4)",
                "(x,x,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,-z+1/2)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".2.",
            "-1",
            "3..",
            "3..",
            "-3..",
            "32"
        ],
        "Space Group": "P -3 2/c 1",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "458": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)"
            ],
            [
                "(x,-x,z)",
                "(x,2x,z)",
                "(-2x,-x,z)",
                "(-x,x,-z)",
                "(2x,x,-z)",
                "(-x,-2x,-z)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)",
                "(-x,0,1/2)",
                "(0,-x,1/2)",
                "(x,x,1/2)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)",
                "(-x,0,0)",
                "(0,-x,0)",
                "(x,x,0)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "36",
            "18",
            "18",
            "18",
            "9",
            "9",
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            ".m",
            "0.2",
            "0.2",
            ".2/m",
            ".2/m",
            "3m",
            "-3m",
            "-3m"
        ],
        "Space Group": "R -3 2/m ",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "459": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(z,x,y)",
                "(y,z,x)",
                "(-y,-x,-z)",
                "(-x,-z,-y)",
                "(-z,-y,-x)",
                "(-x,-y,-z)",
                "(-z,-x,-y)",
                "(-y,-z,-x)",
                "(y,x,z)",
                "(x,z,y)",
                "(z,y,x)"
            ],
            [
                "(x,x,z)",
                "(z,x,x)",
                "(x,z,x)",
                "(-x,-x,-z)",
                "(-x,-z,-x)",
                "(-z,-x,-x)"
            ],
            [
                "(x,-x,1/2)",
                "(1/2,x,-x)",
                "(-x,1/2,x)",
                "(-x,x,1/2)",
                "(1/2,-x,x)",
                "(x,1/2,-x)"
            ],
            [
                "(x,-x,0)",
                "(0,x,-x)",
                "(-x,0,x)",
                "(-x,x,0)",
                "(0,-x,x)",
                "(x,0,-x)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,-x)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "3",
            "3",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m",
            "0.2",
            "0.2",
            ".2/m",
            ".2/m",
            "3m",
            "-3m",
            "-3m"
        ],
        "Space Group": "R -3 2/m ",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "460": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(y,x,-z+1/2)",
                "(x-y,-y,-z+1/2)",
                "(-x,-x+y,-z+1/2)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)"
            ],
            [
                "(x,0,1/4)",
                "(0,x,1/4)",
                "(-x,-x,1/4)",
                "(-x,0,3/4)",
                "(0,-x,3/4)",
                "(x,x,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "36",
            "18",
            "18",
            "12",
            "6",
            "6"
        ],
        "Site Symmetry": [
            "1",
            "0.2",
            "-1",
            "3",
            "-3",
            "32"
        ],
        "Space Group": "R -3 2/c ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "461": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(z,x,y)",
                "(y,z,x)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(-x+1/2,-z+1/2,-y+1/2)",
                "(-z+1/2,-y+1/2,-x+1/2)",
                "(-x,-y,-z)",
                "(-z,-x,-y)",
                "(-y,-z,-x)",
                "(y+1/2,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,y+1/2)",
                "(z+1/2,y+1/2,x+1/2)"
            ],
            [
                "(x,-x+1/2,1/4)",
                "(1/4,x,-x+1/2)",
                "(-x+1/2,1/4,x)",
                "(-x,x+1/2,3/4)",
                "(3/4,-x,x+1/2)",
                "(x+1/2,3/4,-x)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(-x,-x,-x)",
                "(x+1/2,x+1/2,x+1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "0.2",
            "-1",
            "3",
            "-3",
            "32"
        ],
        "Space Group": "R -3 2/c ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "462": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z)",
                "(y,-x+y,z)",
                "(x-y,x,z)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)",
                "(1/2,1/2,z)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "2",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "3..",
            "6.."
        ],
        "Space Group": "P 6  ",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "463": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+1/3)",
                "(-x+y,-x,z+2/3)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+5/6)",
                "(x-y,x,z+1/6)"
            ]
        ],
        "Multiplicity": [
            "6"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 6sub1  ",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "464": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+2/3)",
                "(-x+y,-x,z+1/3)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/6)",
                "(x-y,x,z+5/6)"
            ]
        ],
        "Multiplicity": [
            "6"
        ],
        "Site Symmetry": [
            "1"
        ],
        "Space Group": "P 6sub5  ",
        "Wyckoff Letter": [
            "a"
        ]
    },
    "465": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+2/3)",
                "(-x+y,-x,z+1/3)",
                "(-x,-y,z)",
                "(y,-x+y,z+2/3)",
                "(x-y,x,z+1/3)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,0,z+2/3)",
                "(0,1/2,z+1/3)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+2/3)",
                "(0,0,z+1/3)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2.."
        ],
        "Space Group": "P 6sub2  ",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "466": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+1/3)",
                "(-x+y,-x,z+2/3)",
                "(-x,-y,z)",
                "(y,-x+y,z+1/3)",
                "(x-y,x,z+2/3)"
            ],
            [
                "(1/2,1/2,z)",
                "(1/2,0,z+1/3)",
                "(0,1/2,z+2/3)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/3)",
                "(0,0,z+2/3)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2.."
        ],
        "Space Group": "P 6sub4  ",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "467": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/2)",
                "(x-y,x,z+1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "6",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "3..",
            "3.."
        ],
        "Space Group": "P 6sub3  ",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "468": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(x,y,-z)",
                "(-y,x-y,-z)",
                "(-x+y,-x,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-y,x-y,1/2)",
                "(-x+y,-x,1/2)"
            ],
            [
                "(x,y,0)",
                "(-y,x-y,0)",
                "(-x+y,-x,0)"
            ],
            [
                "(2/3,1/3,z)",
                "(2/3,1/3,-z)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(2/3,1/3,1/2)"
            ],
            [
                "(2/3,1/3,0)"
            ],
            [
                "(1/3,2/3,1/2)"
            ],
            [
                "(1/3,2/3,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "6",
            "3",
            "3",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            "3..",
            "3..",
            "3..",
            "-6..",
            "-6..",
            "-6..",
            "-6..",
            "-6..",
            "-6.."
        ],
        "Space Group": "P -6  ",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "469": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z)",
                "(y,-x+y,z)",
                "(x-y,x,z)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(x,y,-z)",
                "(-y,x-y,-z)",
                "(-x+y,-x,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-y,x-y,1/2)",
                "(-x+y,-x,1/2)",
                "(-x,-y,1/2)",
                "(y,-x+y,1/2)",
                "(x-y,x,1/2)"
            ],
            [
                "(x,y,0)",
                "(-y,x-y,0)",
                "(-x+y,-x,0)",
                "(-x,-y,0)",
                "(y,-x+y,0)",
                "(x-y,x,0)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)",
                "(1/2,1/2,z)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,-z)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/3,2/3,1/2)",
                "(2/3,1/3,1/2)"
            ],
            [
                "(1/3,2/3,0)",
                "(2/3,1/3,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "4",
            "3",
            "3",
            "2",
            "2",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            "2..",
            "3..",
            "2/m..",
            "2/m..",
            "6..",
            "-6..",
            "-6..",
            "6/m..",
            "6/m.."
        ],
        "Space Group": "P 6/m  ",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "470": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/2)",
                "(x-y,x,z+1/2)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(x,y,-z+1/2)",
                "(-y,x-y,-z+1/2)",
                "(-x+y,-x,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-y,x-y,1/4)",
                "(-x+y,-x,1/4)",
                "(-x,-y,3/4)",
                "(y,-x+y,3/4)",
                "(x-y,x,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z+1/2)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(0,0,-z)",
                "(0,0,-z+1/2)"
            ],
            [
                "(2/3,1/3,1/4)",
                "(1/3,2/3,3/4)"
            ],
            [
                "(1/3,2/3,1/4)",
                "(2/3,1/3,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "-1",
            "3..",
            "3..",
            "-6..",
            "-6..",
            "-3..",
            "-6.."
        ],
        "Space Group": "P 6sub3/m  ",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "471": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z)",
                "(y,-x+y,z)",
                "(x-y,x,z)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)",
                "(-y,-x,-z)",
                "(-x+y,y,-z)",
                "(x,x-y,-z)"
            ],
            [
                "(x,-x,1/2)",
                "(x,2x,1/2)",
                "(-2x,-x,1/2)",
                "(-x,x,1/2)",
                "(-x,-2x,1/2)",
                "(2x,x,1/2)"
            ],
            [
                "(x,-x,0)",
                "(x,2x,0)",
                "(-2x,-x,0)",
                "(-x,x,0)",
                "(-x,-2x,0)",
                "(2x,x,0)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)",
                "(-x,0,1/2)",
                "(0,-x,1/2)",
                "(x,x,1/2)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)",
                "(-x,0,0)",
                "(0,-x,0)",
                "(x,x,0)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)",
                "(1/2,1/2,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,-z)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/3,2/3,1/2)",
                "(2/3,1/3,1/2)"
            ],
            [
                "(1/3,2/3,0)",
                "(2/3,1/3,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "6",
            "6",
            "4",
            "3",
            "3",
            "2",
            "2",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "3..",
            "222",
            "222",
            "6..",
            "3.2",
            "3.2",
            "622",
            "622"
        ],
        "Space Group": "P 6 2 2",
        "Wyckoff Letter": [
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "472": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+1/3)",
                "(-x+y,-x,z+2/3)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+5/6)",
                "(x-y,x,z+1/6)",
                "(y,x,-z+1/3)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z+2/3)",
                "(-y,-x,-z+5/6)",
                "(-x+y,y,-z+1/2)",
                "(x,x-y,-z+1/6)"
            ],
            [
                "(x,2x,1/4)",
                "(-2x,-x,7/12)",
                "(x,-x,11/12)",
                "(-x,-2x,3/4)",
                "(2x,x,1/12)",
                "(-x,x,5/12)"
            ],
            [
                "(x,0,0)",
                "(0,x,1/3)",
                "(-x,-x,2/3)",
                "(-x,0,1/2)",
                "(0,-x,5/6)",
                "(x,x,1/6)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2."
        ],
        "Space Group": "P 6sub1 2 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "473": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+2/3)",
                "(-x+y,-x,z+1/3)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/6)",
                "(x-y,x,z+5/6)",
                "(y,x,-z+2/3)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z+1/3)",
                "(-y,-x,-z+1/6)",
                "(-x+y,y,-z+1/2)",
                "(x,x-y,-z+5/6)"
            ],
            [
                "(x,2x,3/4)",
                "(-2x,-x,5/12)",
                "(x,-x,1/12)",
                "(-x,-2x,1/4)",
                "(2x,x,11/12)",
                "(-x,x,7/12)"
            ],
            [
                "(x,0,0)",
                "(0,x,2/3)",
                "(-x,-x,1/3)",
                "(-x,0,1/2)",
                "(0,-x,1/6)",
                "(x,x,5/6)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2."
        ],
        "Space Group": "P 6sub5 2 2",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "474": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+2/3)",
                "(-x+y,-x,z+1/3)",
                "(-x,-y,z)",
                "(y,-x+y,z+2/3)",
                "(x-y,x,z+1/3)",
                "(y,x,-z+2/3)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z+1/3)",
                "(-y,-x,-z+2/3)",
                "(-x+y,y,-z)",
                "(x,x-y,-z+1/3)"
            ],
            [
                "(x,2x,1/2)",
                "(-2x,-x,1/6)",
                "(x,-x,5/6)",
                "(-x,-2x,1/2)",
                "(2x,x,1/6)",
                "(-x,x,5/6)"
            ],
            [
                "(x,2x,0)",
                "(-2x,-x,2/3)",
                "(x,-x,1/3)",
                "(-x,-2x,0)",
                "(2x,x,2/3)",
                "(-x,x,1/3)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,1/6)",
                "(-x,-x,5/6)",
                "(-x,0,1/2)",
                "(0,-x,1/6)",
                "(x,x,5/6)"
            ],
            [
                "(x,0,0)",
                "(0,x,2/3)",
                "(-x,-x,1/3)",
                "(-x,0,0)",
                "(0,-x,2/3)",
                "(x,x,1/3)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z+2/3)",
                "(1/2,1/2,z+1/3)",
                "(0,1/2,-z+2/3)",
                "(1/2,0,-z)",
                "(1/2,1/2,-z+1/3)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+2/3)",
                "(0,0,z+1/3)",
                "(0,0,-z+2/3)",
                "(0,0,-z)",
                "(0,0,-z+1/3)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/6)",
                "(1/2,1/2,5/6)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,2/3)",
                "(1/2,1/2,1/3)"
            ],
            [
                "(0,0,1/2)",
                "(0,0,1/6)",
                "(0,0,5/6)"
            ],
            [
                "(0,0,0)",
                "(0,0,2/3)",
                "(0,0,1/3)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "6",
            "6",
            "6",
            "3",
            "3",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 6sub2 2 2",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "475": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z+1/3)",
                "(-x+y,-x,z+2/3)",
                "(-x,-y,z)",
                "(y,-x+y,z+1/3)",
                "(x-y,x,z+2/3)",
                "(y,x,-z+1/3)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z+2/3)",
                "(-y,-x,-z+1/3)",
                "(-x+y,y,-z)",
                "(x,x-y,-z+2/3)"
            ],
            [
                "(x,2x,1/2)",
                "(-2x,-x,5/6)",
                "(x,-x,1/6)",
                "(-x,-2x,1/2)",
                "(2x,x,5/6)",
                "(-x,x,1/6)"
            ],
            [
                "(x,2x,0)",
                "(-2x,-x,1/3)",
                "(x,-x,2/3)",
                "(-x,-2x,0)",
                "(2x,x,1/3)",
                "(-x,x,2/3)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,5/6)",
                "(-x,-x,1/6)",
                "(-x,0,1/2)",
                "(0,-x,5/6)",
                "(x,x,1/6)"
            ],
            [
                "(x,0,0)",
                "(0,x,1/3)",
                "(-x,-x,2/3)",
                "(-x,0,0)",
                "(0,-x,1/3)",
                "(x,x,2/3)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z+1/3)",
                "(1/2,1/2,z+2/3)",
                "(0,1/2,-z+1/3)",
                "(1/2,0,-z)",
                "(1/2,1/2,-z+2/3)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/3)",
                "(0,0,z+2/3)",
                "(0,0,-z+1/3)",
                "(0,0,-z)",
                "(0,0,-z+2/3)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,5/6)",
                "(1/2,1/2,1/6)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,1/3)",
                "(1/2,1/2,2/3)"
            ],
            [
                "(0,0,1/2)",
                "(0,0,5/6)",
                "(0,0,1/6)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/3)",
                "(0,0,2/3)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "6",
            "6",
            "6",
            "3",
            "3",
            "3",
            "3"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            ".2.",
            ".2.",
            "2..",
            "2..",
            "222",
            "222",
            "222",
            "222"
        ],
        "Space Group": "P 6sub4 2 2",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "476": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/2)",
                "(x-y,x,z+1/2)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)",
                "(-y,-x,-z+1/2)",
                "(-x+y,y,-z+1/2)",
                "(x,x-y,-z+1/2)"
            ],
            [
                "(x,2x,1/4)",
                "(-2x,-x,1/4)",
                "(x,-x,1/4)",
                "(-x,-2x,3/4)",
                "(2x,x,3/4)",
                "(-x,x,3/4)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)",
                "(-x,0,1/2)",
                "(0,-x,1/2)",
                "(x,x,1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z+1/2)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(0,0,-z)",
                "(0,0,-z+1/2)"
            ],
            [
                "(1/3,2/3,3/4)",
                "(2/3,1/3,1/4)"
            ],
            [
                "(1/3,2/3,1/4)",
                "(2/3,1/3,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".2.",
            "3..",
            "3..",
            "3.2",
            "3.2",
            "3.2",
            "32"
        ],
        "Space Group": "P 6sub3 2 2",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "477": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z)",
                "(y,-x+y,z)",
                "(x-y,x,z)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)",
                "(y,x,z)",
                "(x-y,-y,z)",
                "(-x,-x+y,z)"
            ],
            [
                "(x,-x,z)",
                "(x,2x,z)",
                "(-2x,-x,z)",
                "(-x,x,z)",
                "(-x,-2x,z)",
                "(2x,x,z)"
            ],
            [
                "(x,0,z)",
                "(0,x,z)",
                "(-x,-x,z)",
                "(-x,0,z)",
                "(0,-x,z)",
                "(x,x,z)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)",
                "(1/2,1/2,z)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z)"
            ],
            [
                "(0,0,z)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "3",
            "2",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "..m",
            "2mm",
            "3m.",
            "6mm"
        ],
        "Space Group": "P 6 m m",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "478": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z)",
                "(y,-x+y,z)",
                "(x-y,x,z)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)",
                "(y,x,z+1/2)",
                "(x-y,-y,z+1/2)",
                "(-x,-x+y,z+1/2)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)",
                "(1/2,1/2,z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z)",
                "(1/3,2/3,z+1/2)",
                "(2/3,1/3,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "3..",
            "6.."
        ],
        "Space Group": "P 6 c c",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "479": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/2)",
                "(x-y,x,z+1/2)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)",
                "(y,x,z)",
                "(x-y,-y,z)",
                "(-x,-x+y,z)"
            ],
            [
                "(x,0,z)",
                "(0,x,z)",
                "(-x,-x,z)",
                "(-x,0,z+1/2)",
                "(0,-x,z+1/2)",
                "(x,x,z+1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z+1/2)",
                "(1/3,2/3,z+1/2)",
                "(2/3,1/3,z)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "3..",
            "3.m"
        ],
        "Space Group": "P 6sub3 c m",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "480": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/2)",
                "(x-y,x,z+1/2)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)",
                "(y,x,z+1/2)",
                "(x-y,-y,z+1/2)",
                "(-x,-x+y,z+1/2)"
            ],
            [
                "(x,-x,z)",
                "(x,2x,z)",
                "(-2x,-x,z)",
                "(-x,x,z+1/2)",
                "(-x,-2x,z+1/2)",
                "(2x,x,z+1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "3m.",
            "3m."
        ],
        "Space Group": "P 6sub3 m c",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "481": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(x,y,-z)",
                "(-y,x-y,-z)",
                "(-x+y,-x,-z)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)",
                "(-y,-x,-z)",
                "(-x+y,y,-z)",
                "(x,x-y,-z)"
            ],
            [
                "(x,-x,z)",
                "(x,2x,z)",
                "(-2x,-x,z)",
                "(x,-x,-z)",
                "(x,2x,-z)",
                "(-2x,-x,-z)"
            ],
            [
                "(x,y,1/2)",
                "(-y,x-y,1/2)",
                "(-x+y,-x,1/2)",
                "(-y,-x,1/2)",
                "(-x+y,y,1/2)",
                "(x,x-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-y,x-y,0)",
                "(-x+y,-x,0)",
                "(-y,-x,0)",
                "(-x+y,y,0)",
                "(x,x-y,0)"
            ],
            [
                "(x,-x,1/2)",
                "(x,2x,1/2)",
                "(-2x,-x,1/2)"
            ],
            [
                "(x,-x,0)",
                "(x,2x,0)",
                "(-2x,-x,0)"
            ],
            [
                "(2/3,1/3,z)",
                "(2/3,1/3,-z)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(2/3,1/3,1/2)"
            ],
            [
                "(2/3,1/3,0)"
            ],
            [
                "(1/3,2/3,1/2)"
            ],
            [
                "(1/3,2/3,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "3",
            "3",
            "2",
            "2",
            "2",
            "1",
            "1",
            "1",
            "1",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            "m..",
            "mm2",
            "mm2",
            "3m.",
            "3m.",
            "3m.",
            "-6m2",
            "-6m2",
            "-6m2",
            "-6m2",
            "-6m2",
            "-6m2"
        ],
        "Space Group": "P -6 m 2",
        "Wyckoff Letter": [
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "482": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(x,y,-z+1/2)",
                "(-y,x-y,-z+1/2)",
                "(-x+y,-x,-z+1/2)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)",
                "(-y,-x,-z)",
                "(-x+y,y,-z)",
                "(x,x-y,-z)"
            ],
            [
                "(x,y,1/4)",
                "(-y,x-y,1/4)",
                "(-x+y,-x,1/4)",
                "(-y,-x,3/4)",
                "(-x+y,y,3/4)",
                "(x,x-y,3/4)"
            ],
            [
                "(x,-x,0)",
                "(x,2x,0)",
                "(-2x,-x,0)",
                "(x,-x,1/2)",
                "(x,2x,1/2)",
                "(-2x,-x,1/2)"
            ],
            [
                "(2/3,1/3,z)",
                "(2/3,1/3,-z+1/2)",
                "(2/3,1/3,z+1/2)",
                "(2/3,1/3,-z)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,-z+1/2)",
                "(1/3,2/3,z+1/2)",
                "(1/3,2/3,-z)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,z+1/2)",
                "(0,0,-z)"
            ],
            [
                "(2/3,1/3,1/4)",
                "(2/3,1/3,3/4)"
            ],
            [
                "(2/3,1/3,0)",
                "(2/3,1/3,1/2)"
            ],
            [
                "(1/3,2/3,1/4)",
                "(1/3,2/3,3/4)"
            ],
            [
                "(1/3,2/3,0)",
                "(1/3,2/3,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "4",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            "3..",
            "3..",
            "3..",
            "-6..",
            "3.2",
            "-6..",
            "3.2",
            "-6..",
            "3.2"
        ],
        "Space Group": "P -6 c 2",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "483": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(x,y,-z)",
                "(-y,x-y,-z)",
                "(-x+y,-x,-z)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)",
                "(y,x,z)",
                "(x-y,-y,z)",
                "(-x,-x+y,z)"
            ],
            [
                "(x,y,1/2)",
                "(-y,x-y,1/2)",
                "(-x+y,-x,1/2)",
                "(y,x,1/2)",
                "(x-y,-y,1/2)",
                "(-x,-x+y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-y,x-y,0)",
                "(-x+y,-x,0)",
                "(y,x,0)",
                "(x-y,-y,0)",
                "(-x,-x+y,0)"
            ],
            [
                "(x,0,z)",
                "(0,x,z)",
                "(-x,-x,z)",
                "(x,0,-z)",
                "(0,x,-z)",
                "(-x,-x,-z)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,-z)",
                "(2/3,1/3,-z)",
                "(2/3,1/3,z)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/3,2/3,1/2)",
                "(2/3,1/3,1/2)"
            ],
            [
                "(1/3,2/3,0)",
                "(2/3,1/3,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "4",
            "3",
            "3",
            "2",
            "2",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            "..m",
            "3..",
            "m2m",
            "m2m",
            "3.m",
            "-6..",
            "-6..",
            "-62m",
            "-62m"
        ],
        "Space Group": "P -6 2 m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "484": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(x,y,-z+1/2)",
                "(-y,x-y,-z+1/2)",
                "(-x+y,-x,-z+1/2)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)",
                "(y,x,z+1/2)",
                "(x-y,-y,z+1/2)",
                "(-x,-x+y,z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-y,x-y,1/4)",
                "(-x+y,-x,1/4)",
                "(y,x,3/4)",
                "(x-y,-y,3/4)",
                "(-x,-x+y,3/4)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)",
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(1/3,2/3,-z+1/2)",
                "(2/3,1/3,-z)",
                "(2/3,1/3,z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(2/3,1/3,1/4)",
                "(1/3,2/3,3/4)"
            ],
            [
                "(1/3,2/3,1/4)",
                "(2/3,1/3,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".2.",
            "3..",
            "3..",
            "-6..",
            "-6..",
            "-6..",
            "32"
        ],
        "Space Group": "P -6 2 c",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "485": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z)",
                "(y,-x+y,z)",
                "(x-y,x,z)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)",
                "(-y,-x,-z)",
                "(-x+y,y,-z)",
                "(x,x-y,-z)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(x,y,-z)",
                "(-y,x-y,-z)",
                "(-x+y,-x,-z)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)",
                "(y,x,z)",
                "(x-y,-y,z)",
                "(-x,-x+y,z)"
            ],
            [
                "(x,y,1/2)",
                "(-y,x-y,1/2)",
                "(-x+y,-x,1/2)",
                "(-x,-y,1/2)",
                "(y,-x+y,1/2)",
                "(x-y,x,1/2)",
                "(y,x,1/2)",
                "(x-y,-y,1/2)",
                "(-x,-x+y,1/2)",
                "(-y,-x,1/2)",
                "(-x+y,y,1/2)",
                "(x,x-y,1/2)"
            ],
            [
                "(x,y,0)",
                "(-y,x-y,0)",
                "(-x+y,-x,0)",
                "(-x,-y,0)",
                "(y,-x+y,0)",
                "(x-y,x,0)",
                "(y,x,0)",
                "(x-y,-y,0)",
                "(-x,-x+y,0)",
                "(-y,-x,0)",
                "(-x+y,y,0)",
                "(x,x-y,0)"
            ],
            [
                "(x,2x,z)",
                "(-2x,-x,z)",
                "(x,-x,z)",
                "(-x,-2x,z)",
                "(2x,x,z)",
                "(-x,x,z)",
                "(2x,x,-z)",
                "(-x,-2x,-z)",
                "(-x,x,-z)",
                "(-2x,-x,-z)",
                "(x,2x,-z)",
                "(x,-x,-z)"
            ],
            [
                "(x,0,z)",
                "(0,x,z)",
                "(-x,-x,z)",
                "(-x,0,z)",
                "(0,-x,z)",
                "(x,x,z)",
                "(0,x,-z)",
                "(x,0,-z)",
                "(-x,-x,-z)",
                "(0,-x,-z)",
                "(-x,0,-z)",
                "(x,x,-z)"
            ],
            [
                "(x,2x,1/2)",
                "(-2x,-x,1/2)",
                "(x,-x,1/2)",
                "(-x,-2x,1/2)",
                "(2x,x,1/2)",
                "(-x,x,1/2)"
            ],
            [
                "(x,2x,0)",
                "(-2x,-x,0)",
                "(x,-x,0)",
                "(-x,-2x,0)",
                "(2x,x,0)",
                "(-x,x,0)"
            ],
            [
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)",
                "(-x,0,1/2)",
                "(0,-x,1/2)",
                "(x,x,1/2)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)",
                "(-x,0,0)",
                "(0,-x,0)",
                "(x,x,0)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)",
                "(1/2,1/2,z)",
                "(0,1/2,-z)",
                "(1/2,0,-z)",
                "(1/2,1/2,-z)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,-z)"
            ],
            [
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z)"
            ],
            [
                "(1/3,2/3,1/2)",
                "(2/3,1/3,1/2)"
            ],
            [
                "(1/3,2/3,0)",
                "(2/3,1/3,0)"
            ],
            [
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "12",
            "12",
            "6",
            "6",
            "6",
            "6",
            "6",
            "4",
            "3",
            "3",
            "2",
            "2",
            "2",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".m.",
            "..m",
            "mm2",
            "mm2",
            "m2m",
            "m2m",
            "2mm",
            "3m.",
            "mmm",
            "mmm",
            "6mm",
            "-6m2",
            "-6m2",
            "6/mmm",
            "6/mmm"
        ],
        "Space Group": "P 6/m 2/m 2/m",
        "Wyckoff Letter": [
            "r",
            "q",
            "p",
            "o",
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "486": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z)",
                "(y,-x+y,z)",
                "(x-y,x,z)",
                "(y,x,-z+1/2)",
                "(x-y,-y,-z+1/2)",
                "(-x,-x+y,-z+1/2)",
                "(-y,-x,-z+1/2)",
                "(-x+y,y,-z+1/2)",
                "(x,x-y,-z+1/2)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(x,y,-z)",
                "(-y,x-y,-z)",
                "(-x+y,-x,-z)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)",
                "(y,x,z+1/2)",
                "(x-y,-y,z+1/2)",
                "(-x,-x+y,z+1/2)"
            ],
            [
                "(x,y,0)",
                "(-y,x-y,0)",
                "(-x+y,-x,0)",
                "(-x,-y,0)",
                "(y,-x+y,0)",
                "(x-y,x,0)",
                "(y,x,1/2)",
                "(x-y,-y,1/2)",
                "(-x,-x+y,1/2)",
                "(-y,-x,1/2)",
                "(-x+y,y,1/2)",
                "(x,x-y,1/2)"
            ],
            [
                "(x,2x,1/4)",
                "(-2x,-x,1/4)",
                "(x,-x,1/4)",
                "(-x,-2x,1/4)",
                "(2x,x,1/4)",
                "(-x,x,1/4)",
                "(-x,-2x,3/4)",
                "(2x,x,3/4)",
                "(-x,x,3/4)",
                "(x,2x,3/4)",
                "(-2x,-x,3/4)",
                "(x,-x,3/4)"
            ],
            [
                "(x,0,1/4)",
                "(0,x,1/4)",
                "(-x,-x,1/4)",
                "(-x,0,1/4)",
                "(0,-x,1/4)",
                "(x,x,1/4)",
                "(-x,0,3/4)",
                "(0,-x,3/4)",
                "(x,x,3/4)",
                "(x,0,3/4)",
                "(0,x,3/4)",
                "(-x,-x,3/4)"
            ],
            [
                "(1/2,0,z)",
                "(0,1/2,z)",
                "(1/2,1/2,z)",
                "(0,1/2,-z+1/2)",
                "(1/2,0,-z+1/2)",
                "(1/2,1/2,-z+1/2)",
                "(1/2,0,-z)",
                "(0,1/2,-z)",
                "(1/2,1/2,-z)",
                "(0,1/2,z+1/2)",
                "(1/2,0,z+1/2)",
                "(1/2,1/2,z+1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z)",
                "(2/3,1/3,-z+1/2)",
                "(1/3,2/3,-z+1/2)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,-z)",
                "(1/3,2/3,z+1/2)",
                "(2/3,1/3,z+1/2)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/2,0,1/4)",
                "(0,1/2,1/4)",
                "(1/2,1/2,1/4)",
                "(1/2,0,3/4)",
                "(0,1/2,3/4)",
                "(1/2,1/2,3/4)"
            ],
            [
                "(0,0,z)",
                "(0,0,-z+1/2)",
                "(0,0,-z)",
                "(0,0,z+1/2)"
            ],
            [
                "(1/3,2/3,0)",
                "(2/3,1/3,0)",
                "(2/3,1/3,1/2)",
                "(1/3,2/3,1/2)"
            ],
            [
                "(1/3,2/3,1/4)",
                "(2/3,1/3,1/4)",
                "(2/3,1/3,3/4)",
                "(1/3,2/3,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "12",
            "12",
            "8",
            "6",
            "6",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            ".2.",
            "2..",
            "3..",
            "2/m..",
            "222",
            "6..",
            "-6..",
            "3.2",
            "6/m..",
            "622"
        ],
        "Space Group": "P 6/m 2/c 2/c",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "487": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/2)",
                "(x-y,x,z+1/2)",
                "(y,x,-z+1/2)",
                "(x-y,-y,-z+1/2)",
                "(-x,-x+y,-z+1/2)",
                "(-y,-x,-z)",
                "(-x+y,y,-z)",
                "(x,x-y,-z)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(x,y,-z+1/2)",
                "(-y,x-y,-z+1/2)",
                "(-x+y,-x,-z+1/2)",
                "(-y,-x,z+1/2)",
                "(-x+y,y,z+1/2)",
                "(x,x-y,z+1/2)",
                "(y,x,z)",
                "(x-y,-y,z)",
                "(-x,-x+y,z)"
            ],
            [
                "(x,0,z)",
                "(0,x,z)",
                "(-x,-x,z)",
                "(-x,0,z+1/2)",
                "(0,-x,z+1/2)",
                "(x,x,z+1/2)",
                "(0,x,-z+1/2)",
                "(x,0,-z+1/2)",
                "(-x,-x,-z+1/2)",
                "(0,-x,-z)",
                "(-x,0,-z)",
                "(x,x,-z)"
            ],
            [
                "(x,y,1/4)",
                "(-y,x-y,1/4)",
                "(-x+y,-x,1/4)",
                "(-x,-y,3/4)",
                "(y,-x+y,3/4)",
                "(x-y,x,3/4)",
                "(y,x,1/4)",
                "(x-y,-y,1/4)",
                "(-x,-x+y,1/4)",
                "(-y,-x,3/4)",
                "(-x+y,y,3/4)",
                "(x,x-y,3/4)"
            ],
            [
                "(x,2x,0)",
                "(-2x,-x,0)",
                "(x,-x,0)",
                "(-x,-2x,1/2)",
                "(2x,x,1/2)",
                "(-x,x,1/2)",
                "(-x,-2x,0)",
                "(2x,x,0)",
                "(-x,x,0)",
                "(x,2x,1/2)",
                "(-2x,-x,1/2)",
                "(x,-x,1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z+1/2)",
                "(2/3,1/3,-z+1/2)",
                "(1/3,2/3,-z)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,-z+1/2)",
                "(1/3,2/3,z+1/2)",
                "(2/3,1/3,z)"
            ],
            [
                "(x,0,1/4)",
                "(0,x,1/4)",
                "(-x,-x,1/4)",
                "(-x,0,3/4)",
                "(0,-x,3/4)",
                "(x,x,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(0,0,-z+1/2)",
                "(0,0,-z)"
            ],
            [
                "(1/3,2/3,0)",
                "(2/3,1/3,1/2)",
                "(2/3,1/3,0)",
                "(1/3,2/3,1/2)"
            ],
            [
                "(1/3,2/3,1/4)",
                "(2/3,1/3,3/4)",
                "(2/3,1/3,1/4)",
                "(1/3,2/3,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "12",
            "8",
            "6",
            "6",
            "4",
            "4",
            "4",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "..2",
            "3..",
            "m2m",
            "..2/m",
            "3.m",
            "3.2",
            "-6..",
            "-3.m",
            "-62m"
        ],
        "Space Group": "P 6sub3/m 2/c 2/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "488": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-y,x-y,z)",
                "(-x+y,-x,z)",
                "(-x,-y,z+1/2)",
                "(y,-x+y,z+1/2)",
                "(x-y,x,z+1/2)",
                "(y,x,-z)",
                "(x-y,-y,-z)",
                "(-x,-x+y,-z)",
                "(-y,-x,-z+1/2)",
                "(-x+y,y,-z+1/2)",
                "(x,x-y,-z+1/2)",
                "(-x,-y,-z)",
                "(y,-x+y,-z)",
                "(x-y,x,-z)",
                "(x,y,-z+1/2)",
                "(-y,x-y,-z+1/2)",
                "(-x+y,-x,-z+1/2)",
                "(-y,-x,z)",
                "(-x+y,y,z)",
                "(x,x-y,z)",
                "(y,x,z+1/2)",
                "(x-y,-y,z+1/2)",
                "(-x,-x+y,z+1/2)"
            ],
            [
                "(x,2x,z)",
                "(-2x,-x,z)",
                "(x,-x,z)",
                "(-x,-2x,z+1/2)",
                "(2x,x,z+1/2)",
                "(-x,x,z+1/2)",
                "(2x,x,-z)",
                "(-x,-2x,-z)",
                "(-x,x,-z)",
                "(-2x,-x,-z+1/2)",
                "(x,2x,-z+1/2)",
                "(x,-x,-z+1/2)"
            ],
            [
                "(x,y,1/4)",
                "(-y,x-y,1/4)",
                "(-x+y,-x,1/4)",
                "(-x,-y,3/4)",
                "(y,-x+y,3/4)",
                "(x-y,x,3/4)",
                "(y,x,3/4)",
                "(x-y,-y,3/4)",
                "(-x,-x+y,3/4)",
                "(-y,-x,1/4)",
                "(-x+y,y,1/4)",
                "(x,x-y,1/4)"
            ],
            [
                "(x,0,0)",
                "(0,x,0)",
                "(-x,-x,0)",
                "(-x,0,1/2)",
                "(0,-x,1/2)",
                "(x,x,1/2)",
                "(-x,0,0)",
                "(0,-x,0)",
                "(x,x,0)",
                "(x,0,1/2)",
                "(0,x,1/2)",
                "(-x,-x,1/2)"
            ],
            [
                "(x,2x,1/4)",
                "(-2x,-x,1/4)",
                "(x,-x,1/4)",
                "(-x,-2x,3/4)",
                "(2x,x,3/4)",
                "(-x,x,3/4)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/3,2/3,z)",
                "(2/3,1/3,z+1/2)",
                "(2/3,1/3,-z)",
                "(1/3,2/3,-z+1/2)"
            ],
            [
                "(0,0,z)",
                "(0,0,z+1/2)",
                "(0,0,-z)",
                "(0,0,-z+1/2)"
            ],
            [
                "(1/3,2/3,3/4)",
                "(2/3,1/3,1/4)"
            ],
            [
                "(1/3,2/3,1/4)",
                "(2/3,1/3,3/4)"
            ],
            [
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,0,0)",
                "(0,0,1/2)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "12",
            "6",
            "6",
            "4",
            "4",
            "2",
            "2",
            "2",
            "2"
        ],
        "Site Symmetry": [
            "1",
            ".m.",
            "m..",
            ".2.",
            "mm2",
            ".2/m.",
            "3m.",
            "3m.",
            "-6m2",
            "-6m2",
            "-6m2",
            "-3m."
        ],
        "Space Group": "P 6sub3/m 2/m 2/c",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "489": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)",
                "(1/2,1/2,x)",
                "(1/2,1/2,-x)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "12",
            "6",
            "6",
            "6",
            "6",
            "4",
            "3",
            "3",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "2..",
            "2..",
            ".3.",
            "222..",
            "222..",
            "23",
            "23"
        ],
        "Space Group": "P 2 3 ",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "490": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x,3/4)",
                "(1/4,1/4,x)",
                "(3/4,1/4,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)"
            ],
            [
                "(3/4,3/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "16",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            ".3.",
            "23",
            "23",
            "23",
            "23"
        ],
        "Space Group": "F 2 3 ",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "491": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "8",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            ".3.",
            "222..",
            "23"
        ],
        "Space Group": "I 2 3 ",
        "Wyckoff Letter": [
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "492": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)"
            ]
        ],
        "Multiplicity": [
            "12",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".3."
        ],
        "Space Group": "P 2sub1 3 ",
        "Wyckoff Letter": [
            "b",
            "a"
        ]
    },
    "493": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(1/4,x,0)",
                "(3/4,-x+1/2,0)",
                "(0,1/4,x)",
                "(0,3/4,-x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            ".3."
        ],
        "Space Group": "I 2sub1 3 ",
        "Wyckoff Letter": [
            "c",
            "b",
            "a"
        ]
    },
    "494": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-z,-x,-y)",
                "(-z,x,y)",
                "(z,x,-y)",
                "(z,-x,y)",
                "(-y,-z,-x)",
                "(y,-z,x)",
                "(-y,z,x)",
                "(y,z,-x)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,z)",
                "(1/2,y,-z)",
                "(1/2,-y,-z)",
                "(z,1/2,y)",
                "(z,1/2,-y)",
                "(-z,1/2,y)",
                "(-z,1/2,-y)",
                "(y,z,1/2)",
                "(-y,z,1/2)",
                "(y,-z,1/2)",
                "(-y,-z,1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(z,0,y)",
                "(z,0,-y)",
                "(-z,0,y)",
                "(-z,0,-y)",
                "(y,z,0)",
                "(-y,z,0)",
                "(y,-z,0)",
                "(-y,-z,0)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(-x,-x,-x)",
                "(x,x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)",
                "(1/2,1/2,x)",
                "(1/2,1/2,-x)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "8",
            "6",
            "6",
            "6",
            "6",
            "3",
            "3",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "m..",
            ".3.",
            "mm2..",
            "mm2..",
            "mm2..",
            "mm2..",
            "mmm..",
            "mmm..",
            "m-3.",
            "m-3."
        ],
        "Space Group": "P 2/m -3 ",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "495": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-z+1/2,-x+1/2,-y+1/2)",
                "(-z+1/2,x+1/2,y+1/2)",
                "(z+1/2,x+1/2,-y+1/2)",
                "(z+1/2,-x+1/2,y+1/2)",
                "(-y+1/2,-z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,x+1/2)",
                "(-y+1/2,z+1/2,x+1/2)",
                "(y+1/2,z+1/2,-x+1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(-x+1/2,0,1/2)",
                "(x+1/2,0,1/2)",
                "(1/2,-x+1/2,0)",
                "(1/2,x+1/2,0)",
                "(0,1/2,-x+1/2)",
                "(0,1/2,x+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(-x+1/2,1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(1/2,x+1/2,1/2)",
                "(1/2,1/2,-x+1/2)",
                "(1/2,1/2,x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(x+1/2,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x+1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(3/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "8",
            "6",
            "4",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            ".3.",
            "222..",
            ".-3.",
            ".-3.",
            "23"
        ],
        "Space Group": "P 2/n -3 ",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "496": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(z,x,y)",
                "(z,-x+1/2,-y+1/2)",
                "(-z+1/2,-x+1/2,y)",
                "(-z+1/2,x,-y+1/2)",
                "(y,z,x)",
                "(-y+1/2,z,-x+1/2)",
                "(y,-z+1/2,-x+1/2)",
                "(-y+1/2,-z+1/2,x)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(-z,-x,-y)",
                "(-z,x+1/2,y+1/2)",
                "(z+1/2,x+1/2,-y)",
                "(z+1/2,-x,y+1/2)",
                "(-y,-z,-x)",
                "(y+1/2,-z,x+1/2)",
                "(-y,z+1/2,x+1/2)",
                "(y+1/2,z+1/2,-x)"
            ],
            [
                "(x,3/4,1/4)",
                "(-x+1/2,3/4,1/4)",
                "(1/4,x,3/4)",
                "(1/4,-x+1/2,3/4)",
                "(3/4,1/4,x)",
                "(3/4,1/4,-x+1/2)",
                "(-x,1/4,3/4)",
                "(x+1/2,1/4,3/4)",
                "(3/4,-x,1/4)",
                "(3/4,x+1/2,1/4)",
                "(1/4,3/4,-x)",
                "(1/4,3/4,x+1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,1/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x+1/2,1/4)",
                "(1/4,1/4,x)",
                "(1/4,1/4,-x+1/2)",
                "(-x,3/4,3/4)",
                "(x+1/2,3/4,3/4)",
                "(3/4,-x,3/4)",
                "(3/4,x+1/2,3/4)",
                "(3/4,3/4,-x)",
                "(3/4,3/4,x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x+1/2,x)",
                "(-x+1/2,x,-x+1/2)",
                "(x,-x+1/2,-x+1/2)",
                "(-x,-x,-x)",
                "(x+1/2,x+1/2,-x)",
                "(x+1/2,-x,x+1/2)",
                "(-x,x+1/2,x+1/2)"
            ],
            [
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(0,0,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "8",
            "6",
            "4",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            ".3.",
            "222..",
            ".-3.",
            ".-3.",
            "23"
        ],
        "Space Group": "P 2/n -3 ",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "497": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-z,-x,-y)",
                "(-z,x,y)",
                "(z,x,-y)",
                "(z,-x,y)",
                "(-y,-z,-x)",
                "(y,-z,x)",
                "(-y,z,x)",
                "(y,z,-x)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(z,0,y)",
                "(z,0,-y)",
                "(-z,0,y)",
                "(-z,0,-y)",
                "(y,z,0)",
                "(-y,z,0)",
                "(y,-z,0)",
                "(-y,-z,0)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x,3/4)",
                "(1/4,1/4,x)",
                "(3/4,1/4,-x)",
                "(-x,3/4,3/4)",
                "(x,1/4,3/4)",
                "(3/4,-x,3/4)",
                "(3/4,x,1/4)",
                "(3/4,3/4,-x)",
                "(1/4,3/4,x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(-x,-x,-x)",
                "(x,x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)",
                "(1/4,0,1/4)",
                "(1/4,0,3/4)",
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "48",
            "32",
            "24",
            "24",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "2..",
            ".3.",
            "mm2..",
            "2/m..",
            "23",
            "m-3.",
            "m-3."
        ],
        "Space Group": "F 2/m -3 ",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "498": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(-x+1/4,-y+1/4,-z+1/4)",
                "(x+1/4,y+1/4,-z+1/4)",
                "(x+1/4,-y+1/4,z+1/4)",
                "(-x+1/4,y+1/4,z+1/4)",
                "(-z+1/4,-x+1/4,-y+1/4)",
                "(-z+1/4,x+1/4,y+1/4)",
                "(z+1/4,x+1/4,-y+1/4)",
                "(z+1/4,-x+1/4,y+1/4)",
                "(-y+1/4,-z+1/4,-x+1/4)",
                "(y+1/4,-z+1/4,x+1/4)",
                "(-y+1/4,z+1/4,x+1/4)",
                "(y+1/4,z+1/4,-x+1/4)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(-x+1/4,1/4,1/4)",
                "(x+1/4,1/4,1/4)",
                "(1/4,-x+1/4,1/4)",
                "(1/4,x+1/4,1/4)",
                "(1/4,1/4,-x+1/4)",
                "(1/4,1/4,x+1/4)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(-x+1/4,-x+1/4,-x+1/4)",
                "(x+1/4,x+1/4,-x+1/4)",
                "(x+1/4,-x+1/4,x+1/4)",
                "(-x+1/4,x+1/4,x+1/4)"
            ],
            [
                "(5/8,5/8,5/8)",
                "(3/8,3/8,5/8)",
                "(3/8,5/8,3/8)",
                "(5/8,3/8,3/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,7/8,1/8)",
                "(7/8,1/8,7/8)",
                "(1/8,7/8,7/8)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/4,1/4,1/4)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "32",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            ".3.",
            ".-3.",
            ".-3.",
            "23",
            "23"
        ],
        "Space Group": "F 2/d -3 ",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "499": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/4,-y+1/4,z)",
                "(-x+1/4,y,-z+1/4)",
                "(x,-y+1/4,-z+1/4)",
                "(z,x,y)",
                "(z,-x+1/4,-y+1/4)",
                "(-z+1/4,-x+1/4,y)",
                "(-z+1/4,x,-y+1/4)",
                "(y,z,x)",
                "(-y+1/4,z,-x+1/4)",
                "(y,-z+1/4,-x+1/4)",
                "(-y+1/4,-z+1/4,x)",
                "(-x,-y,-z)",
                "(x+3/4,y+3/4,-z)",
                "(x+3/4,-y,z+3/4)",
                "(-x,y+3/4,z+3/4)",
                "(-z,-x,-y)",
                "(-z,x+3/4,y+3/4)",
                "(z+3/4,x+3/4,-y)",
                "(z+3/4,-x,y+3/4)",
                "(-y,-z,-x)",
                "(y+3/4,-z,x+3/4)",
                "(-y,z+3/4,x+3/4)",
                "(y+3/4,z+3/4,-x)"
            ],
            [
                "(x,1/8,1/8)",
                "(-x+1/4,1/8,1/8)",
                "(1/8,x,1/8)",
                "(1/8,-x+1/4,1/8)",
                "(1/8,1/8,x)",
                "(1/8,1/8,-x+1/4)",
                "(-x,7/8,7/8)",
                "(x+3/4,7/8,7/8)",
                "(7/8,-x,7/8)",
                "(7/8,x+3/4,7/8)",
                "(7/8,7/8,-x)",
                "(7/8,7/8,x+3/4)"
            ],
            [
                "(x,x,x)",
                "(-x+1/4,-x+1/4,x)",
                "(-x+1/4,x,-x+1/4)",
                "(x,-x+1/4,-x+1/4)",
                "(-x,-x,-x)",
                "(x+3/4,x+3/4,-x)",
                "(x+3/4,-x,x+3/4)",
                "(-x,x+3/4,x+3/4)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(3/4,3/4,1/2)",
                "(3/4,1/2,3/4)",
                "(1/2,3/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/4,1/4,0)",
                "(1/4,0,1/4)",
                "(0,1/4,1/4)"
            ],
            [
                "(5/8,5/8,5/8)",
                "(3/8,3/8,3/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,7/8,7/8)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "32",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            ".3.",
            ".-3.",
            ".-3.",
            "23",
            "23"
        ],
        "Space Group": "F 2/d -3 ",
        "Wyckoff Letter": [
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "500": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-z,-x,-y)",
                "(-z,x,y)",
                "(z,x,-y)",
                "(z,-x,y)",
                "(-y,-z,-x)",
                "(y,-z,x)",
                "(-y,z,x)",
                "(y,z,-x)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(z,0,y)",
                "(z,0,-y)",
                "(-z,0,y)",
                "(-z,0,-y)",
                "(y,z,0)",
                "(-y,z,0)",
                "(y,-z,0)",
                "(-y,-z,0)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(-x,-x,-x)",
                "(x,x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "16",
            "12",
            "12",
            "8",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            ".3.",
            "mm2..",
            "mm2..",
            ".-3.",
            "mmm..",
            "m-3."
        ],
        "Space Group": "I 2/m -3 ",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "501": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z)",
                "(-z,-x,-y)",
                "(-z+1/2,x+1/2,y)",
                "(z+1/2,x,-y+1/2)",
                "(z,-x+1/2,y+1/2)",
                "(-y,-z,-x)",
                "(y,-z+1/2,x+1/2)",
                "(-y+1/2,z+1/2,x)",
                "(y+1/2,z,-x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)",
                "(-x,-x,-x)",
                "(x+1/2,x,-x+1/2)",
                "(x,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "24",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            ".3.",
            ".-3.",
            ".-3."
        ],
        "Space Group": "P 2sub1/a -3 ",
        "Wyckoff Letter": [
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "502": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z)",
                "(-z,-x,-y)",
                "(-z+1/2,x+1/2,y)",
                "(z+1/2,x,-y+1/2)",
                "(z,-x+1/2,y+1/2)",
                "(-y,-z,-x)",
                "(y,-z+1/2,x+1/2)",
                "(-y+1/2,z+1/2,x)",
                "(y+1/2,z,-x+1/2)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(1/4,x,0)",
                "(3/4,-x+1/2,0)",
                "(0,1/4,x)",
                "(0,3/4,-x+1/2)",
                "(-x,0,3/4)",
                "(x+1/2,0,1/4)",
                "(3/4,-x,0)",
                "(1/4,x+1/2,0)",
                "(0,3/4,-x)",
                "(0,1/4,x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)",
                "(-x,-x,-x)",
                "(x+1/2,x,-x+1/2)",
                "(x,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,3/4,3/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            ".3.",
            ".-3.",
            ".-3."
        ],
        "Space Group": "I 2sub1/a -3 ",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "503": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(y,-x,z)",
                "(-y,x,z)",
                "(x,z,-y)",
                "(-x,z,y)",
                "(-x,-z,-y)",
                "(x,-z,y)",
                "(z,y,-x)",
                "(z,-y,x)",
                "(-z,y,x)",
                "(-z,-y,-x)"
            ],
            [
                "(1/2,y,y)",
                "(1/2,-y,y)",
                "(1/2,y,-y)",
                "(1/2,-y,-y)",
                "(y,1/2,y)",
                "(y,1/2,-y)",
                "(-y,1/2,y)",
                "(-y,1/2,-y)",
                "(y,y,1/2)",
                "(-y,y,1/2)",
                "(y,-y,1/2)",
                "(-y,-y,1/2)"
            ],
            [
                "(0,y,y)",
                "(0,-y,y)",
                "(0,y,-y)",
                "(0,-y,-y)",
                "(y,0,y)",
                "(y,0,-y)",
                "(-y,0,y)",
                "(-y,0,-y)",
                "(y,y,0)",
                "(-y,y,0)",
                "(y,-y,0)",
                "(-y,-y,0)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,1/2,-x)",
                "(0,1/2,x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x,x,-x)",
                "(-x,-x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)",
                "(1/2,1/2,x)",
                "(1/2,1/2,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "12",
            "8",
            "6",
            "6",
            "3",
            "3",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "2..",
            ".3.",
            "4..",
            "4..",
            "42.2",
            "42.2",
            "432",
            "432"
        ],
        "Space Group": "P 4 3 2",
        "Wyckoff Letter": [
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "504": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,-y+1/2)",
                "(-x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,-z+1/2,-y+1/2)",
                "(x+1/2,-z+1/2,y+1/2)",
                "(z+1/2,y+1/2,-x+1/2)",
                "(z+1/2,-y+1/2,x+1/2)",
                "(-z+1/2,y+1/2,x+1/2)",
                "(-z+1/2,-y+1/2,-x+1/2)"
            ],
            [
                "(1/4,y,y+1/2)",
                "(3/4,-y,y+1/2)",
                "(3/4,y,-y+1/2)",
                "(1/4,-y,-y+1/2)",
                "(y+1/2,1/4,y)",
                "(y+1/2,3/4,-y)",
                "(-y+1/2,3/4,y)",
                "(-y+1/2,1/4,-y)",
                "(y,y+1/2,1/4)",
                "(-y,y+1/2,3/4)",
                "(y,-y+1/2,3/4)",
                "(-y,-y+1/2,1/4)"
            ],
            [
                "(1/4,y,-y+1/2)",
                "(3/4,-y,-y+1/2)",
                "(3/4,y,y+1/2)",
                "(1/4,-y,y+1/2)",
                "(-y+1/2,1/4,y)",
                "(-y+1/2,3/4,-y)",
                "(y+1/2,3/4,y)",
                "(y+1/2,1/4,-y)",
                "(y,-y+1/2,1/4)",
                "(-y,-y+1/2,3/4)",
                "(y,y+1/2,3/4)",
                "(-y,y+1/2,1/4)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(0,x+1/2,1/2)",
                "(0,-x+1/2,1/2)",
                "(x+1/2,1/2,0)",
                "(-x+1/2,1/2,0)",
                "(1/2,0,-x+1/2)",
                "(1/2,0,x+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)",
                "(1/2,x+1/2,0)",
                "(1/2,-x+1/2,0)",
                "(x+1/2,0,1/2)",
                "(-x+1/2,0,1/2)",
                "(0,1/2,-x+1/2)",
                "(0,1/2,x+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(1/2,x+1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(1/2,1/2,-x+1/2)",
                "(1/2,1/2,x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x+1/2,x+1/2,-x+1/2)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x+1/2)"
            ],
            [
                "(1/4,1/2,0)",
                "(3/4,1/2,0)",
                "(0,1/4,1/2)",
                "(0,3/4,1/2)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)",
                "(1/2,1/4,0)",
                "(1/2,3/4,0)",
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(3/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "12",
            "12",
            "12",
            "8",
            "6",
            "6",
            "6",
            "4",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "2..",
            "2..",
            "2..",
            ".3.",
            "2.22",
            "2.22",
            "222..",
            "0.32",
            "0.32",
            "23"
        ],
        "Space Group": "P 4sub2 3 2",
        "Wyckoff Letter": [
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "505": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(y,-x,z)",
                "(-y,x,z)",
                "(x,z,-y)",
                "(-x,z,y)",
                "(-x,-z,-y)",
                "(x,-z,y)",
                "(z,y,-x)",
                "(z,-y,x)",
                "(-z,y,x)",
                "(-z,-y,-x)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x,3/4)",
                "(1/4,1/4,x)",
                "(3/4,1/4,-x)",
                "(1/4,x,3/4)",
                "(3/4,-x,3/4)",
                "(x,1/4,3/4)",
                "(-x,1/4,1/4)",
                "(1/4,1/4,-x)",
                "(1/4,3/4,x)"
            ],
            [
                "(1/2,y,y)",
                "(1/2,-y,y)",
                "(1/2,y,-y)",
                "(1/2,-y,-y)",
                "(y,1/2,y)",
                "(y,1/2,-y)",
                "(-y,1/2,y)",
                "(-y,1/2,-y)",
                "(y,y,1/2)",
                "(-y,y,1/2)",
                "(y,-y,1/2)",
                "(-y,-y,1/2)"
            ],
            [
                "(0,y,y)",
                "(0,-y,y)",
                "(0,y,-y)",
                "(0,-y,-y)",
                "(y,0,y)",
                "(y,0,-y)",
                "(-y,0,y)",
                "(-y,0,-y)",
                "(y,y,0)",
                "(-y,y,0)",
                "(y,-y,0)",
                "(-y,-y,0)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x,x,-x)",
                "(-x,-x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)",
                "(1/4,0,1/4)",
                "(1/4,0,3/4)",
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "48",
            "48",
            "32",
            "24",
            "24",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "..2",
            "..2",
            ".3.",
            "4..",
            "2.22",
            "23",
            "432",
            "432"
        ],
        "Space Group": "F 4 3 2",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "506": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y,-z+1/2)",
                "(z,x,y)",
                "(z+1/2,-x,-y+1/2)",
                "(-z,-x+1/2,y+1/2)",
                "(-z+1/2,x+1/2,-y)",
                "(y,z,x)",
                "(-y+1/2,z+1/2,-x)",
                "(y+1/2,-z,-x+1/2)",
                "(-y,-z+1/2,x+1/2)",
                "(y+3/4,x+1/4,-z+3/4)",
                "(-y+1/4,-x+1/4,-z+1/4)",
                "(y+1/4,-x+3/4,z+3/4)",
                "(-y+3/4,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-y+3/4)",
                "(-x+3/4,z+3/4,y+1/4)",
                "(-x+1/4,-z+1/4,-y+1/4)",
                "(x+1/4,-z+3/4,y+3/4)",
                "(z+3/4,y+1/4,-x+3/4)",
                "(z+1/4,-y+3/4,x+3/4)",
                "(-z+3/4,y+3/4,x+1/4)",
                "(-z+1/4,-y+1/4,-x+1/4)"
            ],
            [
                "(1/8,y,-y+1/4)",
                "(7/8,-y+1/2,-y+3/4)",
                "(3/8,y+1/2,y+3/4)",
                "(5/8,-y,y+1/4)",
                "(-y+1/4,1/8,y)",
                "(-y+3/4,7/8,-y+1/2)",
                "(y+3/4,3/8,y+1/2)",
                "(y+1/4,5/8,-y)",
                "(y,-y+1/4,1/8)",
                "(-y+1/2,-y+3/4,7/8)",
                "(y+1/2,y+3/4,3/8)",
                "(-y,y+1/4,5/8)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)",
                "(0,x,0)",
                "(1/2,-x,1/2)",
                "(0,0,x)",
                "(1/2,1/2,-x)",
                "(3/4,x+1/4,3/4)",
                "(1/4,-x+1/4,1/4)",
                "(x+3/4,1/4,3/4)",
                "(-x+3/4,3/4,1/4)",
                "(3/4,1/4,-x+3/4)",
                "(1/4,3/4,x+3/4)"
            ],
            [
                "(x,x,x)",
                "(-x,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,-x)",
                "(x+1/2,-x,-x+1/2)",
                "(x+3/4,x+1/4,-x+3/4)",
                "(-x+1/4,-x+1/4,-x+1/4)",
                "(x+1/4,-x+3/4,x+3/4)",
                "(-x+3/4,x+3/4,x+1/4)"
            ],
            [
                "(5/8,5/8,5/8)",
                "(3/8,7/8,1/8)",
                "(7/8,1/8,3/8)",
                "(1/8,3/8,7/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,3/8,5/8)",
                "(3/8,5/8,7/8)",
                "(5/8,7/8,3/8)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(0,0,0)",
                "(3/4,1/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "48",
            "32",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            ".3.",
            "0.32",
            "0.32",
            "23",
            "23"
        ],
        "Space Group": "F 4sub1 3 2",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "507": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(y,-x,z)",
                "(-y,x,z)",
                "(x,z,-y)",
                "(-x,z,y)",
                "(-x,-z,-y)",
                "(x,-z,y)",
                "(z,y,-x)",
                "(z,-y,x)",
                "(-z,y,x)",
                "(-z,-y,-x)"
            ],
            [
                "(1/4,y,-y+1/2)",
                "(3/4,-y,-y+1/2)",
                "(3/4,y,y+1/2)",
                "(1/4,-y,y+1/2)",
                "(-y+1/2,1/4,y)",
                "(-y+1/2,3/4,-y)",
                "(y+1/2,3/4,y)",
                "(y+1/2,1/4,-y)",
                "(y,-y+1/2,1/4)",
                "(-y,-y+1/2,3/4)",
                "(y,y+1/2,3/4)",
                "(-y,y+1/2,1/4)"
            ],
            [
                "(0,y,y)",
                "(0,-y,y)",
                "(0,y,-y)",
                "(0,-y,-y)",
                "(y,0,y)",
                "(y,0,-y)",
                "(-y,0,y)",
                "(-y,0,-y)",
                "(y,y,0)",
                "(-y,y,0)",
                "(y,-y,0)",
                "(-y,-y,0)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,1/2,-x)",
                "(0,1/2,x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x,x,-x)",
                "(-x,-x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(1/4,1/2,0)",
                "(3/4,1/2,0)",
                "(0,1/4,1/2)",
                "(0,3/4,1/2)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "24",
            "16",
            "12",
            "12",
            "8",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "2..",
            ".3.",
            "4..",
            "2.22",
            "0.32",
            "42.2",
            "432"
        ],
        "Space Group": "I 4 3 2",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "508": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)",
                "(y+1/4,x+3/4,-z+3/4)",
                "(-y+1/4,-x+1/4,-z+1/4)",
                "(y+3/4,-x+3/4,z+1/4)",
                "(-y+3/4,x+1/4,z+3/4)",
                "(x+1/4,z+3/4,-y+3/4)",
                "(-x+3/4,z+1/4,y+3/4)",
                "(-x+1/4,-z+1/4,-y+1/4)",
                "(x+3/4,-z+3/4,y+1/4)",
                "(z+1/4,y+3/4,-x+3/4)",
                "(z+3/4,-y+3/4,x+1/4)",
                "(-z+3/4,y+1/4,x+3/4)",
                "(-z+1/4,-y+1/4,-x+1/4)"
            ],
            [
                "(1/8,y,-y+1/4)",
                "(3/8,-y,-y+3/4)",
                "(7/8,y+1/2,y+1/4)",
                "(5/8,-y+1/2,y+3/4)",
                "(-y+1/4,1/8,y)",
                "(-y+3/4,3/8,-y)",
                "(y+1/4,7/8,y+1/2)",
                "(y+3/4,5/8,-y+1/2)",
                "(y,-y+1/4,1/8)",
                "(-y,-y+3/4,3/8)",
                "(y+1/2,y+1/4,7/8)",
                "(-y+1/2,y+3/4,5/8)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)",
                "(x+1/4,x+3/4,-x+3/4)",
                "(-x+1/4,-x+1/4,-x+1/4)",
                "(x+3/4,-x+3/4,x+1/4)",
                "(-x+3/4,x+1/4,x+3/4)"
            ],
            [
                "(5/8,5/8,5/8)",
                "(7/8,3/8,1/8)",
                "(3/8,1/8,7/8)",
                "(1/8,7/8,3/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(3/8,7/8,5/8)",
                "(7/8,5/8,3/8)",
                "(5/8,3/8,7/8)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".3.",
            "0.32",
            "0.32"
        ],
        "Space Group": "P 4sub3 3 2",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "509": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)",
                "(y+3/4,x+1/4,-z+1/4)",
                "(-y+3/4,-x+3/4,-z+3/4)",
                "(y+1/4,-x+1/4,z+3/4)",
                "(-y+1/4,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-y+1/4)",
                "(-x+1/4,z+3/4,y+1/4)",
                "(-x+3/4,-z+3/4,-y+3/4)",
                "(x+1/4,-z+1/4,y+3/4)",
                "(z+3/4,y+1/4,-x+1/4)",
                "(z+1/4,-y+1/4,x+3/4)",
                "(-z+1/4,y+3/4,x+1/4)",
                "(-z+3/4,-y+3/4,-x+3/4)"
            ],
            [
                "(1/8,y,y+1/4)",
                "(3/8,-y,y+3/4)",
                "(7/8,y+1/2,-y+1/4)",
                "(5/8,-y+1/2,-y+3/4)",
                "(y+1/4,1/8,y)",
                "(y+3/4,3/8,-y)",
                "(-y+1/4,7/8,y+1/2)",
                "(-y+3/4,5/8,-y+1/2)",
                "(y,y+1/4,1/8)",
                "(-y,y+3/4,3/8)",
                "(y+1/2,-y+1/4,7/8)",
                "(-y+1/2,-y+3/4,5/8)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)",
                "(x+3/4,x+1/4,-x+1/4)",
                "(-x+3/4,-x+3/4,-x+3/4)",
                "(x+1/4,-x+1/4,x+3/4)",
                "(-x+1/4,x+3/4,x+1/4)"
            ],
            [
                "(7/8,7/8,7/8)",
                "(5/8,1/8,3/8)",
                "(1/8,3/8,5/8)",
                "(3/8,5/8,1/8)"
            ],
            [
                "(3/8,3/8,3/8)",
                "(1/8,5/8,7/8)",
                "(5/8,7/8,1/8)",
                "(7/8,1/8,5/8)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            ".3.",
            "0.32",
            "0.32"
        ],
        "Space Group": "P 4sub1 3 2",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "510": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)",
                "(y+3/4,x+1/4,-z+1/4)",
                "(-y+3/4,-x+3/4,-z+3/4)",
                "(y+1/4,-x+1/4,z+3/4)",
                "(-y+1/4,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-y+1/4)",
                "(-x+1/4,z+3/4,y+1/4)",
                "(-x+3/4,-z+3/4,-y+3/4)",
                "(x+1/4,-z+1/4,y+3/4)",
                "(z+3/4,y+1/4,-x+1/4)",
                "(z+1/4,-y+1/4,x+3/4)",
                "(-z+1/4,y+3/4,x+1/4)",
                "(-z+3/4,-y+3/4,-x+3/4)"
            ],
            [
                "(1/8,y,-y+1/4)",
                "(3/8,-y,-y+3/4)",
                "(7/8,y+1/2,y+1/4)",
                "(5/8,-y+1/2,y+3/4)",
                "(-y+1/4,1/8,y)",
                "(-y+3/4,3/8,-y)",
                "(y+1/4,7/8,y+1/2)",
                "(y+3/4,5/8,-y+1/2)",
                "(y,-y+1/4,1/8)",
                "(-y,-y+3/4,3/8)",
                "(y+1/2,y+1/4,7/8)",
                "(-y+1/2,y+3/4,5/8)"
            ],
            [
                "(1/8,y,y+1/4)",
                "(3/8,-y,y+3/4)",
                "(7/8,y+1/2,-y+1/4)",
                "(5/8,-y+1/2,-y+3/4)",
                "(y+1/4,1/8,y)",
                "(y+3/4,3/8,-y)",
                "(-y+1/4,7/8,y+1/2)",
                "(-y+3/4,5/8,-y+1/2)",
                "(y,y+1/4,1/8)",
                "(-y,y+3/4,3/8)",
                "(y+1/2,-y+1/4,7/8)",
                "(-y+1/2,-y+3/4,5/8)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(1/4,x,0)",
                "(3/4,-x+1/2,0)",
                "(0,1/4,x)",
                "(0,3/4,-x+1/2)",
                "(3/4,x+1/4,0)",
                "(3/4,-x+3/4,1/2)",
                "(x+3/4,1/2,1/4)",
                "(-x+1/4,0,1/4)",
                "(0,1/4,-x+1/4)",
                "(1/2,1/4,x+3/4)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)",
                "(x+3/4,x+1/4,-x+1/4)",
                "(-x+3/4,-x+3/4,-x+3/4)",
                "(x+1/4,-x+1/4,x+3/4)",
                "(-x+1/4,x+3/4,x+1/4)"
            ],
            [
                "(5/8,0,1/4)",
                "(7/8,0,3/4)",
                "(1/4,5/8,0)",
                "(3/4,7/8,0)",
                "(0,1/4,5/8)",
                "(0,3/4,7/8)"
            ],
            [
                "(1/8,0,1/4)",
                "(3/8,0,3/4)",
                "(1/4,1/8,0)",
                "(3/4,3/8,0)",
                "(0,1/4,1/8)",
                "(0,3/4,3/8)"
            ],
            [
                "(7/8,7/8,7/8)",
                "(5/8,1/8,3/8)",
                "(1/8,3/8,5/8)",
                "(3/8,5/8,1/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(3/8,7/8,5/8)",
                "(7/8,5/8,3/8)",
                "(5/8,3/8,7/8)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "24",
            "16",
            "12",
            "12",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..2",
            "2..",
            ".3.",
            "2.22",
            "2.22",
            "0.32",
            "0.32"
        ],
        "Space Group": "I 4sub1 3 2",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "511": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,z)",
                "(-y,-x,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,z,y)",
                "(-x,z,-y)",
                "(-x,-z,y)",
                "(x,-z,-y)",
                "(z,y,x)",
                "(z,-y,-x)",
                "(-z,y,-x)",
                "(-z,-y,x)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(z,x,x)",
                "(z,-x,-x)",
                "(-z,-x,x)",
                "(-z,x,-x)",
                "(x,z,x)",
                "(-x,z,-x)",
                "(x,-z,-x)",
                "(-x,-z,x)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,1/2,x)",
                "(0,1/2,-x)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)",
                "(1/2,1/2,x)",
                "(1/2,1/2,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "6",
            "6",
            "4",
            "3",
            "3",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "2.mm",
            "2.mm",
            ".3m",
            "-42.m",
            "-42.m",
            "-43m",
            "-43m"
        ],
        "Space Group": "P -4 3 m",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "512": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,z)",
                "(-y,-x,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,z,y)",
                "(-x,z,-y)",
                "(-x,-z,y)",
                "(x,-z,-y)",
                "(z,y,x)",
                "(z,-y,-x)",
                "(-z,y,-x)",
                "(-z,-y,x)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(z,x,x)",
                "(z,-x,-x)",
                "(-z,-x,x)",
                "(-z,x,-x)",
                "(x,z,x)",
                "(-x,z,-x)",
                "(x,-z,-x)",
                "(-x,-z,x)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x,3/4)",
                "(1/4,1/4,x)",
                "(3/4,1/4,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)"
            ],
            [
                "(3/4,3/4,3/4)"
            ],
            [
                "(1/4,1/4,1/4)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "24",
            "24",
            "16",
            "4",
            "4",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2.mm",
            "2.mm",
            ".3m",
            "-43m",
            "-43m",
            "-43m",
            "-43m"
        ],
        "Space Group": "F -4 3 m",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "513": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,z)",
                "(-y,-x,z)",
                "(y,-x,-z)",
                "(-y,x,-z)",
                "(x,z,y)",
                "(-x,z,-y)",
                "(-x,-z,y)",
                "(x,-z,-y)",
                "(z,y,x)",
                "(z,-y,-x)",
                "(-z,y,-x)",
                "(-z,-y,x)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(z,x,x)",
                "(z,-x,-x)",
                "(-z,-x,x)",
                "(-z,x,-x)",
                "(x,z,x)",
                "(-x,z,-x)",
                "(x,-z,-x)",
                "(-x,-z,x)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,1/2,x)",
                "(0,1/2,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(1/4,1/2,0)",
                "(3/4,1/2,0)",
                "(0,1/4,1/2)",
                "(0,3/4,1/2)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "12",
            "12",
            "8",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "2..",
            "2.mm",
            "-4..",
            ".3m",
            "-42.m",
            "-43m"
        ],
        "Space Group": "I -4 3 m",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "514": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y+1/2,x+1/2,z+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,-z+1/2)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,z+1/2,-y+1/2)",
                "(-x+1/2,-z+1/2,y+1/2)",
                "(x+1/2,-z+1/2,-y+1/2)",
                "(z+1/2,y+1/2,x+1/2)",
                "(z+1/2,-y+1/2,-x+1/2)",
                "(-z+1/2,y+1/2,-x+1/2)",
                "(-z+1/2,-y+1/2,x+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)",
                "(1/2,x+1/2,0)",
                "(1/2,-x+1/2,0)",
                "(x+1/2,0,1/2)",
                "(-x+1/2,0,1/2)",
                "(0,1/2,x+1/2)",
                "(0,1/2,-x+1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(0,x+1/2,1/2)",
                "(0,-x+1/2,1/2)",
                "(x+1/2,1/2,0)",
                "(-x+1/2,1/2,0)",
                "(1/2,0,x+1/2)",
                "(1/2,0,-x+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(1/2,x+1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(1/2,1/2,x+1/2)",
                "(1/2,1/2,-x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x+1/2,x+1/2,x+1/2)",
                "(-x+1/2,-x+1/2,x+1/2)",
                "(x+1/2,-x+1/2,-x+1/2)",
                "(-x+1/2,x+1/2,-x+1/2)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)",
                "(1/2,1/4,0)",
                "(1/2,3/4,0)",
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(1/4,1/2,0)",
                "(3/4,1/2,0)",
                "(0,1/4,1/2)",
                "(0,3/4,1/2)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "24",
            "12",
            "12",
            "12",
            "8",
            "6",
            "6",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            "2..",
            ".3.",
            "-4..",
            "-4..",
            "222..",
            "23"
        ],
        "Space Group": "P -4 3 n",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "515": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y+1/2,x+1/2,z+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,-x+1/2,-z+1/2)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,z+1/2,-y+1/2)",
                "(-x+1/2,-z+1/2,y+1/2)",
                "(x+1/2,-z+1/2,-y+1/2)",
                "(z+1/2,y+1/2,x+1/2)",
                "(z+1/2,-y+1/2,-x+1/2)",
                "(-z+1/2,y+1/2,-x+1/2)",
                "(-z+1/2,-y+1/2,x+1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x,3/4)",
                "(1/4,1/4,x)",
                "(3/4,1/4,-x)",
                "(3/4,x+1/2,3/4)",
                "(1/4,-x+1/2,3/4)",
                "(x+1/2,3/4,3/4)",
                "(-x+1/2,3/4,1/4)",
                "(3/4,3/4,x+1/2)",
                "(3/4,1/4,-x+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(1/2,x+1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(1/2,1/2,x+1/2)",
                "(1/2,1/2,-x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x+1/2,x+1/2,x+1/2)",
                "(-x+1/2,-x+1/2,x+1/2)",
                "(x+1/2,-x+1/2,-x+1/2)",
                "(-x+1/2,x+1/2,-x+1/2)"
            ],
            [
                "(1/4,0,0)",
                "(3/4,0,0)",
                "(0,1/4,0)",
                "(0,3/4,0)",
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)",
                "(1/4,0,1/4)",
                "(1/4,0,3/4)",
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "48",
            "32",
            "24",
            "24",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            "2..",
            ".3.",
            "-4..",
            "-4..",
            "23",
            "23"
        ],
        "Space Group": "F -4 3 c",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "516": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)",
                "(y+1/4,x+1/4,z+1/4)",
                "(-y+1/4,-x+3/4,z+3/4)",
                "(y+3/4,-x+1/4,-z+3/4)",
                "(-y+3/4,x+3/4,-z+1/4)",
                "(x+1/4,z+1/4,y+1/4)",
                "(-x+3/4,z+3/4,-y+1/4)",
                "(-x+1/4,-z+3/4,y+3/4)",
                "(x+3/4,-z+1/4,-y+3/4)",
                "(z+1/4,y+1/4,x+1/4)",
                "(z+3/4,-y+1/4,-x+3/4)",
                "(-z+3/4,y+3/4,-x+1/4)",
                "(-z+1/4,-y+3/4,x+3/4)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(1/4,x,0)",
                "(3/4,-x+1/2,0)",
                "(0,1/4,x)",
                "(0,3/4,-x+1/2)",
                "(1/4,x+1/4,1/2)",
                "(1/4,-x+3/4,0)",
                "(x+1/4,1/2,1/4)",
                "(-x+3/4,0,1/4)",
                "(1/2,1/4,x+1/4)",
                "(0,1/4,-x+3/4)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)",
                "(x+1/4,x+1/4,x+1/4)",
                "(-x+1/4,-x+3/4,x+3/4)",
                "(x+3/4,-x+1/4,-x+3/4)",
                "(-x+3/4,x+3/4,-x+1/4)"
            ],
            [
                "(7/8,0,1/4)",
                "(5/8,0,3/4)",
                "(1/4,7/8,0)",
                "(3/4,5/8,0)",
                "(0,1/4,7/8)",
                "(0,3/4,5/8)"
            ],
            [
                "(3/8,0,1/4)",
                "(1/8,0,3/4)",
                "(1/4,3/8,0)",
                "(3/4,1/8,0)",
                "(0,1/4,3/8)",
                "(0,3/4,1/8)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "16",
            "12",
            "12"
        ],
        "Site Symmetry": [
            "1",
            "2..",
            ".3.",
            "-4..",
            "-4.."
        ],
        "Space Group": "I -4 3 d",
        "Wyckoff Letter": [
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "517": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(y,-x,z)",
                "(-y,x,z)",
                "(x,z,-y)",
                "(-x,z,y)",
                "(-x,-z,-y)",
                "(x,-z,y)",
                "(z,y,-x)",
                "(z,-y,x)",
                "(-z,y,x)",
                "(-z,-y,-x)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-z,-x,-y)",
                "(-z,x,y)",
                "(z,x,-y)",
                "(z,-x,y)",
                "(-y,-z,-x)",
                "(y,-z,x)",
                "(-y,z,x)",
                "(y,z,-x)",
                "(-y,-x,z)",
                "(y,x,z)",
                "(-y,x,-z)",
                "(y,-x,-z)",
                "(-x,-z,y)",
                "(x,-z,-y)",
                "(x,z,y)",
                "(-x,z,-y)",
                "(-z,-y,x)",
                "(-z,y,-x)",
                "(z,-y,-x)",
                "(z,y,x)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(z,x,x)",
                "(z,-x,-x)",
                "(-z,-x,x)",
                "(-z,x,-x)",
                "(x,z,x)",
                "(-x,z,-x)",
                "(x,-z,-x)",
                "(-x,-z,x)",
                "(x,x,-z)",
                "(-x,-x,-z)",
                "(x,-x,z)",
                "(-x,x,z)",
                "(x,z,-x)",
                "(-x,z,x)",
                "(-x,-z,-x)",
                "(x,-z,x)",
                "(z,x,-x)",
                "(z,-x,x)",
                "(-z,x,x)",
                "(-z,-x,-x)"
            ],
            [
                "(1/2,y,z)",
                "(1/2,-y,z)",
                "(1/2,y,-z)",
                "(1/2,-y,-z)",
                "(z,1/2,y)",
                "(z,1/2,-y)",
                "(-z,1/2,y)",
                "(-z,1/2,-y)",
                "(y,z,1/2)",
                "(-y,z,1/2)",
                "(y,-z,1/2)",
                "(-y,-z,1/2)",
                "(y,1/2,-z)",
                "(-y,1/2,-z)",
                "(y,1/2,z)",
                "(-y,1/2,z)",
                "(1/2,z,-y)",
                "(1/2,z,y)",
                "(1/2,-z,-y)",
                "(1/2,-z,y)",
                "(z,y,1/2)",
                "(z,-y,1/2)",
                "(-z,y,1/2)",
                "(-z,-y,1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(z,0,y)",
                "(z,0,-y)",
                "(-z,0,y)",
                "(-z,0,-y)",
                "(y,z,0)",
                "(-y,z,0)",
                "(y,-z,0)",
                "(-y,-z,0)",
                "(y,0,-z)",
                "(-y,0,-z)",
                "(y,0,z)",
                "(-y,0,z)",
                "(0,z,-y)",
                "(0,z,y)",
                "(0,-z,-y)",
                "(0,-z,y)",
                "(z,y,0)",
                "(z,-y,0)",
                "(-z,y,0)",
                "(-z,-y,0)"
            ],
            [
                "(1/2,y,y)",
                "(1/2,-y,y)",
                "(1/2,y,-y)",
                "(1/2,-y,-y)",
                "(y,1/2,y)",
                "(y,1/2,-y)",
                "(-y,1/2,y)",
                "(-y,1/2,-y)",
                "(y,y,1/2)",
                "(-y,y,1/2)",
                "(y,-y,1/2)",
                "(-y,-y,1/2)"
            ],
            [
                "(0,y,y)",
                "(0,-y,y)",
                "(0,y,-y)",
                "(0,-y,-y)",
                "(y,0,y)",
                "(y,0,-y)",
                "(-y,0,y)",
                "(-y,0,-y)",
                "(y,y,0)",
                "(-y,y,0)",
                "(y,-y,0)",
                "(-y,-y,0)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(0,1/2,-x)",
                "(0,1/2,x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x,x,-x)",
                "(-x,-x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,1/2,1/2)",
                "(-x,1/2,1/2)",
                "(1/2,x,1/2)",
                "(1/2,-x,1/2)",
                "(1/2,1/2,x)",
                "(1/2,1/2,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "24",
            "12",
            "12",
            "12",
            "8",
            "6",
            "6",
            "3",
            "3",
            "1",
            "1"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "m..",
            "m.m2",
            "m.m2",
            "mm2..",
            ".3m",
            "4m.m",
            "4m.m",
            "4/mm.m",
            "4/mm.m",
            "m-3m",
            "m-3m"
        ],
        "Space Group": "P 4/m -3 2/m",
        "Wyckoff Letter": [
            "n",
            "m",
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "518": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(y,-x,z)",
                "(-y,x,z)",
                "(x,z,-y)",
                "(-x,z,y)",
                "(-x,-z,-y)",
                "(x,-z,y)",
                "(z,y,-x)",
                "(z,-y,x)",
                "(-z,y,x)",
                "(-z,-y,-x)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-z+1/2,-x+1/2,-y+1/2)",
                "(-z+1/2,x+1/2,y+1/2)",
                "(z+1/2,x+1/2,-y+1/2)",
                "(z+1/2,-x+1/2,y+1/2)",
                "(-y+1/2,-z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,x+1/2)",
                "(-y+1/2,z+1/2,x+1/2)",
                "(y+1/2,z+1/2,-x+1/2)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,-z+1/2)",
                "(-x+1/2,-z+1/2,y+1/2)",
                "(x+1/2,-z+1/2,-y+1/2)",
                "(x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,z+1/2,-y+1/2)",
                "(-z+1/2,-y+1/2,x+1/2)",
                "(-z+1/2,y+1/2,-x+1/2)",
                "(z+1/2,-y+1/2,-x+1/2)",
                "(z+1/2,y+1/2,x+1/2)"
            ],
            [
                "(0,y,y)",
                "(0,-y,y)",
                "(0,y,-y)",
                "(0,-y,-y)",
                "(y,0,y)",
                "(y,0,-y)",
                "(-y,0,y)",
                "(-y,0,-y)",
                "(y,y,0)",
                "(-y,y,0)",
                "(y,-y,0)",
                "(-y,-y,0)",
                "(1/2,-y+1/2,-y+1/2)",
                "(1/2,y+1/2,-y+1/2)",
                "(1/2,-y+1/2,y+1/2)",
                "(1/2,y+1/2,y+1/2)",
                "(-y+1/2,1/2,-y+1/2)",
                "(-y+1/2,1/2,y+1/2)",
                "(y+1/2,1/2,-y+1/2)",
                "(y+1/2,1/2,y+1/2)",
                "(-y+1/2,-y+1/2,1/2)",
                "(y+1/2,-y+1/2,1/2)",
                "(-y+1/2,y+1/2,1/2)",
                "(y+1/2,y+1/2,1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(1/2,0,-x)",
                "(1/2,0,x)",
                "(-x+1/2,1/2,0)",
                "(x+1/2,1/2,0)",
                "(0,-x+1/2,1/2)",
                "(0,x+1/2,1/2)",
                "(1/2,0,-x+1/2)",
                "(1/2,0,x+1/2)",
                "(1/2,-x+1/2,0)",
                "(1/2,x+1/2,0)",
                "(-x+1/2,0,1/2)",
                "(x+1/2,0,1/2)",
                "(0,1/2,x+1/2)",
                "(0,1/2,-x+1/2)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x,x,-x)",
                "(-x,-x,-x)",
                "(x,-x,x)",
                "(-x,x,x)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(x+1/2,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x+1/2)",
                "(-x+1/2,-x+1/2,x+1/2)",
                "(x+1/2,x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(-x+1/2,1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(1/2,x+1/2,1/2)",
                "(1/2,1/2,-x+1/2)",
                "(1/2,1/2,x+1/2)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)",
                "(1/2,1/4,0)",
                "(1/2,3/4,0)",
                "(0,1/2,1/4)",
                "(0,1/2,3/4)",
                "(0,1/4,1/2)",
                "(0,3/4,1/2)",
                "(1/4,1/2,0)",
                "(3/4,1/2,0)",
                "(1/2,0,3/4)",
                "(1/2,0,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(3/4,3/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(1/2,0,0)",
                "(0,1/2,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "16",
            "12",
            "12",
            "8",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            ".3.",
            "4..",
            "-4..",
            ".-3.",
            "42.2",
            "432"
        ],
        "Space Group": "P 4/n -3 2/n",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "519": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(z,x,y)",
                "(z,-x+1/2,-y+1/2)",
                "(-z+1/2,-x+1/2,y)",
                "(-z+1/2,x,-y+1/2)",
                "(y,z,x)",
                "(-y+1/2,z,-x+1/2)",
                "(y,-z+1/2,-x+1/2)",
                "(-y+1/2,-z+1/2,x)",
                "(y,x,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(y,-x+1/2,z)",
                "(-y+1/2,x,z)",
                "(x,z,-y+1/2)",
                "(-x+1/2,z,y)",
                "(-x+1/2,-z+1/2,-y+1/2)",
                "(x,-z+1/2,y)",
                "(z,y,-x+1/2)",
                "(z,-y+1/2,x)",
                "(-z+1/2,y,x)",
                "(-z+1/2,-y+1/2,-x+1/2)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(-z,-x,-y)",
                "(-z,x+1/2,y+1/2)",
                "(z+1/2,x+1/2,-y)",
                "(z+1/2,-x,y+1/2)",
                "(-y,-z,-x)",
                "(y+1/2,-z,x+1/2)",
                "(-y,z+1/2,x+1/2)",
                "(y+1/2,z+1/2,-x)",
                "(-y,-x,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)",
                "(-y,x+1/2,-z)",
                "(y+1/2,-x,-z)",
                "(-x,-z,y+1/2)",
                "(x+1/2,-z,-y)",
                "(x+1/2,z+1/2,y+1/2)",
                "(-x,z+1/2,-y)",
                "(-z,-y,x+1/2)",
                "(-z,y+1/2,-x)",
                "(z+1/2,-y,-x)",
                "(z+1/2,y+1/2,x+1/2)"
            ],
            [
                "(1/4,y,y)",
                "(1/4,-y+1/2,y)",
                "(1/4,y,-y+1/2)",
                "(1/4,-y+1/2,-y+1/2)",
                "(y,1/4,y)",
                "(y,1/4,-y+1/2)",
                "(-y+1/2,1/4,y)",
                "(-y+1/2,1/4,-y+1/2)",
                "(y,y,1/4)",
                "(-y+1/2,y,1/4)",
                "(y,-y+1/2,1/4)",
                "(-y+1/2,-y+1/2,1/4)",
                "(3/4,-y,-y)",
                "(3/4,y+1/2,-y)",
                "(3/4,-y,y+1/2)",
                "(3/4,y+1/2,y+1/2)",
                "(-y,3/4,-y)",
                "(-y,3/4,y+1/2)",
                "(y+1/2,3/4,-y)",
                "(y+1/2,3/4,y+1/2)",
                "(-y,-y,3/4)",
                "(y+1/2,-y,3/4)",
                "(-y,y+1/2,3/4)",
                "(y+1/2,y+1/2,3/4)"
            ],
            [
                "(x,3/4,1/4)",
                "(-x+1/2,3/4,1/4)",
                "(1/4,x,3/4)",
                "(1/4,-x+1/2,3/4)",
                "(3/4,1/4,x)",
                "(3/4,1/4,-x+1/2)",
                "(3/4,x,1/4)",
                "(3/4,-x+1/2,1/4)",
                "(x,1/4,3/4)",
                "(-x+1/2,1/4,3/4)",
                "(1/4,3/4,-x+1/2)",
                "(1/4,3/4,x)",
                "(-x,1/4,3/4)",
                "(x+1/2,1/4,3/4)",
                "(3/4,-x,1/4)",
                "(3/4,x+1/2,1/4)",
                "(1/4,3/4,-x)",
                "(1/4,3/4,x+1/2)",
                "(1/4,-x,3/4)",
                "(1/4,x+1/2,3/4)",
                "(-x,3/4,1/4)",
                "(x+1/2,3/4,1/4)",
                "(3/4,1/4,x+1/2)",
                "(3/4,1/4,-x)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x+1/2,x)",
                "(-x+1/2,x,-x+1/2)",
                "(x,-x+1/2,-x+1/2)",
                "(x,x,-x+1/2)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(x,-x+1/2,x)",
                "(-x+1/2,x,x)",
                "(-x,-x,-x)",
                "(x+1/2,x+1/2,-x)",
                "(x+1/2,-x,x+1/2)",
                "(-x,x+1/2,x+1/2)",
                "(-x,-x,x+1/2)",
                "(x+1/2,x+1/2,x+1/2)",
                "(-x,x+1/2,-x)",
                "(x+1/2,-x,-x)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,1/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x+1/2,1/4)",
                "(1/4,1/4,x)",
                "(1/4,1/4,-x+1/2)",
                "(-x,3/4,3/4)",
                "(x+1/2,3/4,3/4)",
                "(3/4,-x,3/4)",
                "(3/4,x+1/2,3/4)",
                "(3/4,3/4,-x)",
                "(3/4,3/4,x+1/2)"
            ],
            [
                "(0,3/4,1/4)",
                "(1/2,3/4,1/4)",
                "(1/4,0,3/4)",
                "(1/4,1/2,3/4)",
                "(3/4,1/4,0)",
                "(3/4,1/4,1/2)",
                "(3/4,0,1/4)",
                "(3/4,1/2,1/4)",
                "(0,1/4,3/4)",
                "(1/2,1/4,3/4)",
                "(1/4,3/4,1/2)",
                "(1/4,3/4,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(0,0,1/2)",
                "(1/2,1/2,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(3/4,1/4,1/4)",
                "(1/4,3/4,1/4)",
                "(1/4,1/4,3/4)",
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)",
                "(3/4,3/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "16",
            "12",
            "12",
            "8",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            ".3.",
            "4..",
            "-4..",
            ".-3.",
            "42.2",
            "432"
        ],
        "Space Group": "P 4/n -3 2/n",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "520": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,-y+1/2)",
                "(-x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,-z+1/2,-y+1/2)",
                "(x+1/2,-z+1/2,y+1/2)",
                "(z+1/2,y+1/2,-x+1/2)",
                "(z+1/2,-y+1/2,x+1/2)",
                "(-z+1/2,y+1/2,x+1/2)",
                "(-z+1/2,-y+1/2,-x+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-z,-x,-y)",
                "(-z,x,y)",
                "(z,x,-y)",
                "(z,-x,y)",
                "(-y,-z,-x)",
                "(y,-z,x)",
                "(-y,z,x)",
                "(y,z,-x)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,-z+1/2)",
                "(-x+1/2,-z+1/2,y+1/2)",
                "(x+1/2,-z+1/2,-y+1/2)",
                "(x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,z+1/2,-y+1/2)",
                "(-z+1/2,-y+1/2,x+1/2)",
                "(-z+1/2,y+1/2,-x+1/2)",
                "(z+1/2,-y+1/2,-x+1/2)",
                "(z+1/2,y+1/2,x+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(z,0,y)",
                "(z,0,-y)",
                "(-z,0,y)",
                "(-z,0,-y)",
                "(y,z,0)",
                "(-y,z,0)",
                "(y,-z,0)",
                "(-y,-z,0)",
                "(y+1/2,1/2,-z+1/2)",
                "(-y+1/2,1/2,-z+1/2)",
                "(y+1/2,1/2,z+1/2)",
                "(-y+1/2,1/2,z+1/2)",
                "(1/2,z+1/2,-y+1/2)",
                "(1/2,z+1/2,y+1/2)",
                "(1/2,-z+1/2,-y+1/2)",
                "(1/2,-z+1/2,y+1/2)",
                "(z+1/2,y+1/2,1/2)",
                "(z+1/2,-y+1/2,1/2)",
                "(-z+1/2,y+1/2,1/2)",
                "(-z+1/2,-y+1/2,1/2)"
            ],
            [
                "(1/4,y,y+1/2)",
                "(3/4,-y,y+1/2)",
                "(3/4,y,-y+1/2)",
                "(1/4,-y,-y+1/2)",
                "(y+1/2,1/4,y)",
                "(y+1/2,3/4,-y)",
                "(-y+1/2,3/4,y)",
                "(-y+1/2,1/4,-y)",
                "(y,y+1/2,1/4)",
                "(-y,y+1/2,3/4)",
                "(y,-y+1/2,3/4)",
                "(-y,-y+1/2,1/4)",
                "(3/4,-y,-y+1/2)",
                "(1/4,y,-y+1/2)",
                "(1/4,-y,y+1/2)",
                "(3/4,y,y+1/2)",
                "(-y+1/2,3/4,-y)",
                "(-y+1/2,1/4,y)",
                "(y+1/2,1/4,-y)",
                "(y+1/2,3/4,y)",
                "(-y,-y+1/2,3/4)",
                "(y,-y+1/2,1/4)",
                "(-y,y+1/2,1/4)",
                "(y,y+1/2,3/4)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x+1/2,x+1/2,-x+1/2)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x+1/2)",
                "(-x,-x,-x)",
                "(x,x,-x)",
                "(x,-x,x)",
                "(-x,x,x)",
                "(-x+1/2,-x+1/2,x+1/2)",
                "(x+1/2,x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x+1/2)"
            ],
            [
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(1/2,0,x)",
                "(1/2,0,-x)",
                "(0,x+1/2,1/2)",
                "(0,-x+1/2,1/2)",
                "(x+1/2,1/2,0)",
                "(-x+1/2,1/2,0)",
                "(1/2,0,-x+1/2)",
                "(1/2,0,x+1/2)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)",
                "(1/2,x+1/2,0)",
                "(1/2,-x+1/2,0)",
                "(x+1/2,0,1/2)",
                "(-x+1/2,0,1/2)",
                "(0,1/2,-x+1/2)",
                "(0,1/2,x+1/2)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(1/2,x+1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(1/2,1/2,-x+1/2)",
                "(1/2,1/2,x+1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)",
                "(3/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,1/2,0)",
                "(3/4,1/2,0)",
                "(0,1/4,1/2)",
                "(0,3/4,1/2)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)",
                "(1/2,1/4,0)",
                "(1/2,3/4,0)",
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "16",
            "12",
            "12",
            "12",
            "8",
            "6",
            "6",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            ".3.",
            "mm2..",
            "mm2..",
            "mm2..",
            "0.32",
            "-4m.2",
            "-4m.2",
            "mmm..",
            "m-3."
        ],
        "Space Group": "P 4sub2/m -3 2/n",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "521": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,-y+1/2)",
                "(-x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,-z+1/2,-y+1/2)",
                "(x+1/2,-z+1/2,y+1/2)",
                "(z+1/2,y+1/2,-x+1/2)",
                "(z+1/2,-y+1/2,x+1/2)",
                "(-z+1/2,y+1/2,x+1/2)",
                "(-z+1/2,-y+1/2,-x+1/2)",
                "(-x+1/2,-y+1/2,-z+1/2)",
                "(x+1/2,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z+1/2)",
                "(-z+1/2,-x+1/2,-y+1/2)",
                "(-z+1/2,x+1/2,y+1/2)",
                "(z+1/2,x+1/2,-y+1/2)",
                "(z+1/2,-x+1/2,y+1/2)",
                "(-y+1/2,-z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,x+1/2)",
                "(-y+1/2,z+1/2,x+1/2)",
                "(y+1/2,z+1/2,-x+1/2)",
                "(-y,-x,z)",
                "(y,x,z)",
                "(-y,x,-z)",
                "(y,-x,-z)",
                "(-x,-z,y)",
                "(x,-z,-y)",
                "(x,z,y)",
                "(-x,z,-y)",
                "(-z,-y,x)",
                "(-z,y,-x)",
                "(z,-y,-x)",
                "(z,y,x)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(z,x,x)",
                "(z,-x,-x)",
                "(-z,-x,x)",
                "(-z,x,-x)",
                "(x,z,x)",
                "(-x,z,-x)",
                "(x,-z,-x)",
                "(-x,-z,x)",
                "(x+1/2,x+1/2,-z+1/2)",
                "(-x+1/2,-x+1/2,-z+1/2)",
                "(x+1/2,-x+1/2,z+1/2)",
                "(-x+1/2,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,-x+1/2)",
                "(-x+1/2,z+1/2,x+1/2)",
                "(-x+1/2,-z+1/2,-x+1/2)",
                "(x+1/2,-z+1/2,x+1/2)",
                "(z+1/2,x+1/2,-x+1/2)",
                "(z+1/2,-x+1/2,x+1/2)",
                "(-z+1/2,x+1/2,x+1/2)",
                "(-z+1/2,-x+1/2,-x+1/2)"
            ],
            [
                "(1/4,y,y+1/2)",
                "(3/4,-y,y+1/2)",
                "(3/4,y,-y+1/2)",
                "(1/4,-y,-y+1/2)",
                "(y+1/2,1/4,y)",
                "(y+1/2,3/4,-y)",
                "(-y+1/2,3/4,y)",
                "(-y+1/2,1/4,-y)",
                "(y,y+1/2,1/4)",
                "(-y,y+1/2,3/4)",
                "(y,-y+1/2,3/4)",
                "(-y,-y+1/2,1/4)",
                "(1/4,-y+1/2,-y)",
                "(3/4,y+1/2,-y)",
                "(3/4,-y+1/2,y)",
                "(1/4,y+1/2,y)",
                "(-y,1/4,-y+1/2)",
                "(-y,3/4,y+1/2)",
                "(y,3/4,-y+1/2)",
                "(y,1/4,y+1/2)",
                "(-y+1/2,-y,1/4)",
                "(y+1/2,-y,3/4)",
                "(-y+1/2,y,3/4)",
                "(y+1/2,y,1/4)"
            ],
            [
                "(1/4,y,-y+1/2)",
                "(3/4,-y,-y+1/2)",
                "(3/4,y,y+1/2)",
                "(1/4,-y,y+1/2)",
                "(-y+1/2,1/4,y)",
                "(-y+1/2,3/4,-y)",
                "(y+1/2,3/4,y)",
                "(y+1/2,1/4,-y)",
                "(y,-y+1/2,1/4)",
                "(-y,-y+1/2,3/4)",
                "(y,y+1/2,3/4)",
                "(-y,y+1/2,1/4)",
                "(1/4,-y+1/2,y)",
                "(3/4,y+1/2,y)",
                "(3/4,-y+1/2,-y)",
                "(1/4,y+1/2,-y)",
                "(y,1/4,-y+1/2)",
                "(y,3/4,y+1/2)",
                "(-y,3/4,-y+1/2)",
                "(-y,1/4,y+1/2)",
                "(-y+1/2,y,1/4)",
                "(y+1/2,y,3/4)",
                "(-y+1/2,-y,3/4)",
                "(y+1/2,-y,1/4)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)",
                "(1/2,x+1/2,0)",
                "(1/2,-x+1/2,0)",
                "(x+1/2,0,1/2)",
                "(-x+1/2,0,1/2)",
                "(0,1/2,-x+1/2)",
                "(0,1/2,x+1/2)",
                "(-x+1/2,1/2,0)",
                "(x+1/2,1/2,0)",
                "(0,-x+1/2,1/2)",
                "(0,x+1/2,1/2)",
                "(1/2,0,-x+1/2)",
                "(1/2,0,x+1/2)",
                "(0,-x,1/2)",
                "(0,x,1/2)",
                "(-x,1/2,0)",
                "(x,1/2,0)",
                "(1/2,0,x)",
                "(1/2,0,-x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(1/2,x+1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(1/2,1/2,-x+1/2)",
                "(1/2,1/2,x+1/2)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)",
                "(1/2,1/4,0)",
                "(1/2,3/4,0)",
                "(0,1/2,1/4)",
                "(0,1/2,3/4)",
                "(1/4,1/2,0)",
                "(3/4,1/2,0)",
                "(0,1/4,1/2)",
                "(0,3/4,1/2)",
                "(1/2,0,1/4)",
                "(1/2,0,3/4)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x+1/2,x+1/2,-x+1/2)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x+1/2)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)",
                "(0,1/2,0)",
                "(1/2,0,0)",
                "(0,0,1/2)"
            ],
            [
                "(3/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "24",
            "24",
            "12",
            "12",
            "8",
            "6",
            "4",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "2..",
            "2.mm",
            "2.22",
            ".3m",
            "-42.m",
            ".-3m",
            ".-3m",
            "-43m"
        ],
        "Space Group": "P 4sub2/n -3 2/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "522": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y+1/2,z)",
                "(-x+1/2,y,-z+1/2)",
                "(x,-y+1/2,-z+1/2)",
                "(z,x,y)",
                "(z,-x+1/2,-y+1/2)",
                "(-z+1/2,-x+1/2,y)",
                "(-z+1/2,x,-y+1/2)",
                "(y,z,x)",
                "(-y+1/2,z,-x+1/2)",
                "(y,-z+1/2,-x+1/2)",
                "(-y+1/2,-z+1/2,x)",
                "(y+1/2,x+1/2,-z)",
                "(-y,-x,-z)",
                "(y+1/2,-x,z+1/2)",
                "(-y,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,-y)",
                "(-x,z+1/2,y+1/2)",
                "(-x,-z,-y)",
                "(x+1/2,-z,y+1/2)",
                "(z+1/2,y+1/2,-x)",
                "(z+1/2,-y,x+1/2)",
                "(-z,y+1/2,x+1/2)",
                "(-z,-y,-x)",
                "(-x,-y,-z)",
                "(x+1/2,y+1/2,-z)",
                "(x+1/2,-y,z+1/2)",
                "(-x,y+1/2,z+1/2)",
                "(-z,-x,-y)",
                "(-z,x+1/2,y+1/2)",
                "(z+1/2,x+1/2,-y)",
                "(z+1/2,-x,y+1/2)",
                "(-y,-z,-x)",
                "(y+1/2,-z,x+1/2)",
                "(-y,z+1/2,x+1/2)",
                "(y+1/2,z+1/2,-x)",
                "(-y+1/2,-x+1/2,z)",
                "(y,x,z)",
                "(-y+1/2,x,-z+1/2)",
                "(y,-x+1/2,-z+1/2)",
                "(-x+1/2,-z+1/2,y)",
                "(x,-z+1/2,-y+1/2)",
                "(x,z,y)",
                "(-x+1/2,z,-y+1/2)",
                "(-z+1/2,-y+1/2,x)",
                "(-z+1/2,y,-x+1/2)",
                "(z,-y+1/2,-x+1/2)",
                "(z,y,x)"
            ],
            [
                "(x,x,z)",
                "(-x+1/2,-x+1/2,z)",
                "(-x+1/2,x,-z+1/2)",
                "(x,-x+1/2,-z+1/2)",
                "(z,x,x)",
                "(z,-x+1/2,-x+1/2)",
                "(-z+1/2,-x+1/2,x)",
                "(-z+1/2,x,-x+1/2)",
                "(x,z,x)",
                "(-x+1/2,z,-x+1/2)",
                "(x,-z+1/2,-x+1/2)",
                "(-x+1/2,-z+1/2,x)",
                "(x+1/2,x+1/2,-z)",
                "(-x,-x,-z)",
                "(x+1/2,-x,z+1/2)",
                "(-x,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,-x)",
                "(-x,z+1/2,x+1/2)",
                "(-x,-z,-x)",
                "(x+1/2,-z,x+1/2)",
                "(z+1/2,x+1/2,-x)",
                "(z+1/2,-x,x+1/2)",
                "(-z,x+1/2,x+1/2)",
                "(-z,-x,-x)"
            ],
            [
                "(1/2,y,-y)",
                "(0,-y+1/2,-y)",
                "(0,y,y+1/2)",
                "(1/2,-y+1/2,y+1/2)",
                "(-y,1/2,y)",
                "(-y,0,-y+1/2)",
                "(y+1/2,0,y)",
                "(y+1/2,1/2,-y+1/2)",
                "(y,-y,1/2)",
                "(-y+1/2,-y,0)",
                "(y,y+1/2,0)",
                "(-y+1/2,y+1/2,1/2)",
                "(1/2,-y,y)",
                "(0,y+1/2,y)",
                "(0,-y,-y+1/2)",
                "(1/2,y+1/2,-y+1/2)",
                "(y,1/2,-y)",
                "(y,0,y+1/2)",
                "(-y+1/2,0,-y)",
                "(-y+1/2,1/2,y+1/2)",
                "(-y,y,1/2)",
                "(y+1/2,y,0)",
                "(-y,-y+1/2,0)",
                "(y+1/2,-y+1/2,1/2)"
            ],
            [
                "(1/2,y,y+1/2)",
                "(0,-y+1/2,y+1/2)",
                "(0,y,-y)",
                "(1/2,-y+1/2,-y)",
                "(y+1/2,1/2,y)",
                "(y+1/2,0,-y+1/2)",
                "(-y,0,y)",
                "(-y,1/2,-y+1/2)",
                "(y,y+1/2,1/2)",
                "(-y+1/2,y+1/2,0)",
                "(y,-y,0)",
                "(-y+1/2,-y,1/2)",
                "(1/2,-y,-y+1/2)",
                "(0,y+1/2,-y+1/2)",
                "(0,-y,y)",
                "(1/2,y+1/2,y)",
                "(-y+1/2,1/2,-y)",
                "(-y+1/2,0,y+1/2)",
                "(y,0,-y)",
                "(y,1/2,y+1/2)",
                "(-y,-y+1/2,1/2)",
                "(y+1/2,-y+1/2,0)",
                "(-y,y,0)",
                "(y+1/2,y,1/2)"
            ],
            [
                "(x,1/4,3/4)",
                "(-x+1/2,1/4,3/4)",
                "(3/4,x,1/4)",
                "(3/4,-x+1/2,1/4)",
                "(1/4,3/4,x)",
                "(1/4,3/4,-x+1/2)",
                "(3/4,x+1/2,1/4)",
                "(3/4,-x,1/4)",
                "(x+1/2,1/4,3/4)",
                "(-x,1/4,3/4)",
                "(1/4,3/4,-x)",
                "(1/4,3/4,x+1/2)",
                "(-x,3/4,1/4)",
                "(x+1/2,3/4,1/4)",
                "(1/4,-x,3/4)",
                "(1/4,x+1/2,3/4)",
                "(3/4,1/4,-x)",
                "(3/4,1/4,x+1/2)",
                "(1/4,-x+1/2,3/4)",
                "(1/4,x,3/4)",
                "(-x+1/2,3/4,1/4)",
                "(x,3/4,1/4)",
                "(3/4,1/4,x)",
                "(3/4,1/4,-x+1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x+1/2,1/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x+1/2,1/4)",
                "(1/4,1/4,x)",
                "(1/4,1/4,-x+1/2)",
                "(3/4,x+1/2,3/4)",
                "(3/4,-x,3/4)",
                "(x+1/2,3/4,3/4)",
                "(-x,3/4,3/4)",
                "(3/4,3/4,-x)",
                "(3/4,3/4,x+1/2)"
            ],
            [
                "(1/2,1/4,3/4)",
                "(0,1/4,3/4)",
                "(3/4,1/2,1/4)",
                "(3/4,0,1/4)",
                "(1/4,3/4,1/2)",
                "(1/4,3/4,0)",
                "(1/2,3/4,1/4)",
                "(0,3/4,1/4)",
                "(1/4,1/2,3/4)",
                "(1/4,0,3/4)",
                "(3/4,1/4,1/2)",
                "(3/4,1/4,0)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x+1/2,x)",
                "(-x+1/2,x,-x+1/2)",
                "(x,-x+1/2,-x+1/2)",
                "(x+1/2,x+1/2,-x)",
                "(-x,-x,-x)",
                "(x+1/2,-x,x+1/2)",
                "(-x,x+1/2,x+1/2)"
            ],
            [
                "(1/4,3/4,3/4)",
                "(3/4,1/4,3/4)",
                "(3/4,3/4,1/4)",
                "(1/4,3/4,1/4)",
                "(3/4,1/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(0,0,1/2)",
                "(0,1/2,0)",
                "(1/2,0,0)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "48",
            "24",
            "24",
            "24",
            "24",
            "12",
            "12",
            "8",
            "6",
            "4",
            "4",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "..2",
            "..2",
            "2..",
            "2.mm",
            "2.22",
            ".3m",
            "-42.m",
            ".-3m",
            ".-3m",
            "-43m"
        ],
        "Space Group": "P 4sub2/n -3 2/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "523": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(y,-x,z)",
                "(-y,x,z)",
                "(x,z,-y)",
                "(-x,z,y)",
                "(-x,-z,-y)",
                "(x,-z,y)",
                "(z,y,-x)",
                "(z,-y,x)",
                "(-z,y,x)",
                "(-z,-y,-x)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-z,-x,-y)",
                "(-z,x,y)",
                "(z,x,-y)",
                "(z,-x,y)",
                "(-y,-z,-x)",
                "(y,-z,x)",
                "(-y,z,x)",
                "(y,z,-x)",
                "(-y,-x,z)",
                "(y,x,z)",
                "(-y,x,-z)",
                "(y,-x,-z)",
                "(-x,-z,y)",
                "(x,-z,-y)",
                "(x,z,y)",
                "(-x,z,-y)",
                "(-z,-y,x)",
                "(-z,y,-x)",
                "(z,-y,-x)",
                "(z,y,x)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(z,x,x)",
                "(z,-x,-x)",
                "(-z,-x,x)",
                "(-z,x,-x)",
                "(x,z,x)",
                "(-x,z,-x)",
                "(x,-z,-x)",
                "(-x,-z,x)",
                "(x,x,-z)",
                "(-x,-x,-z)",
                "(x,-x,z)",
                "(-x,x,z)",
                "(x,z,-x)",
                "(-x,z,x)",
                "(-x,-z,-x)",
                "(x,-z,x)",
                "(z,x,-x)",
                "(z,-x,x)",
                "(-z,x,x)",
                "(-z,-x,-x)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(z,0,y)",
                "(z,0,-y)",
                "(-z,0,y)",
                "(-z,0,-y)",
                "(y,z,0)",
                "(-y,z,0)",
                "(y,-z,0)",
                "(-y,-z,0)",
                "(y,0,-z)",
                "(-y,0,-z)",
                "(y,0,z)",
                "(-y,0,z)",
                "(0,z,-y)",
                "(0,z,y)",
                "(0,-z,-y)",
                "(0,-z,y)",
                "(z,y,0)",
                "(z,-y,0)",
                "(-z,y,0)",
                "(-z,-y,0)"
            ],
            [
                "(1/2,y,y)",
                "(1/2,-y,y)",
                "(1/2,y,-y)",
                "(1/2,-y,-y)",
                "(y,1/2,y)",
                "(y,1/2,-y)",
                "(-y,1/2,y)",
                "(-y,1/2,-y)",
                "(y,y,1/2)",
                "(-y,y,1/2)",
                "(y,-y,1/2)",
                "(-y,-y,1/2)"
            ],
            [
                "(0,y,y)",
                "(0,-y,y)",
                "(0,y,-y)",
                "(0,-y,-y)",
                "(y,0,y)",
                "(y,0,-y)",
                "(-y,0,y)",
                "(-y,0,-y)",
                "(y,y,0)",
                "(-y,y,0)",
                "(y,-y,0)",
                "(-y,-y,0)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x,3/4)",
                "(1/4,1/4,x)",
                "(3/4,1/4,-x)",
                "(1/4,x,3/4)",
                "(3/4,-x,3/4)",
                "(x,1/4,3/4)",
                "(-x,1/4,1/4)",
                "(1/4,1/4,-x)",
                "(1/4,3/4,x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x,x,-x)",
                "(-x,-x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)",
                "(1/4,0,1/4)",
                "(1/4,0,3/4)",
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(1/4,1/4,3/4)"
            ],
            [
                "(1/2,1/2,1/2)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "192",
            "96",
            "96",
            "48",
            "48",
            "48",
            "32",
            "24",
            "24",
            "8",
            "4",
            "4"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "m.m2",
            "m.m2",
            "2.mm",
            ".3m",
            "4m.m",
            "m.mm",
            "-43m",
            "m-3m",
            "m-3m"
        ],
        "Space Group": "F 4/m -3 2/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "524": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y+1/2,x+1/2,-z+1/2)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,z+1/2)",
                "(-y+1/2,x+1/2,z+1/2)",
                "(x+1/2,z+1/2,-y+1/2)",
                "(-x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,-z+1/2,-y+1/2)",
                "(x+1/2,-z+1/2,y+1/2)",
                "(z+1/2,y+1/2,-x+1/2)",
                "(z+1/2,-y+1/2,x+1/2)",
                "(-z+1/2,y+1/2,x+1/2)",
                "(-z+1/2,-y+1/2,-x+1/2)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-z,-x,-y)",
                "(-z,x,y)",
                "(z,x,-y)",
                "(z,-x,y)",
                "(-y,-z,-x)",
                "(y,-z,x)",
                "(-y,z,x)",
                "(y,z,-x)",
                "(-y+1/2,-x+1/2,z+1/2)",
                "(y+1/2,x+1/2,z+1/2)",
                "(-y+1/2,x+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,-z+1/2)",
                "(-x+1/2,-z+1/2,y+1/2)",
                "(x+1/2,-z+1/2,-y+1/2)",
                "(x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,z+1/2,-y+1/2)",
                "(-z+1/2,-y+1/2,x+1/2)",
                "(-z+1/2,y+1/2,-x+1/2)",
                "(z+1/2,-y+1/2,-x+1/2)",
                "(z+1/2,y+1/2,x+1/2)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(z,0,y)",
                "(z,0,-y)",
                "(-z,0,y)",
                "(-z,0,-y)",
                "(y,z,0)",
                "(-y,z,0)",
                "(y,-z,0)",
                "(-y,-z,0)",
                "(y+1/2,1/2,-z+1/2)",
                "(-y+1/2,1/2,-z+1/2)",
                "(y+1/2,1/2,z+1/2)",
                "(-y+1/2,1/2,z+1/2)",
                "(1/2,z+1/2,-y+1/2)",
                "(1/2,z+1/2,y+1/2)",
                "(1/2,-z+1/2,-y+1/2)",
                "(1/2,-z+1/2,y+1/2)",
                "(z+1/2,y+1/2,1/2)",
                "(z+1/2,-y+1/2,1/2)",
                "(-z+1/2,y+1/2,1/2)",
                "(-z+1/2,-y+1/2,1/2)"
            ],
            [
                "(1/4,y,y)",
                "(3/4,-y,y)",
                "(3/4,y,-y)",
                "(1/4,-y,-y)",
                "(y,1/4,y)",
                "(y,3/4,-y)",
                "(-y,3/4,y)",
                "(-y,1/4,-y)",
                "(y,y,1/4)",
                "(-y,y,3/4)",
                "(y,-y,3/4)",
                "(-y,-y,1/4)",
                "(3/4,-y,-y)",
                "(1/4,y,-y)",
                "(1/4,-y,y)",
                "(3/4,y,y)",
                "(-y,3/4,-y)",
                "(-y,1/4,y)",
                "(y,1/4,-y)",
                "(y,3/4,y)",
                "(-y,-y,3/4)",
                "(y,-y,1/4)",
                "(-y,y,1/4)",
                "(y,y,3/4)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x+1/2,x+1/2,-x+1/2)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x+1/2)",
                "(-x,-x,-x)",
                "(x,x,-x)",
                "(x,-x,x)",
                "(-x,x,x)",
                "(-x+1/2,-x+1/2,x+1/2)",
                "(x+1/2,x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x+1/2)"
            ],
            [
                "(x,1/4,1/4)",
                "(-x,3/4,1/4)",
                "(1/4,x,1/4)",
                "(1/4,-x,3/4)",
                "(1/4,1/4,x)",
                "(3/4,1/4,-x)",
                "(-x,3/4,3/4)",
                "(x,1/4,3/4)",
                "(3/4,-x,3/4)",
                "(3/4,x,1/4)",
                "(3/4,3/4,-x)",
                "(1/4,3/4,x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)",
                "(1/2,x+1/2,1/2)",
                "(1/2,-x+1/2,1/2)",
                "(x+1/2,1/2,1/2)",
                "(-x+1/2,1/2,1/2)",
                "(1/2,1/2,-x+1/2)",
                "(1/2,1/2,x+1/2)"
            ],
            [
                "(0,1/4,1/4)",
                "(0,3/4,1/4)",
                "(1/4,0,1/4)",
                "(1/4,0,3/4)",
                "(1/4,1/4,0)",
                "(3/4,1/4,0)"
            ],
            [
                "(1/4,0,0)",
                "(3/4,0,0)",
                "(0,1/4,0)",
                "(0,3/4,0)",
                "(0,0,1/4)",
                "(0,0,3/4)"
            ],
            [
                "(0,0,0)",
                "(1/2,1/2,1/2)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "192",
            "96",
            "96",
            "64",
            "48",
            "48",
            "24",
            "24",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "m..",
            "..2",
            ".3.",
            "4..",
            "mm2..",
            "4/m..",
            "-4m.2",
            "m-3.",
            "432"
        ],
        "Space Group": "F 4/m -3 2/c",
        "Wyckoff Letter": [
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "525": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y,-z+1/2)",
                "(z,x,y)",
                "(z+1/2,-x,-y+1/2)",
                "(-z,-x+1/2,y+1/2)",
                "(-z+1/2,x+1/2,-y)",
                "(y,z,x)",
                "(-y+1/2,z+1/2,-x)",
                "(y+1/2,-z,-x+1/2)",
                "(-y,-z+1/2,x+1/2)",
                "(y+3/4,x+1/4,-z+3/4)",
                "(-y+1/4,-x+1/4,-z+1/4)",
                "(y+1/4,-x+3/4,z+3/4)",
                "(-y+3/4,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-y+3/4)",
                "(-x+3/4,z+3/4,y+1/4)",
                "(-x+1/4,-z+1/4,-y+1/4)",
                "(x+1/4,-z+3/4,y+3/4)",
                "(z+3/4,y+1/4,-x+3/4)",
                "(z+1/4,-y+3/4,x+3/4)",
                "(-z+3/4,y+3/4,x+1/4)",
                "(-z+1/4,-y+1/4,-x+1/4)",
                "(-x+1/4,-y+1/4,-z+1/4)",
                "(x+1/4,y+3/4,-z+3/4)",
                "(x+3/4,-y+3/4,z+1/4)",
                "(-x+3/4,y+1/4,z+3/4)",
                "(-z+1/4,-x+1/4,-y+1/4)",
                "(-z+3/4,x+1/4,y+3/4)",
                "(z+1/4,x+3/4,-y+3/4)",
                "(z+3/4,-x+3/4,y+1/4)",
                "(-y+1/4,-z+1/4,-x+1/4)",
                "(y+3/4,-z+3/4,x+1/4)",
                "(-y+3/4,z+1/4,x+3/4)",
                "(y+1/4,z+3/4,-x+3/4)",
                "(-y+1/2,-x,z+1/2)",
                "(y,x,z)",
                "(-y,x+1/2,-z+1/2)",
                "(y+1/2,-x+1/2,-z)",
                "(-x+1/2,-z,y+1/2)",
                "(x+1/2,-z+1/2,-y)",
                "(x,z,y)",
                "(-x,z+1/2,-y+1/2)",
                "(-z+1/2,-y,x+1/2)",
                "(-z,y+1/2,-x+1/2)",
                "(z+1/2,-y+1/2,-x)",
                "(z,y,x)"
            ],
            [
                "(1/8,y,-y+1/4)",
                "(7/8,-y+1/2,-y+3/4)",
                "(3/8,y+1/2,y+3/4)",
                "(5/8,-y,y+1/4)",
                "(-y+1/4,1/8,y)",
                "(-y+3/4,7/8,-y+1/2)",
                "(y+3/4,3/8,y+1/2)",
                "(y+1/4,5/8,-y)",
                "(y,-y+1/4,1/8)",
                "(-y+1/2,-y+3/4,7/8)",
                "(y+1/2,y+3/4,3/8)",
                "(-y,y+1/4,5/8)",
                "(1/8,-y+1/4,y)",
                "(3/8,y+3/4,y+1/2)",
                "(7/8,-y+3/4,-y+1/2)",
                "(5/8,y+1/4,-y)",
                "(y,1/8,-y+1/4)",
                "(y+1/2,3/8,y+3/4)",
                "(-y+1/2,7/8,-y+3/4)",
                "(-y,5/8,y+1/4)",
                "(-y+1/4,y,1/8)",
                "(y+3/4,y+1/2,3/8)",
                "(-y+3/4,-y+1/2,7/8)",
                "(y+1/4,-y,5/8)"
            ],
            [
                "(x,x,z)",
                "(-x,-x+1/2,z+1/2)",
                "(-x+1/2,x+1/2,-z)",
                "(x+1/2,-x,-z+1/2)",
                "(z,x,x)",
                "(z+1/2,-x,-x+1/2)",
                "(-z,-x+1/2,x+1/2)",
                "(-z+1/2,x+1/2,-x)",
                "(x,z,x)",
                "(-x+1/2,z+1/2,-x)",
                "(x+1/2,-z,-x+1/2)",
                "(-x,-z+1/2,x+1/2)",
                "(x+3/4,x+1/4,-z+3/4)",
                "(-x+1/4,-x+1/4,-z+1/4)",
                "(x+1/4,-x+3/4,z+3/4)",
                "(-x+3/4,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-x+3/4)",
                "(-x+3/4,z+3/4,x+1/4)",
                "(-x+1/4,-z+1/4,-x+1/4)",
                "(x+1/4,-z+3/4,x+3/4)",
                "(z+3/4,x+1/4,-x+3/4)",
                "(z+1/4,-x+3/4,x+3/4)",
                "(-z+3/4,x+3/4,x+1/4)",
                "(-z+1/4,-x+1/4,-x+1/4)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)",
                "(0,x,0)",
                "(1/2,-x,1/2)",
                "(0,0,x)",
                "(1/2,1/2,-x)",
                "(3/4,x+1/4,3/4)",
                "(1/4,-x+1/4,1/4)",
                "(x+3/4,1/4,3/4)",
                "(-x+3/4,3/4,1/4)",
                "(3/4,1/4,-x+3/4)",
                "(1/4,3/4,x+3/4)"
            ],
            [
                "(x,x,x)",
                "(-x,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,-x)",
                "(x+1/2,-x,-x+1/2)",
                "(x+3/4,x+1/4,-x+3/4)",
                "(-x+1/4,-x+1/4,-x+1/4)",
                "(x+1/4,-x+3/4,x+3/4)",
                "(-x+3/4,x+3/4,x+1/4)"
            ],
            [
                "(5/8,5/8,5/8)",
                "(3/8,7/8,1/8)",
                "(7/8,1/8,3/8)",
                "(1/8,3/8,7/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,3/8,5/8)",
                "(3/8,5/8,7/8)",
                "(5/8,7/8,3/8)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(1/4,3/4,1/4)"
            ],
            [
                "(0,0,0)",
                "(3/4,1/4,3/4)"
            ]
        ],
        "Multiplicity": [
            "192",
            "96",
            "96",
            "48",
            "32",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..m",
            "2.mm",
            ".3m",
            ".-3m",
            ".-3m",
            "-43m",
            "-43m"
        ],
        "Space Group": "F 4sub1/d -3 2/m",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "526": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+3/4,-y+1/4,z+1/2)",
                "(-x+1/4,y+1/2,-z+3/4)",
                "(x+1/2,-y+3/4,-z+1/4)",
                "(z,x,y)",
                "(z+1/2,-x+3/4,-y+1/4)",
                "(-z+3/4,-x+1/4,y+1/2)",
                "(-z+1/4,x+1/2,-y+3/4)",
                "(y,z,x)",
                "(-y+1/4,z+1/2,-x+3/4)",
                "(y+1/2,-z+3/4,-x+1/4)",
                "(-y+3/4,-z+1/4,x+1/2)",
                "(y+3/4,x+1/4,-z+1/2)",
                "(-y,-x,-z)",
                "(y+1/4,-x+1/2,z+3/4)",
                "(-y+1/2,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-y+1/2)",
                "(-x+1/2,z+3/4,y+1/4)",
                "(-x,-z,-y)",
                "(x+1/4,-z+1/2,y+3/4)",
                "(z+3/4,y+1/4,-x+1/2)",
                "(z+1/4,-y+1/2,x+3/4)",
                "(-z+1/2,y+3/4,x+1/4)",
                "(-z,-y,-x)",
                "(-x,-y,-z)",
                "(x+1/4,y+3/4,-z+1/2)",
                "(x+3/4,-y+1/2,z+1/4)",
                "(-x+1/2,y+1/4,z+3/4)",
                "(-z,-x,-y)",
                "(-z+1/2,x+1/4,y+3/4)",
                "(z+1/4,x+3/4,-y+1/2)",
                "(z+3/4,-x+1/2,y+1/4)",
                "(-y,-z,-x)",
                "(y+3/4,-z+1/2,x+1/4)",
                "(-y+1/2,z+1/4,x+3/4)",
                "(y+1/4,z+3/4,-x+1/2)",
                "(-y+1/4,-x+3/4,z+1/2)",
                "(y,x,z)",
                "(-y+3/4,x+1/2,-z+1/4)",
                "(y+1/2,-x+1/4,-z+3/4)",
                "(-x+1/4,-z+3/4,y+1/2)",
                "(x+1/2,-z+1/4,-y+3/4)",
                "(x,z,y)",
                "(-x+3/4,z+1/2,-y+1/4)",
                "(-z+1/4,-y+3/4,x+1/2)",
                "(-z+3/4,y+1/2,-x+1/4)",
                "(z+1/2,-y+1/4,-x+3/4)",
                "(z,y,x)"
            ],
            [
                "(0,y,-y)",
                "(3/4,-y+1/4,-y+1/2)",
                "(1/4,y+1/2,y+3/4)",
                "(1/2,-y+3/4,y+1/4)",
                "(-y,0,y)",
                "(-y+1/2,3/4,-y+1/4)",
                "(y+3/4,1/4,y+1/2)",
                "(y+1/4,1/2,-y+3/4)",
                "(y,-y,0)",
                "(-y+1/4,-y+1/2,3/4)",
                "(y+1/2,y+3/4,1/4)",
                "(-y+3/4,y+1/4,1/2)",
                "(0,-y,y)",
                "(1/4,y+3/4,y+1/2)",
                "(3/4,-y+1/2,-y+1/4)",
                "(1/2,y+1/4,-y+3/4)",
                "(y,0,-y)",
                "(y+1/2,1/4,y+3/4)",
                "(-y+1/4,3/4,-y+1/2)",
                "(-y+3/4,1/2,y+1/4)",
                "(-y,y,0)",
                "(y+3/4,y+1/2,1/4)",
                "(-y+1/2,-y+1/4,3/4)",
                "(y+1/4,-y+3/4,1/2)"
            ],
            [
                "(x,x,z)",
                "(-x+3/4,-x+1/4,z+1/2)",
                "(-x+1/4,x+1/2,-z+3/4)",
                "(x+1/2,-x+3/4,-z+1/4)",
                "(z,x,x)",
                "(z+1/2,-x+3/4,-x+1/4)",
                "(-z+3/4,-x+1/4,x+1/2)",
                "(-z+1/4,x+1/2,-x+3/4)",
                "(x,z,x)",
                "(-x+1/4,z+1/2,-x+3/4)",
                "(x+1/2,-z+3/4,-x+1/4)",
                "(-x+3/4,-z+1/4,x+1/2)",
                "(x+3/4,x+1/4,-z+1/2)",
                "(-x,-x,-z)",
                "(x+1/4,-x+1/2,z+3/4)",
                "(-x+1/2,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-x+1/2)",
                "(-x+1/2,z+3/4,x+1/4)",
                "(-x,-z,-x)",
                "(x+1/4,-z+1/2,x+3/4)",
                "(z+3/4,x+1/4,-x+1/2)",
                "(z+1/4,-x+1/2,x+3/4)",
                "(-z+1/2,x+3/4,x+1/4)",
                "(-z,-x,-x)"
            ],
            [
                "(x,1/8,1/8)",
                "(-x+3/4,1/8,5/8)",
                "(1/8,x,1/8)",
                "(5/8,-x+3/4,1/8)",
                "(1/8,1/8,x)",
                "(1/8,5/8,-x+3/4)",
                "(7/8,x+1/4,3/8)",
                "(7/8,-x,7/8)",
                "(x+3/4,3/8,3/8)",
                "(-x+1/2,7/8,3/8)",
                "(7/8,3/8,-x+1/2)",
                "(3/8,3/8,x+3/4)"
            ],
            [
                "(x,x,x)",
                "(-x+3/4,-x+1/4,x+1/2)",
                "(-x+1/4,x+1/2,-x+3/4)",
                "(x+1/2,-x+3/4,-x+1/4)",
                "(x+3/4,x+1/4,-x+1/2)",
                "(-x,-x,-x)",
                "(x+1/4,-x+1/2,x+3/4)",
                "(-x+1/2,x+3/4,x+1/4)"
            ],
            [
                "(1/2,1/2,1/2)",
                "(1/4,3/4,0)",
                "(3/4,0,1/4)",
                "(0,1/4,3/4)"
            ],
            [
                "(0,0,0)",
                "(3/4,1/4,1/2)",
                "(1/4,1/2,3/4)",
                "(1/2,3/4,1/4)"
            ],
            [
                "(3/8,3/8,3/8)",
                "(1/8,5/8,1/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,3/8,3/8)"
            ]
        ],
        "Multiplicity": [
            "192",
            "96",
            "96",
            "48",
            "32",
            "16",
            "16",
            "8",
            "8"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "..m",
            "2.mm",
            ".3m",
            ".-3m",
            ".-3m",
            "-43m",
            "-43m"
        ],
        "Space Group": "F 4sub1/d -3 2/m",
        "Wyckoff Letter": [
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "527": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,-z)",
                "(x+1/2,-y,-z+1/2)",
                "(z,x,y)",
                "(z+1/2,-x,-y+1/2)",
                "(-z,-x+1/2,y+1/2)",
                "(-z+1/2,x+1/2,-y)",
                "(y,z,x)",
                "(-y+1/2,z+1/2,-x)",
                "(y+1/2,-z,-x+1/2)",
                "(-y,-z+1/2,x+1/2)",
                "(y+3/4,x+1/4,-z+3/4)",
                "(-y+1/4,-x+1/4,-z+1/4)",
                "(y+1/4,-x+3/4,z+3/4)",
                "(-y+3/4,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-y+3/4)",
                "(-x+3/4,z+3/4,y+1/4)",
                "(-x+1/4,-z+1/4,-y+1/4)",
                "(x+1/4,-z+3/4,y+3/4)",
                "(z+3/4,y+1/4,-x+3/4)",
                "(z+1/4,-y+3/4,x+3/4)",
                "(-z+3/4,y+3/4,x+1/4)",
                "(-z+1/4,-y+1/4,-x+1/4)",
                "(-x+3/4,-y+3/4,-z+3/4)",
                "(x+3/4,y+1/4,-z+1/4)",
                "(x+1/4,-y+1/4,z+3/4)",
                "(-x+1/4,y+3/4,z+1/4)",
                "(-z+3/4,-x+3/4,-y+3/4)",
                "(-z+1/4,x+3/4,y+1/4)",
                "(z+3/4,x+1/4,-y+1/4)",
                "(z+1/4,-x+1/4,y+3/4)",
                "(-y+3/4,-z+3/4,-x+3/4)",
                "(y+1/4,-z+1/4,x+3/4)",
                "(-y+1/4,z+3/4,x+1/4)",
                "(y+3/4,z+1/4,-x+1/4)",
                "(-y,-x+1/2,z)",
                "(y+1/2,x+1/2,z+1/2)",
                "(-y+1/2,x,-z)",
                "(y,-x,-z+1/2)",
                "(-x,-z+1/2,y)",
                "(x,-z,-y+1/2)",
                "(x+1/2,z+1/2,y+1/2)",
                "(-x+1/2,z,-y)",
                "(-z,-y+1/2,x)",
                "(-z+1/2,y,-x)",
                "(z,-y,-x+1/2)",
                "(z+1/2,y+1/2,x+1/2)"
            ],
            [
                "(1/8,y,-y+1/4)",
                "(7/8,-y+1/2,-y+3/4)",
                "(3/8,y+1/2,y+3/4)",
                "(5/8,-y,y+1/4)",
                "(-y+1/4,1/8,y)",
                "(-y+3/4,7/8,-y+1/2)",
                "(y+3/4,3/8,y+1/2)",
                "(y+1/4,5/8,-y)",
                "(y,-y+1/4,1/8)",
                "(-y+1/2,-y+3/4,7/8)",
                "(y+1/2,y+3/4,3/8)",
                "(-y,y+1/4,5/8)",
                "(5/8,-y+3/4,y+1/2)",
                "(7/8,y+1/4,y)",
                "(3/8,-y+1/4,-y)",
                "(1/8,y+3/4,-y+1/2)",
                "(y+1/2,5/8,-y+3/4)",
                "(y,7/8,y+1/4)",
                "(-y,3/8,-y+1/4)",
                "(-y+1/2,1/8,y+3/4)",
                "(-y+3/4,y+1/2,5/8)",
                "(y+1/4,y,7/8)",
                "(-y+1/4,-y,3/8)",
                "(y+3/4,-y+1/2,1/8)"
            ],
            [
                "(x,0,0)",
                "(-x,1/2,1/2)",
                "(0,x,0)",
                "(1/2,-x,1/2)",
                "(0,0,x)",
                "(1/2,1/2,-x)",
                "(3/4,x+1/4,3/4)",
                "(1/4,-x+1/4,1/4)",
                "(x+3/4,1/4,3/4)",
                "(-x+3/4,3/4,1/4)",
                "(3/4,1/4,-x+3/4)",
                "(1/4,3/4,x+3/4)",
                "(-x+3/4,3/4,3/4)",
                "(x+3/4,1/4,1/4)",
                "(3/4,-x+3/4,3/4)",
                "(1/4,x+3/4,1/4)",
                "(3/4,3/4,-x+3/4)",
                "(1/4,1/4,x+3/4)",
                "(0,-x+1/2,0)",
                "(1/2,x+1/2,1/2)",
                "(-x,1/2,0)",
                "(x,0,1/2)",
                "(0,1/2,x)",
                "(1/2,0,-x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,-x)",
                "(x+1/2,-x,-x+1/2)",
                "(x+3/4,x+1/4,-x+3/4)",
                "(-x+1/4,-x+1/4,-x+1/4)",
                "(x+1/4,-x+3/4,x+3/4)",
                "(-x+3/4,x+3/4,x+1/4)",
                "(-x+3/4,-x+3/4,-x+3/4)",
                "(x+3/4,x+1/4,-x+1/4)",
                "(x+1/4,-x+1/4,x+3/4)",
                "(-x+1/4,x+3/4,x+1/4)",
                "(-x,-x+1/2,x)",
                "(x+1/2,x+1/2,x+1/2)",
                "(-x+1/2,x,-x)",
                "(x,-x,-x+1/2)"
            ],
            [
                "(1/4,0,0)",
                "(3/4,1/2,1/2)",
                "(0,1/4,0)",
                "(1/2,3/4,1/2)",
                "(0,0,1/4)",
                "(1/2,1/2,3/4)",
                "(3/4,1/2,3/4)",
                "(1/4,0,1/4)",
                "(0,1/4,3/4)",
                "(1/2,3/4,1/4)",
                "(3/4,1/4,1/2)",
                "(1/4,3/4,0)"
            ],
            [
                "(3/8,3/8,3/8)",
                "(5/8,1/8,7/8)",
                "(1/8,7/8,5/8)",
                "(7/8,5/8,1/8)",
                "(1/8,5/8,3/8)",
                "(7/8,7/8,7/8)",
                "(5/8,3/8,1/8)",
                "(3/8,1/8,5/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,3/8,5/8)",
                "(3/8,5/8,7/8)",
                "(5/8,7/8,3/8)",
                "(5/8,5/8,5/8)",
                "(7/8,3/8,1/8)",
                "(3/8,1/8,7/8)",
                "(1/8,7/8,3/8)"
            ],
            [
                "(0,0,0)",
                "(3/4,1/4,3/4)",
                "(3/4,3/4,3/4)",
                "(0,1/2,0)"
            ]
        ],
        "Multiplicity": [
            "192",
            "96",
            "96",
            "64",
            "48",
            "32",
            "32",
            "16"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            ".3.",
            "-4..",
            ".-3.",
            "0.32",
            "23"
        ],
        "Space Group": "F 4sub1/d -3 2/c",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "528": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/4,-y+3/4,z+1/2)",
                "(-x+3/4,y+1/2,-z+1/4)",
                "(x+1/2,-y+1/4,-z+3/4)",
                "(z,x,y)",
                "(z+1/2,-x+1/4,-y+3/4)",
                "(-z+1/4,-x+3/4,y+1/2)",
                "(-z+3/4,x+1/2,-y+1/4)",
                "(y,z,x)",
                "(-y+3/4,z+1/2,-x+1/4)",
                "(y+1/2,-z+1/4,-x+3/4)",
                "(-y+1/4,-z+3/4,x+1/2)",
                "(y+3/4,x+1/4,-z)",
                "(-y+1/2,-x+1/2,-z+1/2)",
                "(y+1/4,-x,z+3/4)",
                "(-y,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-y)",
                "(-x,z+3/4,y+1/4)",
                "(-x+1/2,-z+1/2,-y+1/2)",
                "(x+1/4,-z,y+3/4)",
                "(z+3/4,y+1/4,-x)",
                "(z+1/4,-y,x+3/4)",
                "(-z,y+3/4,x+1/4)",
                "(-z+1/2,-y+1/2,-x+1/2)",
                "(-x,-y,-z)",
                "(x+3/4,y+1/4,-z+1/2)",
                "(x+1/4,-y+1/2,z+3/4)",
                "(-x+1/2,y+3/4,z+1/4)",
                "(-z,-x,-y)",
                "(-z+1/2,x+3/4,y+1/4)",
                "(z+3/4,x+1/4,-y+1/2)",
                "(z+1/4,-x+1/2,y+3/4)",
                "(-y,-z,-x)",
                "(y+1/4,-z+1/2,x+3/4)",
                "(-y+1/2,z+3/4,x+1/4)",
                "(y+3/4,z+1/4,-x+1/2)",
                "(-y+1/4,-x+3/4,z)",
                "(y+1/2,x+1/2,z+1/2)",
                "(-y+3/4,x,-z+1/4)",
                "(y,-x+1/4,-z+3/4)",
                "(-x+1/4,-z+3/4,y)",
                "(x,-z+1/4,-y+3/4)",
                "(x+1/2,z+1/2,y+1/2)",
                "(-x+3/4,z,-y+1/4)",
                "(-z+1/4,-y+3/4,x)",
                "(-z+3/4,y,-x+1/4)",
                "(z,-y+1/4,-x+3/4)",
                "(z+1/2,y+1/2,x+1/2)"
            ],
            [
                "(1/4,y,-y)",
                "(0,-y+3/4,-y+1/2)",
                "(1/2,y+1/2,y+1/4)",
                "(3/4,-y+1/4,y+3/4)",
                "(-y,1/4,y)",
                "(-y+1/2,0,-y+3/4)",
                "(y+1/4,1/2,y+1/2)",
                "(y+3/4,3/4,-y+1/4)",
                "(y,-y,1/4)",
                "(-y+3/4,-y+1/2,0)",
                "(y+1/2,y+1/4,1/2)",
                "(-y+1/4,y+3/4,3/4)",
                "(3/4,-y,y)",
                "(0,y+1/4,y+1/2)",
                "(1/2,-y+1/2,-y+3/4)",
                "(1/4,y+3/4,-y+1/4)",
                "(y,3/4,-y)",
                "(y+1/2,0,y+1/4)",
                "(-y+3/4,1/2,-y+1/2)",
                "(-y+1/4,1/4,y+3/4)",
                "(-y,y,3/4)",
                "(y+1/4,y+1/2,0)",
                "(-y+1/2,-y+3/4,1/2)",
                "(y+3/4,-y+1/4,1/4)"
            ],
            [
                "(x,1/8,1/8)",
                "(-x+1/4,5/8,5/8)",
                "(1/8,x,1/8)",
                "(5/8,-x+1/4,5/8)",
                "(1/8,1/8,x)",
                "(5/8,5/8,-x+1/4)",
                "(7/8,x+1/4,7/8)",
                "(3/8,-x+1/2,3/8)",
                "(x+3/4,3/8,7/8)",
                "(-x,7/8,3/8)",
                "(7/8,3/8,-x)",
                "(3/8,7/8,x+3/4)",
                "(-x,7/8,7/8)",
                "(x+3/4,3/8,3/8)",
                "(7/8,-x,7/8)",
                "(3/8,x+3/4,3/8)",
                "(7/8,7/8,-x)",
                "(3/8,3/8,x+3/4)",
                "(1/8,-x+3/4,1/8)",
                "(5/8,x+1/2,5/8)",
                "(-x+1/4,5/8,1/8)",
                "(x,1/8,5/8)",
                "(1/8,5/8,x)",
                "(5/8,1/8,-x+1/4)"
            ],
            [
                "(x,x,x)",
                "(-x+1/4,-x+3/4,x+1/2)",
                "(-x+3/4,x+1/2,-x+1/4)",
                "(x+1/2,-x+1/4,-x+3/4)",
                "(x+3/4,x+1/4,-x)",
                "(-x+1/2,-x+1/2,-x+1/2)",
                "(x+1/4,-x,x+3/4)",
                "(-x,x+3/4,x+1/4)",
                "(-x,-x,-x)",
                "(x+3/4,x+1/4,-x+1/2)",
                "(x+1/4,-x+1/2,x+3/4)",
                "(-x+1/2,x+3/4,x+1/4)",
                "(-x+1/4,-x+3/4,x)",
                "(x+1/2,x+1/2,x+1/2)",
                "(-x+3/4,x,-x+1/4)",
                "(x,-x+1/4,-x+3/4)"
            ],
            [
                "(7/8,1/8,1/8)",
                "(3/8,5/8,5/8)",
                "(1/8,7/8,1/8)",
                "(5/8,3/8,5/8)",
                "(1/8,1/8,7/8)",
                "(5/8,5/8,3/8)",
                "(7/8,1/8,7/8)",
                "(3/8,5/8,3/8)",
                "(5/8,3/8,7/8)",
                "(1/8,7/8,3/8)",
                "(7/8,3/8,1/8)",
                "(3/8,7/8,5/8)"
            ],
            [
                "(0,0,0)",
                "(1/4,3/4,1/2)",
                "(3/4,1/2,1/4)",
                "(1/2,1/4,3/4)",
                "(3/4,1/4,0)",
                "(1/2,1/2,1/2)",
                "(1/4,0,3/4)",
                "(0,3/4,1/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(0,1/2,3/4)",
                "(1/2,3/4,0)",
                "(3/4,0,1/2)",
                "(3/4,3/4,3/4)",
                "(0,1/2,1/4)",
                "(1/2,1/4,0)",
                "(1/4,0,1/2)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(7/8,3/8,7/8)",
                "(7/8,7/8,7/8)",
                "(1/8,5/8,1/8)"
            ]
        ],
        "Multiplicity": [
            "192",
            "96",
            "96",
            "64",
            "48",
            "32",
            "32",
            "16"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            ".3.",
            "-4..",
            ".-3.",
            "0.32",
            "23"
        ],
        "Space Group": "F 4sub1/d -3 2/c",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "529": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x,-y,z)",
                "(-x,y,-z)",
                "(x,-y,-z)",
                "(z,x,y)",
                "(z,-x,-y)",
                "(-z,-x,y)",
                "(-z,x,-y)",
                "(y,z,x)",
                "(-y,z,-x)",
                "(y,-z,-x)",
                "(-y,-z,x)",
                "(y,x,-z)",
                "(-y,-x,-z)",
                "(y,-x,z)",
                "(-y,x,z)",
                "(x,z,-y)",
                "(-x,z,y)",
                "(-x,-z,-y)",
                "(x,-z,y)",
                "(z,y,-x)",
                "(z,-y,x)",
                "(-z,y,x)",
                "(-z,-y,-x)",
                "(-x,-y,-z)",
                "(x,y,-z)",
                "(x,-y,z)",
                "(-x,y,z)",
                "(-z,-x,-y)",
                "(-z,x,y)",
                "(z,x,-y)",
                "(z,-x,y)",
                "(-y,-z,-x)",
                "(y,-z,x)",
                "(-y,z,x)",
                "(y,z,-x)",
                "(-y,-x,z)",
                "(y,x,z)",
                "(-y,x,-z)",
                "(y,-x,-z)",
                "(-x,-z,y)",
                "(x,-z,-y)",
                "(x,z,y)",
                "(-x,z,-y)",
                "(-z,-y,x)",
                "(-z,y,-x)",
                "(z,-y,-x)",
                "(z,y,x)"
            ],
            [
                "(x,x,z)",
                "(-x,-x,z)",
                "(-x,x,-z)",
                "(x,-x,-z)",
                "(z,x,x)",
                "(z,-x,-x)",
                "(-z,-x,x)",
                "(-z,x,-x)",
                "(x,z,x)",
                "(-x,z,-x)",
                "(x,-z,-x)",
                "(-x,-z,x)",
                "(x,x,-z)",
                "(-x,-x,-z)",
                "(x,-x,z)",
                "(-x,x,z)",
                "(x,z,-x)",
                "(-x,z,x)",
                "(-x,-z,-x)",
                "(x,-z,x)",
                "(z,x,-x)",
                "(z,-x,x)",
                "(-z,x,x)",
                "(-z,-x,-x)"
            ],
            [
                "(0,y,z)",
                "(0,-y,z)",
                "(0,y,-z)",
                "(0,-y,-z)",
                "(z,0,y)",
                "(z,0,-y)",
                "(-z,0,y)",
                "(-z,0,-y)",
                "(y,z,0)",
                "(-y,z,0)",
                "(y,-z,0)",
                "(-y,-z,0)",
                "(y,0,-z)",
                "(-y,0,-z)",
                "(y,0,z)",
                "(-y,0,z)",
                "(0,z,-y)",
                "(0,z,y)",
                "(0,-z,-y)",
                "(0,-z,y)",
                "(z,y,0)",
                "(z,-y,0)",
                "(-z,y,0)",
                "(-z,-y,0)"
            ],
            [
                "(1/4,y,-y+1/2)",
                "(3/4,-y,-y+1/2)",
                "(3/4,y,y+1/2)",
                "(1/4,-y,y+1/2)",
                "(-y+1/2,1/4,y)",
                "(-y+1/2,3/4,-y)",
                "(y+1/2,3/4,y)",
                "(y+1/2,1/4,-y)",
                "(y,-y+1/2,1/4)",
                "(-y,-y+1/2,3/4)",
                "(y,y+1/2,3/4)",
                "(-y,y+1/2,1/4)",
                "(3/4,-y,y+1/2)",
                "(1/4,y,y+1/2)",
                "(1/4,-y,-y+1/2)",
                "(3/4,y,-y+1/2)",
                "(y+1/2,3/4,-y)",
                "(y+1/2,1/4,y)",
                "(-y+1/2,1/4,-y)",
                "(-y+1/2,3/4,y)",
                "(-y,y+1/2,3/4)",
                "(y,y+1/2,1/4)",
                "(-y,-y+1/2,1/4)",
                "(y,-y+1/2,3/4)"
            ],
            [
                "(0,y,y)",
                "(0,-y,y)",
                "(0,y,-y)",
                "(0,-y,-y)",
                "(y,0,y)",
                "(y,0,-y)",
                "(-y,0,y)",
                "(-y,0,-y)",
                "(y,y,0)",
                "(-y,y,0)",
                "(y,-y,0)",
                "(-y,-y,0)"
            ],
            [
                "(x,0,1/2)",
                "(-x,0,1/2)",
                "(1/2,x,0)",
                "(1/2,-x,0)",
                "(0,1/2,x)",
                "(0,1/2,-x)",
                "(0,x,1/2)",
                "(0,-x,1/2)",
                "(x,1/2,0)",
                "(-x,1/2,0)",
                "(1/2,0,-x)",
                "(1/2,0,x)"
            ],
            [
                "(x,x,x)",
                "(-x,-x,x)",
                "(-x,x,-x)",
                "(x,-x,-x)",
                "(x,x,-x)",
                "(-x,-x,-x)",
                "(x,-x,x)",
                "(-x,x,x)"
            ],
            [
                "(x,0,0)",
                "(-x,0,0)",
                "(0,x,0)",
                "(0,-x,0)",
                "(0,0,x)",
                "(0,0,-x)"
            ],
            [
                "(1/4,0,1/2)",
                "(3/4,0,1/2)",
                "(1/2,1/4,0)",
                "(1/2,3/4,0)",
                "(0,1/2,1/4)",
                "(0,1/2,3/4)"
            ],
            [
                "(1/4,1/4,1/4)",
                "(3/4,3/4,1/4)",
                "(3/4,1/4,3/4)",
                "(1/4,3/4,3/4)"
            ],
            [
                "(0,1/2,1/2)",
                "(1/2,0,1/2)",
                "(1/2,1/2,0)"
            ],
            [
                "(0,0,0)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "48",
            "48",
            "24",
            "24",
            "16",
            "12",
            "12",
            "8",
            "6",
            "2"
        ],
        "Site Symmetry": [
            "1",
            "..m",
            "m..",
            "..2",
            "m.m2",
            "mm2..",
            ".3m",
            "4m.m",
            "-4m.2",
            ".-3m",
            "4/mm.m",
            "m-3m"
        ],
        "Space Group": "I 4/m -3 2/m",
        "Wyckoff Letter": [
            "l",
            "k",
            "j",
            "i",
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    },
    "530": {
        "Coordinates": [
            [
                "(x,y,z)",
                "(-x+1/2,-y,z+1/2)",
                "(-x,y+1/2,-z+1/2)",
                "(x+1/2,-y+1/2,-z)",
                "(z,x,y)",
                "(z+1/2,-x+1/2,-y)",
                "(-z+1/2,-x,y+1/2)",
                "(-z,x+1/2,-y+1/2)",
                "(y,z,x)",
                "(-y,z+1/2,-x+1/2)",
                "(y+1/2,-z+1/2,-x)",
                "(-y+1/2,-z,x+1/2)",
                "(y+3/4,x+1/4,-z+1/4)",
                "(-y+3/4,-x+3/4,-z+3/4)",
                "(y+1/4,-x+1/4,z+3/4)",
                "(-y+1/4,x+3/4,z+1/4)",
                "(x+3/4,z+1/4,-y+1/4)",
                "(-x+1/4,z+3/4,y+1/4)",
                "(-x+3/4,-z+3/4,-y+3/4)",
                "(x+1/4,-z+1/4,y+3/4)",
                "(z+3/4,y+1/4,-x+1/4)",
                "(z+1/4,-y+1/4,x+3/4)",
                "(-z+1/4,y+3/4,x+1/4)",
                "(-z+3/4,-y+3/4,-x+3/4)",
                "(-x,-y,-z)",
                "(x+1/2,y,-z+1/2)",
                "(x,-y+1/2,z+1/2)",
                "(-x+1/2,y+1/2,z)",
                "(-z,-x,-y)",
                "(-z+1/2,x+1/2,y)",
                "(z+1/2,x,-y+1/2)",
                "(z,-x+1/2,y+1/2)",
                "(-y,-z,-x)",
                "(y,-z+1/2,x+1/2)",
                "(-y+1/2,z+1/2,x)",
                "(y+1/2,z,-x+1/2)",
                "(-y+1/4,-x+3/4,z+3/4)",
                "(y+1/4,x+1/4,z+1/4)",
                "(-y+3/4,x+3/4,-z+1/4)",
                "(y+3/4,-x+1/4,-z+3/4)",
                "(-x+1/4,-z+3/4,y+3/4)",
                "(x+3/4,-z+1/4,-y+3/4)",
                "(x+1/4,z+1/4,y+1/4)",
                "(-x+3/4,z+3/4,-y+1/4)",
                "(-z+1/4,-y+3/4,x+3/4)",
                "(-z+3/4,y+3/4,-x+1/4)",
                "(z+3/4,-y+1/4,-x+3/4)",
                "(z+1/4,y+1/4,x+1/4)"
            ],
            [
                "(1/8,y,-y+1/4)",
                "(3/8,-y,-y+3/4)",
                "(7/8,y+1/2,y+1/4)",
                "(5/8,-y+1/2,y+3/4)",
                "(-y+1/4,1/8,y)",
                "(-y+3/4,3/8,-y)",
                "(y+1/4,7/8,y+1/2)",
                "(y+3/4,5/8,-y+1/2)",
                "(y,-y+1/4,1/8)",
                "(-y,-y+3/4,3/8)",
                "(y+1/2,y+1/4,7/8)",
                "(-y+1/2,y+3/4,5/8)",
                "(7/8,-y,y+3/4)",
                "(5/8,y,y+1/4)",
                "(1/8,-y+1/2,-y+3/4)",
                "(3/8,y+1/2,-y+1/4)",
                "(y+3/4,7/8,-y)",
                "(y+1/4,5/8,y)",
                "(-y+3/4,1/8,-y+1/2)",
                "(-y+1/4,3/8,y+1/2)",
                "(-y,y+3/4,7/8)",
                "(y,y+1/4,5/8)",
                "(-y+1/2,-y+3/4,1/8)",
                "(y+1/2,-y+1/4,3/8)"
            ],
            [
                "(x,0,1/4)",
                "(-x+1/2,0,3/4)",
                "(1/4,x,0)",
                "(3/4,-x+1/2,0)",
                "(0,1/4,x)",
                "(0,3/4,-x+1/2)",
                "(3/4,x+1/4,0)",
                "(3/4,-x+3/4,1/2)",
                "(x+3/4,1/2,1/4)",
                "(-x+1/4,0,1/4)",
                "(0,1/4,-x+1/4)",
                "(1/2,1/4,x+3/4)",
                "(-x,0,3/4)",
                "(x+1/2,0,1/4)",
                "(3/4,-x,0)",
                "(1/4,x+1/2,0)",
                "(0,3/4,-x)",
                "(0,1/4,x+1/2)",
                "(1/4,-x+3/4,0)",
                "(1/4,x+1/4,1/2)",
                "(-x+1/4,1/2,3/4)",
                "(x+3/4,0,3/4)",
                "(0,3/4,x+3/4)",
                "(1/2,3/4,-x+1/4)"
            ],
            [
                "(x,x,x)",
                "(-x+1/2,-x,x+1/2)",
                "(-x,x+1/2,-x+1/2)",
                "(x+1/2,-x+1/2,-x)",
                "(x+3/4,x+1/4,-x+1/4)",
                "(-x+3/4,-x+3/4,-x+3/4)",
                "(x+1/4,-x+1/4,x+3/4)",
                "(-x+1/4,x+3/4,x+1/4)",
                "(-x,-x,-x)",
                "(x+1/2,x,-x+1/2)",
                "(x,-x+1/2,x+1/2)",
                "(-x+1/2,x+1/2,x)",
                "(-x+1/4,-x+3/4,x+3/4)",
                "(x+1/4,x+1/4,x+1/4)",
                "(-x+3/4,x+3/4,-x+1/4)",
                "(x+3/4,-x+1/4,-x+3/4)"
            ],
            [
                "(3/8,0,1/4)",
                "(1/8,0,3/4)",
                "(1/4,3/8,0)",
                "(3/4,1/8,0)",
                "(0,1/4,3/8)",
                "(0,3/4,1/8)",
                "(3/4,5/8,0)",
                "(3/4,3/8,1/2)",
                "(1/8,1/2,1/4)",
                "(7/8,0,1/4)",
                "(0,1/4,7/8)",
                "(1/2,1/4,1/8)"
            ],
            [
                "(1/8,0,1/4)",
                "(3/8,0,3/4)",
                "(1/4,1/8,0)",
                "(3/4,3/8,0)",
                "(0,1/4,1/8)",
                "(0,3/4,3/8)",
                "(7/8,0,3/4)",
                "(5/8,0,1/4)",
                "(3/4,7/8,0)",
                "(1/4,5/8,0)",
                "(0,3/4,7/8)",
                "(0,1/4,5/8)"
            ],
            [
                "(1/8,1/8,1/8)",
                "(3/8,7/8,5/8)",
                "(7/8,5/8,3/8)",
                "(5/8,3/8,7/8)",
                "(7/8,7/8,7/8)",
                "(5/8,1/8,3/8)",
                "(1/8,3/8,5/8)",
                "(3/8,5/8,1/8)"
            ],
            [
                "(0,0,0)",
                "(1/2,0,1/2)",
                "(0,1/2,1/2)",
                "(1/2,1/2,0)",
                "(3/4,1/4,1/4)",
                "(3/4,3/4,3/4)",
                "(1/4,1/4,3/4)",
                "(1/4,3/4,1/4)"
            ]
        ],
        "Multiplicity": [
            "96",
            "48",
            "48",
            "32",
            "24",
            "24",
            "16",
            "16"
        ],
        "Site Symmetry": [
            "1",
            "..2",
            "2..",
            ".3.",
            "-4..",
            "2.22",
            "0.32",
            ".-3."
        ],
        "Space Group": "I 4sub1/a -3 2/d",
        "Wyckoff Letter": [
            "h",
            "g",
            "f",
            "e",
            "d",
            "c",
            "b",
            "a"
        ]
    }
}